;+
; NAME:
;       FFT_BOX
;
; PURPOSE:
;       This procedure makes FFT calculation of FIS/TSD,
;       and display the result (power spectrum) with zplot_alt
;       Interval of input data are fixed
;       at T (1/25.28(SW) 1/16.86(LW) sec).
;       This procedure is called by FISv.
;
; AUTHOR:
;       ASTRO-F Data Reduction team
;       Web Page: http://www.ir.isas.ac.jp/ASTRO-F/iris_data/
;
; CATEGORY:
;       FIS Data Visualizer
;
; CALLING SEQUENCE:
;       fft_box, data, DNAME=data_name, GROUP_LEADER=tlb
;         * data is input array of the FFT function
;           DNAME is the name of the input data
;
; OTHER PROGRAMS NEEDED:
;
;   The following programs are required to run this one:
;
;    zplot_alt.pro
;    gb_diff2p.pro(in /reduction/gb/lib)
;
; MODIFICATION HISTORY:
;        Written by: Makiuti,S.
;        03/11/28 changed to use "gb_diff2p.pro" instead of "gb_diff3pl.pro"
;-
;-----------------------------------------------------------------
pro DO_FFT, Event
COMMON FFT_Share, fft_data, ytitle, fisv_top_id
COMMON Share

  data = fft_data

  ; Check diff button. If checked then differentiate
  DiffBtn = Widget_info(Event.top, find_by_uname='WID_BUTTON_0')
  diffchk = Widget_info(DiffBtn, /Button_Set)
  IF diffchk EQ 1 THEN BEGIN
    ;result = gb_diff2p(data, diffed_data)
    sval = shift(data, 1)
    diffed_data = data - sval
    data = diffed_data
  ENDIF

  ; Check log buttons.
  DiffBtn = Widget_info(Event.top, find_by_uname='WID_BUTTON_1')
  xlog = Widget_info(DiffBtn, /Button_Set)
  DiffBtn = Widget_info(Event.top, find_by_uname='WID_BUTTON_2')
  ylog = Widget_info(DiffBtn, /Button_Set)

  ; N is the number of the data points
  N=n_elements(data)

  ; T is the interval (sec) of the data points
  IF (STRMATCH(tsd->Get_Datatype(), '*SW*', /FOLD_CASE) EQ 1) THEN T=1.0/25.28 ELSE T=1.0/16.86

  print, 'N=', N, ', T=', T,' (',1/T,'Hz)', (STRMATCH(tsd->Get_Datatype(), '*SW*', /FOLD_CASE) EQ 1) ? ' (SW)':' (LW)'


  ; FFT calculation
  y=ABS(FFT((data[where(finite(data))])))
  x=findgen(N)/(N*T)

  zplot_alt, x,y, xrange=[0,max(x)/2], xstyle=1, $
    xtitle='Frequency (Hz)', ytitle='Power of '+ytitle, title='Power Spectrum', $
    XLOG=xlog, YLOG=ylog, GROUP_LEADER=fisv_top_id

  WIDGET_CONTROL, Event.top, /DESTROY
end
;-----------------------------------------------------------------
pro Cancel_FFT, Event
  WIDGET_CONTROL, Event.top, /DESTROY
end
;
; Empty stub procedure used for autoloading.
pro fft_box_eventcb
end
;-----------------------------------------------------------------
pro WID_BASE_FFT_event, Event

  wTarget = (widget_info(Event.id,/NAME) eq 'TREE' ?  $
      widget_info(Event.id, /tree_root) : event.id)


  wWidget =  Event.top

  case wTarget of

    Widget_Info(wWidget, FIND_BY_UNAME='WID_BUTTON_OK'): begin
      if( Tag_Names(Event, /STRUCTURE_NAME) eq 'WIDGET_BUTTON' )then $
        DO_FFT, Event
    end
    Widget_Info(wWidget, FIND_BY_UNAME='WID_BUTTON_Cancel'): begin
      if( Tag_Names(Event, /STRUCTURE_NAME) eq 'WIDGET_BUTTON' )then $
        Cancel_FFT, Event
    end
    else:
  endcase

end

pro WID_BASE_FFT, GROUP_LEADER=wGroup, _EXTRA=_VWBExtra_
COMMON FFT_Share, fft_data, ytitle

  Resolve_Routine, 'fft_box_eventcb',/COMPILE_FULL_FILE  ; Load event callback routines

  WID_BASE_FFT = Widget_Base( GROUP_LEADER=wGroup,  $
      UNAME='WID_BASE_FFT' ,XOFFSET=5 ,YOFFSET=5 ,/ALIGN_CENTER  $
      ,/BASE_ALIGN_CENTER ,TITLE='FISv/FFT' ,SPACE=3 ,XPAD=3 ,YPAD=3  $
      ,COLUMN=1)

  WID_LABEL_0 = Widget_Label(WID_BASE_FFT, UNAME='WID_LABEL_0'  $
      ,XOFFSET=3 ,YOFFSET=3 ,/ALIGN_CENTER ,VALUE='FISv / FFT Calculation')


  ; check boxes
  WID_BASE_1 = Widget_Base(WID_BASE_FFT, UNAME='WID_BASE_1'  $
      ,XOFFSET=30 ,YOFFSET=20 ,TITLE='IDL' ,COLUMN=1 ,/NONEXCLUSIVE)

  WID_BUTTON_0 = Widget_Button(WID_BASE_1, UNAME='WID_BUTTON_0'  $
      ,/ALIGN_LEFT ,VALUE='Before FFT, differentiate "'+ytitle+'" data')

  WID_BUTTON_1 = Widget_Button(WID_BASE_1, UNAME='WID_BUTTON_1'  $
      ,/ALIGN_LEFT ,VALUE='XLOG', TOOLTIP='Set x-logarithmic scale')

  WID_BUTTON_2 = Widget_Button(WID_BASE_1, UNAME='WID_BUTTON_2'  $
      ,/ALIGN_LEFT ,VALUE='YLOG', TOOLTIP='Set y-logarithmic scale')

  WIDGET_CONTROL, WID_BUTTON_1, /SET_BUTTON
  WIDGET_CONTROL, WID_BUTTON_2, /SET_BUTTON

  ; Exec button
  WID_BASE_2 = Widget_Base(WID_BASE_FFT, UNAME='WID_BASE_2'  $
      ,XOFFSET=3 ,YOFFSET=44 ,TITLE='IDL' ,SPACE=3 ,XPAD=5 ,YPAD=5  $
      ,ROW=1)

  WID_BUTTON_OK = Widget_Button(WID_BASE_2, UNAME='WID_BUTTON_OK'  $
      ,XOFFSET=5 ,YOFFSET=5 ,SCR_XSIZE=80 ,SCR_YSIZE=25  $
      ,/ALIGN_CENTER ,VALUE='FFT!', TOOLTIP='FFT Calculation')

  WID_BUTTON_Cancel = Widget_Button(WID_BASE_2,  $
      UNAME='WID_BUTTON_Cancel' ,XOFFSET=88 ,YOFFSET=5 ,SCR_XSIZE=80  $
      ,SCR_YSIZE=25 ,/ALIGN_CENTER ,VALUE='Cancel', TOOLTIP='Close this window')


  Widget_Control, /REALIZE, WID_BASE_FFT

  XManager, 'WID_BASE_FFT', WID_BASE_FFT, /NO_BLOCK

end
;
; Empty stub procedure used for autoloading.
;
pro fft_box, data, DNAME=data_name, GROUP_LEADER=wGroup, _EXTRA=_VWBExtra_
COMMON FFT_Share, fft_data, ytitle, fisv_top_id

  IF n_elements(data) EQ 0 THEN BEGIN
    x=indgen(100)*0.01
    data = SIN(x*15*360*!PI/180)+SIN(x*20*360*!PI/180)+SIN(x*30*360*!PI/180)
  ENDIF
  fft_data = data

  IF n_elements(data_name) NE 0 THEN ytitle = data_name $
    ELSE ytitle='DET'

  IF n_elements(wGroup) EQ 0 THEN wGroup=0
  fisv_top_id = wGroup

  WID_BASE_FFT, GROUP_LEADER=wGroup, _EXTRA=_VWBExtra_
end
