;+
; NAME:
;       ZPLOT_FLAG
;
; PURPOSE:
;       The purpose of this program is to display a plot of time series data
;       including flag(0/1).
;
; AUTHOR:
;       ASTRO-F Data Reduction team
;       Web Page: http://www.ir.isas.ac.jp/ASTRO-F/iris_data/
;
; CATEGORY:
;       FIS Data Visualizer
;       Widgets program
;
; CALLING SEQUENCE:
;       This program is usually called from zplot_alt.pro
;
; OTHER PROGRAMS NEEDED:
;
;   The following programs are required to run this one:
;
;    getcolor.pro(in ./lib)
;    stat_plot.pro
;
; MODIFY
;       Alternate version of ZPLOT by Makiuti,S.
;       *****
;       #ZPLOT_ALT
;       03/03/27 added argument of psym to OPlot to be able to display symbol plot
;                added buttons (set symbol & close window)
;       03/04/17 max five dependent data can be passed
;                ex.) ZPlot, x, y, Y2DATA=y2, ..., Y5DATA=y5, $
;                       Y1TITLE=y1title, ..., Y5TITLE=y5title
;       03/04/22 double Y plot
;       03/04/24 zoom Y-axis
;       03/05/08 made a little change to zoom function
;       03/05/12 added "Update the plot system variables" in "Double Y Plot"
;                added "STAT" button to call stat_plot.pro
;       03/05/21 added "Print" and "PS" buttons, and changed how to make symbol
;       03/05/27 added "Reset"(range), "x 1/2"(zoom out), "<<",">>"(Scroll X) buttons
;                changed behavior for click (without drag) on plot window
;                added grid display function
;       03/05/28 change how to show legends
;       03/05/30 added differentiation button
;       03/10/07 added auto scale button
;       03/10/23 added FFT button
;       03/11/28 changed to use "gb_diff2p.pro" instead of "gb_diff3pl.pro"
;       03/11/30 changed method for calculation of yrange
;       04/11/01 changed not to use "gb_diff2p.pro"
;       *****
;       #ZPLOT_FLAG
;       04/02/25 "ZPLOT_FLAG" was made from ZPLOT_ALT
;       05/04/05 Extra mode was added
;
;#######
; below is the original header
;#######
; PURPOSE:
;
;       The purpose of this program is to display a line plot in a resizeable
;       graphics window which can be zoomed by drawing an "arrow box" on top
;       of it. To return to the un-zoomed plot, click and release anywhere in the window.
;
; AUTHOR:
;
;   FANNING SOFTWARE CONSULTING
;   David Fanning, Ph.D.
;   1645 Sheely Drive
;   Fort Collins, CO 80526 USA
;   Phone: 970-221-0438
;   E-mail: davidf@dfanning.com
;   Coyote's Guide to IDL Programming: http://www.dfanning.com/
;
; CATEGORY:
;
;      Widgets.
;
; CALLING SEQUENCE:
;
;      ZPlot, x, y
;
; OPTIONAL INPUTS:
;
;      x: If only one positional parameter, this is assumed to be the
;         dependent data. If there are two positional parameters, this
;         is assumed to be the independent data in accordance with the
;         PLOT command.
;
;      y: The dependent data, if the X parameter is present.
;
; KEYWORD PARAMETERS:
;
;       Any valid PLOT keyword can be used with this program. In additon,
;       the following keywords are defined specifically.
;
;       GROUP_LEADER: This keyword is used to assign a group leader to this
;                 program. This program will be destroyed when the group
;                 leader is destroyed. Use this keyword if you are calling
;                 ZIMAGE from another widget program.
;
;       XLOG: Set this keyword if you wish the X axis to be logarthmic style.
;
;       YLOG: Set this keyword if you wish the Y axis to be logarthmic style.
;
;       ZOOM_XSIZE: The initial X size of the plot window. Default is 400 pixels.
;
;       ZOOM_YSIZE: The initial Y size of the plot window. Default is 400 pixels.
;
; COMMON BLOCKS:
;
;       None.
;
; SIDE EFFECTS:
;
;       Drawing colors are loaded into the color table.
;
; RESTRICTIONS:
;
;       Zooms only along the X axis.
;
; PROCEDURE:
;
;       Click and drag the cursor to create an "arrow box". The plot
;       is zoomed into the X coordinates of the box, when released.
;       To restore unzoomed plot, click and release anywhere in the
;       window.
;
; EXAMPLE:
;
;        To display an plot you can zoom into, type:
;
;        ZPLOT
;
; MODIFICATION HISTORY:
;
;        Written by: David Fanning, 15 February 2000.
;        Modified the original rubberband box to be an "arrow box". 1 April 2000. DWF.
;        Added the ability to produce log style plots. 20 November 2001. DWF.
;-
;###########################################################################
;
; LICENSE
;
; This software is OSI Certified Open Source Software.
; OSI Certified is a certification mark of the Open Source Initiative.
;
; Copyright  2000-2001 Fanning Software Consulting
;
; This software is provided "as-is", without any express or
; implied warranty. In no event will the authors be held liable
; for any damages arising from the use of this software.
;
; Permission is granted to anyone to use this software for any
; purpose, including commercial applications, and to alter it and
; redistribute it freely, subject to the following restrictions:
;
; 1. The origin of this software must not be misrepresented; you must
;    not claim you wrote the original software. If you use this software
;    in a product, an acknowledgment in the product documentation
;    would be appreciated, but is not required.
;
; 2. Altered source versions must be plainly marked as such, and must
;    not be misrepresented as being the original software.
;
; 3. This notice may not be removed or altered from any source distribution.
;
; For more information on Open Source Software, visit the Open Source
; web site: http://www.opensource.org.
;
;###########################################################################



PRO ZfPLOT_CLEANUP, tlb

   ; The purpose of this module is to delete the pixmap window
   ; and perform other cleanup chores when the program ZPLOT is
   ; destroyed.

Widget_Control, tlb, Get_UValue=info
IF N_Elements(info) NE 0 THEN BEGIN
   WDelete, info.pixIndex
   Ptr_Free, info.extraKeywords
ENDIF
END ; of ZfPLOT_CLEANUP **********************************************************

;-----------------------------------------------------------------
pro zfClsBtn, Event
  WIDGET_CONTROL, Event.top, /DESTROY
end

;-----------------------------------------------------------------
pro get_prop_plot, info, PSYMS=psyms
; This procedure returns an array 'psyms' which contains psym settings

psyms=make_array(5, /integer) ; for maximum five data

IF (info.psym NE 0) THEN BEGIN
  IF (info.line EQ 0) THEN BEGIN ; with symbol witout line
    psyms[0]=7
    psyms[1]=1
    psyms[2]=4
    psyms[3]=2
    psyms[4]=6
  ENDIF ELSE BEGIN ; with both symbol and line
    psyms[0]=-7
    psyms[1]=-1
    psyms[2]=-4
    psyms[3]=-2
    psyms[4]=-6
  ENDELSE
ENDIF ELSE BEGIN ; line witout symbol
  psyms[*]=0
ENDELSE

end

;-----------------------------------------------------------------
pro zfNewPlot, Event, info

  ; ## DISPLAY START##
  ; Destroy old pixmap and create a new one.
  WDelete, info.pixIndex
  Window, XSize=info.zxsize, YSize=info.zysize, /Free, /Pixmap
  info.pixIndex = !D. Window

  ;  Set up colors for line plot.
  backColor = GetColor('charcoal', !D.Table_Size-2)
  dataColor = GetColor('yellow', !D.Table_Size-3)
  axisColor = GetColor('green', !D.Table_Size-4)
;#
  dataColor2 = GetColor('pink', !D.Table_Size-5)
  dataColor3 = GetColor('aqua', !D.Table_Size-6)
  dataColor4 = GetColor('magenta', !D.Table_Size-7)
  dataColor5 = GetColor('gray', !D.Table_Size-8)
;#
  faxisColor = GetColor('aquamarine', !D.Table_Size-9)

  ;  Set up properties (line and symbol) of line plot.
  get_prop_plot, info, PSYMS=psyms

  ; Plot position
  pos1 = [0.13,0.4,0.965,0.9] ; for normal data
  pos2 = [0.13,0.1,0.965,0.4] ; for flag data

; ## Draw the plot in both windows. ##

;# Back side
  Plot, info.indep, info.dep, $
     XRange=info.xrange, XStyle=info.xstyle, Xticklen=info.ticklen, Xgridstyle=(info.ticklen NE 0.02) ? 1:0, $
     YRange=info.yrange, YStyle=info.ystyle, Yticklen=info.ticklen, Ygridstyle=(info.ticklen NE 0.02) ? 1:0, $
     XLog=info.xlog, YLog=info.ylog, $
     Color=axisColor, /NoData, _Extra=*info.extraKeywords, $
     position = pos1
  OPlot, info.indep, info.dep, Color=dataColor, psym=psyms[0]

  ; Standard position of legends
  title_pos1a = CONVERT_COORD([0.85, 0.90], /NORMAL, /TO_DATA)  ; for symbol
  title_pos1b = CONVERT_COORD([0.86, 0.90], /NORMAL, /TO_DEVICE); for label string
  title_pos1b = CONVERT_COORD(title_pos1b[0],title_pos1b[1]-5, /DEVICE, /TO_DATA)

  title_pos2a = CONVERT_COORD([0.85, 0.85], /NORMAL, /TO_DATA)  ; for symbol
  title_pos2b = CONVERT_COORD([0.86, 0.85], /NORMAL, /TO_DEVICE); for label string
  title_pos2b = CONVERT_COORD(title_pos2b[0],title_pos2b[1]-5, /DEVICE, /TO_DATA)

  title_pos3a = CONVERT_COORD([0.85, 0.80], /NORMAL, /TO_DATA)  ; for symbol
  title_pos3b = CONVERT_COORD([0.86, 0.80], /NORMAL, /TO_DEVICE); for label string
  title_pos3b = CONVERT_COORD(title_pos3b[0],title_pos3b[1]-5, /DEVICE, /TO_DATA)

  title_pos4a = CONVERT_COORD([0.85, 0.75], /NORMAL, /TO_DATA)  ; for symbol
  title_pos4b = CONVERT_COORD([0.86, 0.75], /NORMAL, /TO_DEVICE); for label string
  title_pos4b = CONVERT_COORD(title_pos4b[0],title_pos4b[1]-5, /DEVICE, /TO_DATA)

  title_pos5a = CONVERT_COORD([0.85, 0.70], /NORMAL, /TO_DATA)  ; for symbol
  title_pos5b = CONVERT_COORD([0.86, 0.70], /NORMAL, /TO_DEVICE); for label string
  title_pos5b = CONVERT_COORD(title_pos5b[0],title_pos5b[1]-5, /DEVICE, /TO_DATA)
;#
  IF n_elements(info.dep2) EQ n_elements(info.dep) THEN BEGIN
      OPlot, info.indep, info.dep2, Color=dataColor2, psym = psyms[1]
      XYOUTS, title_pos1b[0], title_pos1b[1],  info.y1title, Color=dataColor
      Oplot, [title_pos1a[0]], [title_pos1a[1]], Color=dataColor, psym = abs(psyms[0])
      XYOUTS, title_pos2b[0], title_pos2b[1], info.y2title, Color=dataColor2
      Oplot, [title_pos2a[0]], [title_pos2a[1]], Color=dataColor2, psym = abs(psyms[1])
  ENDIF
  IF n_elements(info.dep3) EQ n_elements(info.dep) THEN BEGIN
      OPlot, info.indep, info.dep3, Color=dataColor3, psym = psyms[2]
      XYOUTS, title_pos3b[0], title_pos3b[1], info.y3title, Color=dataColor3
      Oplot, [title_pos3a[0]], [title_pos3a[1]], Color=dataColor3, psym = abs(psyms[2])
  ENDIF
  IF n_elements(info.dep4) EQ n_elements(info.dep) THEN BEGIN
      OPlot, info.indep, info.dep4, Color=dataColor4, psym = psyms[3]
      XYOUTS, title_pos4b[0], title_pos4b[1], info.y4title, Color=dataColor4
      Oplot, [title_pos4a[0]], [title_pos4a[1]], Color=dataColor4, psym = abs(psyms[3])
  ENDIF
  IF n_elements(info.dep5) EQ n_elements(info.dep) THEN BEGIN
      OPlot, info.indep, info.dep5, Color=dataColor5, psym = psyms[4]
      XYOUTS, title_pos5b[0], title_pos5b[1], info.y5title, Color=dataColor5
      Oplot, [title_pos5a[0]], [title_pos5a[1]], Color=dataColor5, psym = abs(psyms[4])
  ENDIF
;
; Top side
;######################
  WSet, info.drawIndex

;############;
; Flags plot ;
;############;
; Set y-range
f_yrange_max=2
IF n_elements(info.flag2) EQ n_elements(info.indep) THEN f_yrange_max=f_yrange_max+1.1
IF n_elements(info.flag3) EQ n_elements(info.indep) THEN f_yrange_max=f_yrange_max+1.1
IF n_elements(info.flag4) EQ n_elements(info.indep) THEN f_yrange_max=f_yrange_max+1.1
IF n_elements(info.flag5) EQ n_elements(info.indep) THEN f_yrange_max=f_yrange_max+1.1
f_yrange = [-1, f_yrange_max]

; Base lavel for the next flag plot
next_flag_base = 0 ; initial level

; Plot 1st flag
IF (n_elements(info.flag1) EQ n_elements(info.indep)) THEN BEGIN
  Plot, info.indep, info.flag1, position=pos2, ytickformat="(A1)", $
     Background=backColor, Color=faxisColor, /NoData, _Extra=*info.extraKeywords, $
     XRange=info.xrange, XStyle=info.xstyle, Xticklen=info.ticklen, Xgridstyle=(info.ticklen NE 0.02) ? 1:0, $
     YRange=f_yrange,    YStyle=info.ystyle, Yticklen=info.ticklen, Ygridstyle=(info.ticklen NE 0.02) ? 1:0
  Plot, info.indep, info.flag1, position=pos2, xstyle=info.xstyle+4, ystyle=info.ystyle+4, $
     XRange=info.xrange, YRange=f_yrange, $
     thick=1, psym=psyms[0], /noerase, Color=dataColor
  next_flag_base = next_flag_base + 1.1
ENDIF

; Standard position of legends
title_posF1a = CONVERT_COORD([0.85, 0.15], /NORMAL, /TO_DATA)  ; for symbol
title_posF1b = CONVERT_COORD([0.86, 0.15], /NORMAL, /TO_DEVICE); for label string
title_posF1b = CONVERT_COORD(title_posF1b[0],title_posF1b[1]-5, /DEVICE, /TO_DATA)

title_posF2a = CONVERT_COORD([0.85, 0.19], /NORMAL, /TO_DATA)  ; for symbol
title_posF2b = CONVERT_COORD([0.86, 0.19], /NORMAL, /TO_DEVICE); for label string
title_posF2b = CONVERT_COORD(title_posF2b[0],title_posF2b[1]-5, /DEVICE, /TO_DATA)

title_posF3a = CONVERT_COORD([0.85, 0.23], /NORMAL, /TO_DATA)  ; for symbol
title_posF3b = CONVERT_COORD([0.86, 0.23], /NORMAL, /TO_DEVICE); for label string
title_posF3b = CONVERT_COORD(title_posF3b[0],title_posF3b[1]-5, /DEVICE, /TO_DATA)

title_posF4a = CONVERT_COORD([0.85, 0.27], /NORMAL, /TO_DATA)  ; for symbol
title_posF4b = CONVERT_COORD([0.86, 0.27], /NORMAL, /TO_DEVICE); for label string
title_posF4b = CONVERT_COORD(title_posF4b[0],title_posF4b[1]-5, /DEVICE, /TO_DATA)

title_posF5a = CONVERT_COORD([0.85, 0.31], /NORMAL, /TO_DATA)  ; for symbol
title_posF5b = CONVERT_COORD([0.86, 0.31], /NORMAL, /TO_DEVICE); for label string
title_posF5b = CONVERT_COORD(title_posF5b[0],title_posF5b[1]-5, /DEVICE, /TO_DATA)

; Show the title of flag1
XYOUTS, title_posF1b[0], title_posF1b[1],  info.f1title, Color=dataColor

; Plot other flags
IF (n_elements(info.flag2) EQ n_elements(info.indep)) THEN BEGIN
  Plot, info.indep, info.flag2+next_flag_base,  position=pos2, xstyle=info.xstyle+4, ystyle=info.ystyle+4, $
    XRange=info.xrange, YRange=f_yrange, $
    thick=1, psym=psyms[0], /noerase, Color=dataColor2
  next_flag_base = next_flag_base + 1.1
  XYOUTS, title_posF2b[0], title_posF2b[1],  info.f2title, Color=dataColor2
ENDIF
IF (n_elements(info.flag3) EQ n_elements(info.indep)) THEN BEGIN
  Plot, info.indep, info.flag3+next_flag_base,  position=pos2, xstyle=info.xstyle+4, ystyle=info.ystyle+4, $
    XRange=info.xrange, YRange=f_yrange, $
    thick=1, psym=psyms[0], /noerase, Color=dataColor3
  next_flag_base = next_flag_base + 1.1
  XYOUTS, title_posF3b[0], title_posF3b[1],  info.f3title, Color=dataColor3
ENDIF
IF (n_elements(info.flag4) EQ n_elements(info.indep)) THEN BEGIN
  Plot, info.indep, info.flag4+next_flag_base,  position=pos2, xstyle=info.xstyle+4, ystyle=info.ystyle+4, $
    XRange=info.xrange, YRange=f_yrange, $
    thick=1, psym=psyms[0], /noerase, Color=dataColor4
  next_flag_base = next_flag_base + 1.1
  XYOUTS, title_posF4b[0], title_posF4b[1],  info.f4title, Color=dataColor4
ENDIF
IF (n_elements(info.flag5) EQ n_elements(info.indep)) THEN BEGIN
  Plot, info.indep, info.flag5+next_flag_base,  position=pos2, xstyle=info.xstyle+4, ystyle=info.ystyle+4, $
    XRange=info.xrange, YRange=f_yrange, $
    thick=1, psym=psyms[0], /noerase, Color=dataColor5
  next_flag_base = next_flag_base + 1.1
  XYOUTS, title_posF5b[0], title_posF5b[1],  info.f5title, Color=dataColor5
ENDIF

;############;
; Data Plot  ;
;############;
  Plot, info.indep, info.dep, $
     XRange=info.xrange, XStyle=info.xstyle, Xticklen=info.ticklen, Xgridstyle=(info.ticklen NE 0.02) ? 1:0, $
     YRange=info.yrange, YStyle=info.ystyle, Yticklen=info.ticklen, Ygridstyle=(info.ticklen NE 0.02) ? 1:0, $
     XLog=info.xlog, YLog=info.ylog, $
     Background=backColor, Color=axisColor, /NoData, _Extra=*info.extraKeywords, $
     position = pos1, xtickformat="(A1)", /noerase
  OPlot, info.indep, info.dep, Color=dataColor, psym = psyms[0]
  XYOUTS, title_pos1b[0], title_pos1b[1],  info.y1title, Color=dataColor
  Oplot, [title_pos1a[0]], [title_pos1a[1]], Color=dataColor, psym = abs(psyms[0])
;#
  IF n_elements(info.dep2) EQ n_elements(info.dep) THEN BEGIN
      OPlot, info.indep, info.dep2, Color=dataColor2, psym = psyms[1]
      XYOUTS, title_pos2b[0], title_pos2b[1], info.y2title, Color=dataColor2
      Oplot, [title_pos2a[0]], [title_pos2a[1]], Color=dataColor2, psym = abs(psyms[1])
  ENDIF
  IF n_elements(info.dep3) EQ n_elements(info.dep) THEN BEGIN
      OPlot, info.indep, info.dep3, Color=dataColor3, psym = psyms[2]
      XYOUTS, title_pos3b[0], title_pos3b[1], info.y3title, Color=dataColor3
      Oplot, [title_pos3a[0]], [title_pos3a[1]], Color=dataColor3, psym = abs(psyms[2])
  ENDIF
  IF n_elements(info.dep4) EQ n_elements(info.dep) THEN BEGIN
      OPlot, info.indep, info.dep4, Color=dataColor4, psym = psyms[3]
      XYOUTS, title_pos4b[0], title_pos4b[1], info.y4title, Color=dataColor4
      Oplot, [title_pos4a[0]], [title_pos4a[1]], Color=dataColor4, psym = abs(psyms[3])
  ENDIF
  IF n_elements(info.dep5) EQ n_elements(info.dep) THEN BEGIN
      OPlot, info.indep, info.dep5, Color=dataColor5, psym = psyms[4]
      XYOUTS, title_pos5b[0], title_pos5b[1], info.y5title, Color=dataColor5
      Oplot, [title_pos5a[0]], [title_pos5a[1]], Color=dataColor5, psym = abs(psyms[4])
  ENDIF
  ; ## DISPLAY END##

  ; Update the plot system variables.
  info.x = !X
  info.y = !Y
  info.p = !P

  Widget_Control, event.top, Set_UValue=info, /No_Copy

end

;-----------------------------------------------------------------
pro zfPrintBtn, Event, info

  ; Printer OUTPUT
  result=dialog_printersetup()

  IF result EQ 1 THEN BEGIN
    entry_device = !d.name
    set_plot, 'PRINTER'

  ;  Set up properties (line and symbol) of line plot.
  get_prop_plot, info, PSYMS=psyms

    ; Plot position
    pos1 = [0.13,0.4,0.965,0.9] ; for normal data
    pos2 = [0.13,0.1,0.965,0.4] ; for flag data

      ; Draw the plot
      ;############;
      ; Flags plot ;
      ;############;
      ; Set y-range
      f_yrange_max=2
      IF n_elements(info.flag2) EQ n_elements(info.indep) THEN f_yrange_max=f_yrange_max+1.1
      IF n_elements(info.flag3) EQ n_elements(info.indep) THEN f_yrange_max=f_yrange_max+1.1
      IF n_elements(info.flag4) EQ n_elements(info.indep) THEN f_yrange_max=f_yrange_max+1.1
      IF n_elements(info.flag5) EQ n_elements(info.indep) THEN f_yrange_max=f_yrange_max+1.1
      f_yrange = [-1, f_yrange_max]

      ; Base lavel for the next flag plot
      next_flag_base = 0 ; initial level

      ; Plot 1st flag
      IF (n_elements(info.flag1) EQ n_elements(info.indep)) THEN BEGIN
        Plot, info.indep, info.flag1, position=pos2, ytickformat="(A1)", $
           Color=faxisColor, /NoData, _Extra=*info.extraKeywords, $
           XRange=info.xrange, XStyle=info.xstyle, Xticklen=info.ticklen, Xgridstyle=(info.ticklen NE 0.02) ? 1:0, $
           YRange=f_yrange,    YStyle=info.ystyle, Yticklen=info.ticklen, Ygridstyle=(info.ticklen NE 0.02) ? 1:0
        Plot, info.indep, info.flag1, position=pos2, xstyle=info.xstyle+4, ystyle=info.ystyle+4, $
           XRange=info.xrange, YRange=f_yrange, $
           thick=1, psym=psyms[0], /noerase, Color=dataColor
        next_flag_base = next_flag_base + 1.1
      ENDIF

      ; Standard position of legends
      title_posF1a = CONVERT_COORD([0.85, 0.15], /NORMAL, /TO_DATA)  ; for symbol
      title_posF1b = CONVERT_COORD([0.86, 0.15], /NORMAL, /TO_DEVICE); for label string
      title_posF1b = CONVERT_COORD(title_posF1b[0],title_posF1b[1]-5, /DEVICE, /TO_DATA)

      title_posF2a = CONVERT_COORD([0.85, 0.19], /NORMAL, /TO_DATA)  ; for symbol
      title_posF2b = CONVERT_COORD([0.86, 0.19], /NORMAL, /TO_DEVICE); for label string
      title_posF2b = CONVERT_COORD(title_posF2b[0],title_posF2b[1]-5, /DEVICE, /TO_DATA)

      title_posF3a = CONVERT_COORD([0.85, 0.23], /NORMAL, /TO_DATA)  ; for symbol
      title_posF3b = CONVERT_COORD([0.86, 0.23], /NORMAL, /TO_DEVICE); for label string
      title_posF3b = CONVERT_COORD(title_posF3b[0],title_posF3b[1]-5, /DEVICE, /TO_DATA)

      title_posF4a = CONVERT_COORD([0.85, 0.27], /NORMAL, /TO_DATA)  ; for symbol
      title_posF4b = CONVERT_COORD([0.86, 0.27], /NORMAL, /TO_DEVICE); for label string
      title_posF4b = CONVERT_COORD(title_posF4b[0],title_posF4b[1]-5, /DEVICE, /TO_DATA)

      title_posF5a = CONVERT_COORD([0.85, 0.31], /NORMAL, /TO_DATA)  ; for symbol
      title_posF5b = CONVERT_COORD([0.86, 0.31], /NORMAL, /TO_DEVICE); for label string
      title_posF5b = CONVERT_COORD(title_posF5b[0],title_posF5b[1]-5, /DEVICE, /TO_DATA)

      ; Show the title of flag1
      XYOUTS, title_posF1b[0], title_posF1b[1],  info.f1title, Color=dataColor

      ; Plot other flags
      IF (n_elements(info.flag2) EQ n_elements(info.indep)) THEN BEGIN
        Plot, info.indep, info.flag2+next_flag_base,  position=pos2, xstyle=info.xstyle+4, ystyle=info.ystyle+4, $
          XRange=info.xrange, YRange=f_yrange, $
          thick=1, psym=psyms[0], /noerase, Color=dataColor2
        next_flag_base = next_flag_base + 1.1
        XYOUTS, title_posF2b[0], title_posF2b[1],  info.f2title, Color=dataColor2
      ENDIF
      IF (n_elements(info.flag3) EQ n_elements(info.indep)) THEN BEGIN
        Plot, info.indep, info.flag3+next_flag_base,  position=pos2, xstyle=info.xstyle+4, ystyle=info.ystyle+4, $
          XRange=info.xrange, YRange=f_yrange, $
          thick=1, psym=psyms[0], /noerase, Color=dataColor3
        next_flag_base = next_flag_base + 1.1
        XYOUTS, title_posF3b[0], title_posF3b[1],  info.f3title, Color=dataColor3
      ENDIF
      IF (n_elements(info.flag4) EQ n_elements(info.indep)) THEN BEGIN
        Plot, info.indep, info.flag4+next_flag_base,  position=pos2, xstyle=info.xstyle+4, ystyle=info.ystyle+4, $
          XRange=info.xrange, YRange=f_yrange, $
          thick=1, psym=psyms[0], /noerase, Color=dataColor4
        next_flag_base = next_flag_base + 1.1
        XYOUTS, title_posF4b[0], title_posF4b[1],  info.f4title, Color=dataColor4
      ENDIF
      IF (n_elements(info.flag5) EQ n_elements(info.indep)) THEN BEGIN
        Plot, info.indep, info.flag5+next_flag_base,  position=pos2, xstyle=info.xstyle+4, ystyle=info.ystyle+4, $
          XRange=info.xrange, YRange=f_yrange, $
          thick=1, psym=psyms[0], /noerase, Color=dataColor5
        next_flag_base = next_flag_base + 1.1
        XYOUTS, title_posF5b[0], title_posF5b[1],  info.f5title, Color=dataColor5
      ENDIF

      ;############;
      ; Data Plot  ;
      ;############;
      Plot, info.indep, info.dep, $
         XRange=info.xrange, XStyle=info.xstyle, Xticklen=info.ticklen, Xgridstyle=(info.ticklen NE 0.02) ? 1:0, $
         YRange=info.yrange, YStyle=info.ystyle, Yticklen=info.ticklen, Ygridstyle=(info.ticklen NE 0.02) ? 1:0, $
         XLog=info.xlog, YLog=info.ylog, $
         Background=backColor, Color=axisColor, /NoData, _Extra=*info.extraKeywords, $
         position = pos1, xtickformat="(A1)", /noerase

      ; Standard position of legends
      title_pos1a = CONVERT_COORD([0.85, 0.90], /NORMAL, /TO_DATA)  ; for symbol
      title_pos1b = CONVERT_COORD([0.86, 0.90], /NORMAL, /TO_DEVICE); for label string
      title_pos1b = CONVERT_COORD(title_pos1b[0],title_pos1b[1]-5, /DEVICE, /TO_DATA)

      title_pos2a = CONVERT_COORD([0.85, 0.85], /NORMAL, /TO_DATA)  ; for symbol
      title_pos2b = CONVERT_COORD([0.86, 0.85], /NORMAL, /TO_DEVICE); for label string
      title_pos2b = CONVERT_COORD(title_pos2b[0],title_pos2b[1]-5, /DEVICE, /TO_DATA)

      title_pos3a = CONVERT_COORD([0.85, 0.80], /NORMAL, /TO_DATA)  ; for symbol
      title_pos3b = CONVERT_COORD([0.86, 0.80], /NORMAL, /TO_DEVICE); for label string
      title_pos3b = CONVERT_COORD(title_pos3b[0],title_pos3b[1]-5, /DEVICE, /TO_DATA)

      title_pos4a = CONVERT_COORD([0.85, 0.75], /NORMAL, /TO_DATA)  ; for symbol
      title_pos4b = CONVERT_COORD([0.86, 0.75], /NORMAL, /TO_DEVICE); for label string
      title_pos4b = CONVERT_COORD(title_pos4b[0],title_pos4b[1]-5, /DEVICE, /TO_DATA)

      title_pos5a = CONVERT_COORD([0.85, 0.70], /NORMAL, /TO_DATA)  ; for symbol
      title_pos5b = CONVERT_COORD([0.86, 0.70], /NORMAL, /TO_DEVICE); for label string
      title_pos5b = CONVERT_COORD(title_pos5b[0],title_pos5b[1]-5, /DEVICE, /TO_DATA)

      OPlot, info.indep, info.dep, Color=dataColor, psym = psyms[0]
      XYOUTS, title_pos1b[0], title_pos1b[1],  info.y1title, Color=dataColor
      Oplot, [title_pos1a[0]], [title_pos1a[1]], Color=dataColor, psym = abs(psyms[0])

      ;#
        IF n_elements(info.dep2) EQ n_elements(info.dep) THEN BEGIN
            OPlot, info.indep, info.dep2, Color=dataColor2, psym = psyms[1]
            XYOUTS, title_pos2b[0], title_pos2b[1], info.y2title, Color=dataColor2
            Oplot, [title_pos2a[0]], [title_pos2a[1]], Color=dataColor2, psym = abs(psyms[1])
        ENDIF
        IF n_elements(info.dep3) EQ n_elements(info.dep) THEN BEGIN
            OPlot, info.indep, info.dep3, Color=dataColor3, psym = psyms[2]
            XYOUTS, title_pos3b[0], title_pos3b[1], info.y3title, Color=dataColor3
            Oplot, [title_pos3a[0]], [title_pos3a[1]], Color=dataColor3, psym = abs(psyms[2])
        ENDIF
        IF n_elements(info.dep4) EQ n_elements(info.dep) THEN BEGIN
            OPlot, info.indep, info.dep4, Color=dataColor4, psym = psyms[3]
            XYOUTS, title_pos4b[0], title_pos4b[1], info.y4title, Color=dataColor4
            Oplot, [title_pos4a[0]], [title_pos4a[1]], Color=dataColor4, psym = abs(psyms[3])
        ENDIF
        IF n_elements(info.dep5) EQ n_elements(info.dep) THEN BEGIN
            OPlot, info.indep, info.dep5, Color=dataColor5, psym = psyms[4]
            XYOUTS, title_pos5b[0], title_pos5b[1], info.y5title, Color=dataColor5
            Oplot, [title_pos5a[0]], [title_pos5a[1]], Color=dataColor5, psym = abs(psyms[4])
        ENDIF
        ; ## Draw END##

    XYOUTS, 0.02, -0.02, info.filename, /NORMAL
    XYOUTS, 0.6, 1.0, systime(), /NORMAL

    device, /close_document
    set_plot, entry_device
  ENDIF
  ; End of Printer OUTPUT

  Widget_Control, event.top, Set_UValue=info, /No_Copy

end

;-----------------------------------------------------------------
pro zfPSBtn, Event, info

  ; PS file OUTPUT
  ; Using FSC_PSConfig program
  ;psObject = Obj_New("FSC_PSConfig", Color=0, /European)
  ;psObject->GUI, Cancel=cancelled, /FontInfo
  psObject = Obj_New("FSC_PSConfig", /Color, /European, /Landscape, $
                     XSize=20.0, YSize=15.0, XOffset=3.0029, YOffset=24.859)
  psObject->GUI, Cancel=cancelled, /FontInfo;, Accept_Button_Name='OK'

  IF cancelled EQ 0 THEN BEGIN
    entry_device = !d.name
    set_plot, 'PS'
    ;device, filename="testOUT.ps"
    device, _Extra=psObject->GetKeywords()

    ; For color plot (Default mode is B/W)
    tmp=psObject->GetKeywords()
    IF (tmp.color eq 1) THEN BEGIN

      ;  Set up colors for line plot.
      axisColor = GetColor('black', !D.Table_Size-4)
      dataColor = GetColor('blue', !D.Table_Size-3)
      dataColor2 = GetColor('red', !D.Table_Size-6)
      dataColor3 = GetColor('dodger blue', !D.Table_Size-7)
      dataColor4 = GetColor('magenta', !D.Table_Size-8)
      dataColor5 = GetColor('charcoal', !D.Table_Size-9)
      ;#
      faxisColor = GetColor('black', !D.Table_Size-9)

    ENDIF

    ;  Set up properties (line and symbol) of line plot.
    get_prop_plot, info, PSYMS=psyms

    ; Plot position
    pos1 = [0.13,0.4,0.965,0.9] ; for normal data
    pos2 = [0.13,0.1,0.965,0.4] ; for flag data

      ; Draw the plot
      ;############;
      ; Flags plot ;
      ;############;
      ; Set y-range
      ; Set y-range
      f_yrange_max=2
      IF n_elements(info.flag2) EQ n_elements(info.indep) THEN f_yrange_max=f_yrange_max+1.1
      IF n_elements(info.flag3) EQ n_elements(info.indep) THEN f_yrange_max=f_yrange_max+1.1
      IF n_elements(info.flag4) EQ n_elements(info.indep) THEN f_yrange_max=f_yrange_max+1.1
      IF n_elements(info.flag5) EQ n_elements(info.indep) THEN f_yrange_max=f_yrange_max+1.1
      f_yrange = [-1, f_yrange_max]

      ; Base lavel for the next flag plot
      next_flag_base = 0 ; initial level

      ; Plot 1st flag
      IF (n_elements(info.flag1) EQ n_elements(info.indep)) THEN BEGIN
        Plot, info.indep, info.flag1, position=pos2, ytickformat="(A1)", $
           Color=faxisColor, /NoData, _Extra=*info.extraKeywords, $
           XRange=info.xrange, XStyle=info.xstyle, Xticklen=info.ticklen, Xgridstyle=(info.ticklen NE 0.02) ? 1:0, $
           YRange=f_yrange,    YStyle=info.ystyle, Yticklen=info.ticklen, Ygridstyle=(info.ticklen NE 0.02) ? 1:0
        Plot, info.indep, info.flag1, position=pos2, xstyle=info.xstyle+4, ystyle=info.ystyle+4, $
           XRange=info.xrange, YRange=f_yrange, $
           thick=1, psym=psyms[0], /noerase, Color=dataColor
        next_flag_base = next_flag_base + 1.1
      ENDIF

      ; Standard position of legends
      title_posF1a = CONVERT_COORD([0.85, 0.15], /NORMAL, /TO_DATA)  ; for symbol
      title_posF1b = CONVERT_COORD([0.86, 0.15], /NORMAL, /TO_DEVICE); for label string
      title_posF1b = CONVERT_COORD(title_posF1b[0],title_posF1b[1]-5, /DEVICE, /TO_DATA)

      title_posF2a = CONVERT_COORD([0.85, 0.19], /NORMAL, /TO_DATA)  ; for symbol
      title_posF2b = CONVERT_COORD([0.86, 0.19], /NORMAL, /TO_DEVICE); for label string
      title_posF2b = CONVERT_COORD(title_posF2b[0],title_posF2b[1]-5, /DEVICE, /TO_DATA)

      title_posF3a = CONVERT_COORD([0.85, 0.23], /NORMAL, /TO_DATA)  ; for symbol
      title_posF3b = CONVERT_COORD([0.86, 0.23], /NORMAL, /TO_DEVICE); for label string
      title_posF3b = CONVERT_COORD(title_posF3b[0],title_posF3b[1]-5, /DEVICE, /TO_DATA)

      title_posF4a = CONVERT_COORD([0.85, 0.27], /NORMAL, /TO_DATA)  ; for symbol
      title_posF4b = CONVERT_COORD([0.86, 0.27], /NORMAL, /TO_DEVICE); for label string
      title_posF4b = CONVERT_COORD(title_posF4b[0],title_posF4b[1]-5, /DEVICE, /TO_DATA)

      title_posF5a = CONVERT_COORD([0.85, 0.31], /NORMAL, /TO_DATA)  ; for symbol
      title_posF5b = CONVERT_COORD([0.86, 0.31], /NORMAL, /TO_DEVICE); for label string
      title_posF5b = CONVERT_COORD(title_posF5b[0],title_posF5b[1]-5, /DEVICE, /TO_DATA)

      ; Show the title of flag1
      XYOUTS, title_posF1b[0], title_posF1b[1],  info.f1title, Color=dataColor

      ; Plot other flags
      IF (n_elements(info.flag2) EQ n_elements(info.indep)) THEN BEGIN
        Plot, info.indep, info.flag2+next_flag_base,  position=pos2, xstyle=info.xstyle+4, ystyle=info.ystyle+4, $
          XRange=info.xrange, YRange=f_yrange, $
          thick=1, psym=psyms[0], /noerase, Color=dataColor2
        next_flag_base = next_flag_base + 1.1
        XYOUTS, title_posF2b[0], title_posF2b[1],  info.f2title, Color=dataColor2
      ENDIF
      IF (n_elements(info.flag3) EQ n_elements(info.indep)) THEN BEGIN
        Plot, info.indep, info.flag3+next_flag_base,  position=pos2, xstyle=info.xstyle+4, ystyle=info.ystyle+4, $
          XRange=info.xrange, YRange=f_yrange, $
          thick=1, psym=psyms[0], /noerase, Color=dataColor3
        next_flag_base = next_flag_base + 1.1
        XYOUTS, title_posF3b[0], title_posF3b[1],  info.f3title, Color=dataColor3
      ENDIF
      IF (n_elements(info.flag4) EQ n_elements(info.indep)) THEN BEGIN
        Plot, info.indep, info.flag4+next_flag_base,  position=pos2, xstyle=info.xstyle+4, ystyle=info.ystyle+4, $
          XRange=info.xrange, YRange=f_yrange, $
          thick=1, psym=psyms[0], /noerase, Color=dataColor4
        next_flag_base = next_flag_base + 1.1
        XYOUTS, title_posF4b[0], title_posF4b[1],  info.f4title, Color=dataColor4
      ENDIF
      IF (n_elements(info.flag5) EQ n_elements(info.indep)) THEN BEGIN
        Plot, info.indep, info.flag5+next_flag_base,  position=pos2, xstyle=info.xstyle+4, ystyle=info.ystyle+4, $
          XRange=info.xrange, YRange=f_yrange, $
          thick=1, psym=psyms[0], /noerase, Color=dataColor5
        next_flag_base = next_flag_base + 1.1
        XYOUTS, title_posF5b[0], title_posF5b[1],  info.f5title, Color=dataColor5
      ENDIF

      ;############;
      ; Data Plot  ;
      ;############;
      Plot, info.indep, info.dep, $
         XRange=info.xrange, XStyle=info.xstyle, Xticklen=info.ticklen, Xgridstyle=(info.ticklen NE 0.02) ? 1:0, $
         YRange=info.yrange, YStyle=info.ystyle, Yticklen=info.ticklen, Ygridstyle=(info.ticklen NE 0.02) ? 1:0, $
         XLog=info.xlog, YLog=info.ylog, $
         Color=axisColor, /NoData, _Extra=*info.extraKeywords, $
         position = pos1, xtickformat="(A1)", /noerase

      ; Standard position of legends
      title_pos1a = CONVERT_COORD([0.85, 0.90], /NORMAL, /TO_DATA)  ; for symbol
      title_pos1b = CONVERT_COORD([0.86, 0.90], /NORMAL, /TO_DEVICE); for label string
      title_pos1b = CONVERT_COORD(title_pos1b[0],title_pos1b[1]-5, /DEVICE, /TO_DATA)

      title_pos2a = CONVERT_COORD([0.85, 0.85], /NORMAL, /TO_DATA)  ; for symbol
      title_pos2b = CONVERT_COORD([0.86, 0.85], /NORMAL, /TO_DEVICE); for label string
      title_pos2b = CONVERT_COORD(title_pos2b[0],title_pos2b[1]-5, /DEVICE, /TO_DATA)

      title_pos3a = CONVERT_COORD([0.85, 0.80], /NORMAL, /TO_DATA)  ; for symbol
      title_pos3b = CONVERT_COORD([0.86, 0.80], /NORMAL, /TO_DEVICE); for label string
      title_pos3b = CONVERT_COORD(title_pos3b[0],title_pos3b[1]-5, /DEVICE, /TO_DATA)

      title_pos4a = CONVERT_COORD([0.85, 0.75], /NORMAL, /TO_DATA)  ; for symbol
      title_pos4b = CONVERT_COORD([0.86, 0.75], /NORMAL, /TO_DEVICE); for label string
      title_pos4b = CONVERT_COORD(title_pos4b[0],title_pos4b[1]-5, /DEVICE, /TO_DATA)

      title_pos5a = CONVERT_COORD([0.85, 0.70], /NORMAL, /TO_DATA)  ; for symbol
      title_pos5b = CONVERT_COORD([0.86, 0.70], /NORMAL, /TO_DEVICE); for label string
      title_pos5b = CONVERT_COORD(title_pos5b[0],title_pos5b[1]-5, /DEVICE, /TO_DATA)

      OPlot, info.indep, info.dep, Color=dataColor, psym = psyms[0]
      XYOUTS, title_pos1b[0], title_pos1b[1],  info.y1title, Color=dataColor
      Oplot, [title_pos1a[0]], [title_pos1a[1]], Color=dataColor, psym = abs(psyms[0])

      ;#
        IF n_elements(info.dep2) EQ n_elements(info.dep) THEN BEGIN
            OPlot, info.indep, info.dep2, Color=dataColor2, psym = psyms[1]
            XYOUTS, title_pos2b[0], title_pos2b[1], info.y2title, Color=dataColor2
            Oplot, [title_pos2a[0]], [title_pos2a[1]], Color=dataColor2, psym = abs(psyms[1])
        ENDIF
        IF n_elements(info.dep3) EQ n_elements(info.dep) THEN BEGIN
            OPlot, info.indep, info.dep3, Color=dataColor3, psym = psyms[2]
            XYOUTS, title_pos3b[0], title_pos3b[1], info.y3title, Color=dataColor3
            Oplot, [title_pos3a[0]], [title_pos3a[1]], Color=dataColor3, psym = abs(psyms[2])
        ENDIF
        IF n_elements(info.dep4) EQ n_elements(info.dep) THEN BEGIN
            OPlot, info.indep, info.dep4, Color=dataColor4, psym = psyms[3]
            XYOUTS, title_pos4b[0], title_pos4b[1], info.y4title, Color=dataColor4
            Oplot, [title_pos4a[0]], [title_pos4a[1]], Color=dataColor4, psym = abs(psyms[3])
        ENDIF
        IF n_elements(info.dep5) EQ n_elements(info.dep) THEN BEGIN
            OPlot, info.indep, info.dep5, Color=dataColor5, psym = psyms[4]
            XYOUTS, title_pos5b[0], title_pos5b[1], info.y5title, Color=dataColor5
            Oplot, [title_pos5a[0]], [title_pos5a[1]], Color=dataColor5, psym = abs(psyms[4])
        ENDIF
        ; ## Draw END##


    XYOUTS, 0.02, -0.02, info.filename, /NORMAL
    XYOUTS, 0.6, 1.0, systime(), /NORMAL

    device, /close_file
    set_plot, entry_device
  ENDIF

  Obj_Destroy, psObject
  ; End of PS file OUTPUT

  Widget_Control, event.top, Set_UValue=info, /No_Copy

end

;-----------------------------------------------------------------
pro zfStatBtn, Event, info

  parent = widget_info(Event.top, find_by_uname='PlotWin')
  info2 = info

  ; If size of STAT window to be made is larger than the device screen size
  screen_size = get_screen_size()
  IF ( (info2.zxsize + 300) GT screen_size[0] ) THEN $
    info2.zxsize = screen_size[0] - 300
  IF ( (info2.zysize + 50) GT screen_size[1] ) THEN $
    info2.zysize = screen_size[1] - 50

  stat_plot, info2, GROUP_LEADER=parent, CHARSIZE = (!d.name EQ 'X') ? 1.4 : 1

  Widget_Control, event.top, Set_UValue=info, /No_Copy

end

;-----------------------------------------------------------------
pro zfFFTBtn, Event, info
COMMON TOP_WID_ID, rootID ; rootID is the FISv top level ID when zplot_alt was called by FISv.

  info2 = info

  ; If more than two data are plotted in the window, show message.
  IF n_elements(info2.dep2) GT 1 THEN BEGIN
    ; Show error message
    result = DIALOG_MESSAGE(["Only the first data is used for FFT!"],$
                DIALOG_PARENT=parent, /INFORMATION)
    print, 'Only the first data is used for FFT!'
  ENDIF

  ; The current viewing data will be sent to FFT box.
  data = info2.dep[where( (info2.indep GT info2.xrange[0]) AND (info2.indep LT info2.xrange[1]) $
                          AND (info2.dep GT info2.yrange[0]) AND (info2.dep LT info2.yrange[1]) )]

  fft_box, data, DNAME=info2.y1title, GROUP_LEADER=rootID

  Widget_Control, event.top, Set_UValue=info, /No_Copy

end

;-----------------------------------------------------------------
pro zfExtBtn, Event, info
COMMON TOP_WID_ID, rootID ; rootID is the FISv top level ID when zplot_alt was called by FISv.

  plotwin_geom = widget_info(Event.top, /GEOMETRY); Geometry information of the current window

  extbtn_mode = (info.extbtn_mode EQ 1) ? 0:1

  zplot_flag, info.indep, info.dep, Y2DATA=info.dep2, Y3DATA=info.dep3, Y4DATA=info.dep4, Y5DATA=info.dep5, $
   Y1TITLE=info.y1title, Y2TITLE=info.y2title, Y3TITLE=info.y3title, Y4TITLE=info.y4title, Y5TITLE=info.y5title, $
   FILENAME=info.filename, GROUP_LEADER=rootID, $
   FLAG1=info.flag1, FLAG2=info.flag2, FLAG3=info.flag3, FLAG4=info.flag4, FLAG5=info.flag5, $
   F1TITLE=info.f1title, F2TITLE=info.f2title, F3TITLE=info.f3title, F4TITLE=info.f4title, F5TITLE=info.f5title, $
   ;xrange=info.xrange, yrange=info.yrange, $ ;;<- this causes an error
   psym=info.psym, $
   ZOOM_XSIZE = (info.zxsize GT 600)?info.zxsize:600, ZOOM_YSIZE=info.zysize, $
   extbtn_mode=extbtn_mode, $ ;;; Extra Button Mode ON
   XOffset=plotwin_geom.xoffset, YOffset=plotwin_geom.yoffset, $
   _Extra=*info.extraKeywords

  WIDGET_CONTROL, Event.top, /DESTROY

end

;-----------------------------------------------------------------
PRO ZfPLOT_RESIZE, event

; This event handler reponds to TLB re-size events.
 Widget_Control, event.top, Get_UValue=info, /No_Copy

;#####Start of altered code#####
  wWidget = Event.top
  wTarget = Event.id
  ;help, event, /structure

  case wTarget of

    ; ##Close Plot Window##
    Widget_Info(wWidget, FIND_BY_UNAME='ClsBtn'): begin
    if( Tag_Names(Event, /STRUCTURE_NAME) eq 'WIDGET_BUTTON' )then $
      zfClsBtn, Event
    end

    ; ##Grid On/Off##
    Widget_Info(wWidget, FIND_BY_UNAME='GridBtn'): begin
      IF( Tag_Names(Event, /STRUCTURE_NAME) eq 'WIDGET_BUTTON' ) THEN BEGIN

        ; Switch ticklen
        info.ticklen = (info.ticklen EQ 0.02) ? 1.0:0.02

        ; Make new plot
        zfNewPlot, Event, info
      ENDIF
    end

    ; ##Reset range##
    Widget_Info(wWidget, FIND_BY_UNAME='ResetBtn'): begin
      IF( Tag_Names(Event, /STRUCTURE_NAME) eq 'WIDGET_BUTTON' ) THEN BEGIN

        ; Back to the initial ranges
        info.xrange = info.xrange_def
        info.yrange = info.yrange_def

        ; Set axis style
        info.xstyle = 0
        info.ystyle = 0

        ; Make new plot
        zfNewPlot, Event, info
      ENDIF
    end

    ; ##Auto Scale##
    Widget_Info(wWidget, FIND_BY_UNAME='ASBtn'): begin
      IF( Tag_Names(Event, /STRUCTURE_NAME) eq 'WIDGET_BUTTON' ) THEN BEGIN

        ; Set new range
        x0 = info.xrange[0]
        x1 = info.xrange[1]
        data1 = info.dep[where((info.indep GT x0) AND (info.indep LT x1))]
        data2 = info.dep2[where((info.indep GT x0) AND (info.indep LT x1))]
        data3 = info.dep3[where((info.indep GT x0) AND (info.indep LT x1))]
        data4 = info.dep4[where((info.indep GT x0) AND (info.indep LT x1))]
        data5 = info.dep5[where((info.indep GT x0) AND (info.indep LT x1))]
        info.yrange[0] = min([data1,data2,data3,data4,data5])
        info.yrange[1] = max([data1,data2,data3,data4,data5])

        ; Make new plot
        zfNewPlot, Event, info
      ENDIF
    end

    ; ## X&Y Zoom Out (half) ##
    Widget_Info(wWidget, FIND_BY_UNAME='ZoutBtn'): begin
      IF( Tag_Names(Event, /STRUCTURE_NAME) eq 'WIDGET_BUTTON' ) THEN BEGIN

        ; Set new range
        current_xrange = info.xrange[1] - info.xrange[0]
        info.xrange[0] = info.xrange[0] - current_xrange/2.0
        info.xrange[1] = info.xrange[1] + current_xrange/2.0

        current_yrange = info.yrange[1] - info.yrange[0]
        info.yrange[0] = info.yrange[0] - current_yrange/2.0
        info.yrange[1] = info.yrange[1] + current_yrange/2.0

        ; Make new plot
        zfNewPlot, Event, info
      ENDIF
    end
    ; ## X Zoom Out (half) ##
    Widget_Info(wWidget, FIND_BY_UNAME='xZoutBtn'): begin
      IF( Tag_Names(Event, /STRUCTURE_NAME) eq 'WIDGET_BUTTON' ) THEN BEGIN

        ; Set new range
        current_xrange = info.xrange[1] - info.xrange[0]
        info.xrange[0] = info.xrange[0] - current_xrange/2.0
        info.xrange[1] = info.xrange[1] + current_xrange/2.0

        ; Make new plot
        zfNewPlot, Event, info
      ENDIF
    end
    ; ## Y Zoom Out (half) ##
    Widget_Info(wWidget, FIND_BY_UNAME='yZoutBtn'): begin
      IF( Tag_Names(Event, /STRUCTURE_NAME) eq 'WIDGET_BUTTON' ) THEN BEGIN

        ; Set new range
        current_yrange = info.yrange[1] - info.yrange[0]
        info.yrange[0] = info.yrange[0] - current_yrange/2.0
        info.yrange[1] = info.yrange[1] + current_yrange/2.0

        ; Make new plot
        zfNewPlot, Event, info
      ENDIF
    end
    ; ## X&Y Zoom In (twice) ##
    Widget_Info(wWidget, FIND_BY_UNAME='ZinBtn'): begin
      IF( Tag_Names(Event, /STRUCTURE_NAME) eq 'WIDGET_BUTTON' ) THEN BEGIN

        ; Set new range
        current_xrange = info.xrange[1] - info.xrange[0]
        info.xrange[0] = info.xrange[0] + current_xrange/4.0
        info.xrange[1] = info.xrange[1] - current_xrange/4.0

        current_yrange = info.yrange[1] - info.yrange[0]
        info.yrange[0] = info.yrange[0] + current_yrange/4.0
        info.yrange[1] = info.yrange[1] - current_yrange/4.0

        ; Make new plot
        zfNewPlot, Event, info
      ENDIF
    end
    ; ## X Zoom In (twice) ##
    Widget_Info(wWidget, FIND_BY_UNAME='xZinBtn'): begin
      IF( Tag_Names(Event, /STRUCTURE_NAME) eq 'WIDGET_BUTTON' ) THEN BEGIN

        ; Set new range
        current_xrange = info.xrange[1] - info.xrange[0]
        info.xrange[0] = info.xrange[0] + current_xrange/4.0
        info.xrange[1] = info.xrange[1] - current_xrange/4.0

        ; Make new plot
        zfNewPlot, Event, info
      ENDIF
    end
    ; ## Y Zoom In (twice) ##
    Widget_Info(wWidget, FIND_BY_UNAME='yZinBtn'): begin
      IF( Tag_Names(Event, /STRUCTURE_NAME) eq 'WIDGET_BUTTON' ) THEN BEGIN

        ; Set new range
        current_yrange = info.yrange[1] - info.yrange[0]
        info.yrange[0] = info.yrange[0] + current_yrange/4.0
        info.yrange[1] = info.yrange[1] - current_yrange/4.0

        ; Make new plot
        zfNewPlot, Event, info
      ENDIF
    end

    ; ##Scroll to Left##
    Widget_Info(wWidget, FIND_BY_UNAME='ScrLBtn'): begin
      IF( Tag_Names(Event, /STRUCTURE_NAME) eq 'WIDGET_BUTTON' ) THEN BEGIN

        ; Set new range
        current_xrange = info.xrange[1] - info.xrange[0]
        info.xrange[0] = info.xrange[0] - current_xrange*0.8
        info.xrange[1] = info.xrange[1] - current_xrange*0.8

        ; Make new plot
        zfNewPlot, Event, info
      ENDIF
    end
    ; ##Scroll to Right##
    Widget_Info(wWidget, FIND_BY_UNAME='ScrRBtn'): begin
      IF( Tag_Names(Event, /STRUCTURE_NAME) eq 'WIDGET_BUTTON' ) THEN BEGIN

        ; Set new range
        current_xrange = info.xrange[1] - info.xrange[0]
        info.xrange[0] = info.xrange[0] + current_xrange*0.8
        info.xrange[1] = info.xrange[1] + current_xrange*0.8

        ; Make new plot
        zfNewPlot, Event, info
      ENDIF
    end
    ; ##Scroll Up##
    Widget_Info(wWidget, FIND_BY_UNAME='ScrUBtn'): begin
      IF( Tag_Names(Event, /STRUCTURE_NAME) eq 'WIDGET_BUTTON' ) THEN BEGIN

        ; Set new range
        current_yrange = info.yrange[1] - info.yrange[0]
        info.yrange[0] = info.yrange[0] + current_yrange*0.8
        info.yrange[1] = info.yrange[1] + current_yrange*0.8

        ; Make new plot
        zfNewPlot, Event, info
      ENDIF
    end
    ; ##Scroll Down##
    Widget_Info(wWidget, FIND_BY_UNAME='ScrDBtn'): begin
      IF( Tag_Names(Event, /STRUCTURE_NAME) eq 'WIDGET_BUTTON' ) THEN BEGIN

        ; Set new range
        current_yrange = info.yrange[1] - info.yrange[0]
        info.yrange[0] = info.yrange[0] - current_yrange*0.8
        info.yrange[1] = info.yrange[1] - current_yrange*0.8

        ; Make new plot
        zfNewPlot, Event, info
      ENDIF
    end

    ; ##Switch Extra Button Display##
    Widget_Info(wWidget, FIND_BY_UNAME='ExtBtn'): begin
      IF( Tag_Names(Event, /STRUCTURE_NAME) eq 'WIDGET_BUTTON' ) THEN $
        zfExtBtn, Event, info
    end

    ; ##Open Stat Window##
    Widget_Info(wWidget, FIND_BY_UNAME='StatBtn'): begin
      IF( Tag_Names(Event, /STRUCTURE_NAME) eq 'WIDGET_BUTTON' )THEN $
        zfStatBtn, Event, info
    end


    ; ##Differentiation##
    Widget_Info(wWidget, FIND_BY_UNAME='DiffBtn'): begin
      IF( Tag_Names(Event, /STRUCTURE_NAME) eq 'WIDGET_BUTTON' )THEN BEGIN

        ; Make an array to differentiate
        to_diff = info.dep
        IF n_elements(info.dep) EQ n_elements(info.dep2) THEN $
          to_diff = [[to_diff], [info.dep2]]
        IF n_elements(info.dep) EQ n_elements(info.dep3) THEN $
          to_diff = [[to_diff], [info.dep3]]
        IF n_elements(info.dep) EQ n_elements(info.dep4) THEN $
          to_diff = [[to_diff], [info.dep4]]
        IF n_elements(info.dep) EQ n_elements(info.dep5) THEN $
          to_diff = [[to_diff], [info.dep5]]
        to_diff = transpose(to_diff)

        ; Differentiate
        ;result = gb_diff2p(to_diff, diffed)
        sval = shift(to_diff, 0, 1)
        diffed = to_diff - sval

        ; Write back the array
        info.dep = diffed[0,*]
        IF (n_elements(diffed[*,0]) GE 2) THEN $
          info.dep2 = diffed[1,*]
        IF (n_elements(diffed[*,0]) GE 3) THEN $
          info.dep3 = diffed[2,*]
        IF (n_elements(diffed[*,0]) GE 4) THEN $
          info.dep4 = diffed[3,*]
        IF (n_elements(diffed[*,0]) GE 5) THEN $
          info.dep5 = diffed[4,*]

        ; Set new ranges
        info.yrange = [Min([info.dep,info.dep2,info.dep3,info.dep4,info.dep5]),$
                       Max([info.dep,info.dep2,info.dep3,info.dep4,info.dep5])]
        info.yrange_def = info.yrange

        ; Make new plot
        zfNewPlot, Event, info
      ENDIF
    end

    ; ##FFT##
    Widget_Info(wWidget, FIND_BY_UNAME='FFTBtn'): begin
      IF( Tag_Names(Event, /STRUCTURE_NAME) eq 'WIDGET_BUTTON' )THEN $
        zfFFTBtn, Event, info
    end

    ; ##Print Out##
    Widget_Info(wWidget, FIND_BY_UNAME='PrintBtn'): begin
      IF( Tag_Names(Event, /STRUCTURE_NAME) eq 'WIDGET_BUTTON' )THEN $
        zfPrintBtn, Event, info
    end

    ; ##Make a PS file##
    Widget_Info(wWidget, FIND_BY_UNAME='PSBtn'): begin
      IF( Tag_Names(Event, /STRUCTURE_NAME) eq 'WIDGET_BUTTON' )THEN $
        zfPSBtn, Event, info
    end

    ; ##Symbols On/Off##
    Widget_Info(wWidget, FIND_BY_UNAME='SymbBtn'): begin
      IF( Tag_Names(Event, /STRUCTURE_NAME) eq 'WIDGET_BUTTON' )THEN BEGIN

        ; Get widget ID of SymbBtn
        LineBtn = Widget_Info(wWidget, FIND_BY_UNAME='LineBtn')
        IF (LineBtn NE 0) THEN BEGIN
          ; Switch widgets sensitive
          IF (info.psym EQ 0) THEN BEGIN
            Widget_Control, LineBtn, SENSITIVE=1
          ENDIF ELSE BEGIN
            Widget_Control, LineBtn, /SET_BUTTON
            Widget_Control, LineBtn, SENSITIVE=0
          ENDELSE
          info.line = Widget_Info(LineBtn, /BUTTON_SET)
        ENDIF

        ; Switch info.psym
        info.psym = (info.psym EQ 0) ? -7:0

        ; Make new plot
        zfNewPlot, Event, info

      ENDIF
    end

    ; ##Plot Line On/Off##
    Widget_Info(wWidget, FIND_BY_UNAME='LineBtn'): begin
      IF( Tag_Names(Event, /STRUCTURE_NAME) eq 'WIDGET_BUTTON' )THEN BEGIN

        ; Switch info.line
        info.line = (info.line EQ 0) ? 1:0

        ; Make new plot
        zfNewPlot, Event, info

      ENDIF
    end


    else: BEGIN;#####End of altered code#####

   ; Update window sizes.

event.x = (event.x -    5 > 150) ; adjust for the top level base

IF (!d.name eq 'X') THEN $
  event.y = (event.y -  120 > 200) $ ; adjust for a part of STAT display area (Linux, etc.)
ELSE $ ; !d.name eq 'WIN'
  event.y = (event.y -  100 > 200) ; adjust for a part of STAT display area (Win)

info.zxsize = event.x
info.zysize = event.y

   ; Destroy old pixmap and create a new one.

WDelete, info.pixIndex
Window, XSize=event.x, YSize=event.y, /Free, /Pixmap
info.pixIndex = !D. Window

   ; Resize draw widget.

Widget_Control, info.drawID, Draw_XSize=(event.x > 200), Draw_YSize=(event.y > 150)

   ;  Set up colors for line plot.
   backColor = GetColor('charcoal', !D.Table_Size-2)
   dataColor = GetColor('yellow', !D.Table_Size-3)
   axisColor = GetColor('green', !D.Table_Size-4)
   ;#
   dataColor2 = GetColor('pink', !D.Table_Size-5)
   dataColor3 = GetColor('aqua', !D.Table_Size-6)
   dataColor4 = GetColor('magenta', !D.Table_Size-7)
   dataColor5 = GetColor('gray', !D.Table_Size-8)
   ;#
   faxisColor = GetColor('aquamarine', !D.Table_Size-9)

   ;  Set up properties (line and symbol) of line plot.
   get_prop_plot, info, PSYMS=psyms

  ; Plot position
pos1 = [0.13,0.4,0.965,0.9] ; for normal data
pos2 = [0.13,0.1,0.965,0.4] ; for flag data


   ; Draw the plot in both windows.

;# Back side
  Plot, info.indep, info.dep, $
     XRange=info.xrange, XStyle=info.xstyle, Xticklen=info.ticklen, Xgridstyle=(info.ticklen NE 0.02) ? 1:0, $
     YRange=info.yrange, YStyle=info.ystyle, Yticklen=info.ticklen, Ygridstyle=(info.ticklen NE 0.02) ? 1:0, $
     XLog=info.xlog, YLog=info.ylog, $
     Color=axisColor, /NoData, _Extra=*info.extraKeywords, $
     position = pos1
  OPlot, info.indep, info.dep, Color=dataColor, psym = psyms[0]

  ; Standard position of legends
  title_pos1a = CONVERT_COORD([0.85, 0.90], /NORMAL, /TO_DATA)  ; for symbol
  title_pos1b = CONVERT_COORD([0.86, 0.90], /NORMAL, /TO_DEVICE); for label string
  title_pos1b = CONVERT_COORD(title_pos1b[0],title_pos1b[1]-5, /DEVICE, /TO_DATA)

  title_pos2a = CONVERT_COORD([0.85, 0.85], /NORMAL, /TO_DATA)  ; for symbol
  title_pos2b = CONVERT_COORD([0.86, 0.85], /NORMAL, /TO_DEVICE); for label string
  title_pos2b = CONVERT_COORD(title_pos2b[0],title_pos2b[1]-5, /DEVICE, /TO_DATA)

  title_pos3a = CONVERT_COORD([0.85, 0.80], /NORMAL, /TO_DATA)  ; for symbol
  title_pos3b = CONVERT_COORD([0.86, 0.80], /NORMAL, /TO_DEVICE); for label string
  title_pos3b = CONVERT_COORD(title_pos3b[0],title_pos3b[1]-5, /DEVICE, /TO_DATA)

  title_pos4a = CONVERT_COORD([0.85, 0.75], /NORMAL, /TO_DATA)  ; for symbol
  title_pos4b = CONVERT_COORD([0.86, 0.75], /NORMAL, /TO_DEVICE); for label string
  title_pos4b = CONVERT_COORD(title_pos4b[0],title_pos4b[1]-5, /DEVICE, /TO_DATA)

  title_pos5a = CONVERT_COORD([0.85, 0.70], /NORMAL, /TO_DATA)  ; for symbol
  title_pos5b = CONVERT_COORD([0.86, 0.70], /NORMAL, /TO_DEVICE); for label string
  title_pos5b = CONVERT_COORD(title_pos5b[0],title_pos5b[1]-5, /DEVICE, /TO_DATA)
;#
  IF n_elements(info.dep2) EQ n_elements(info.dep) THEN BEGIN
      OPlot, info.indep, info.dep2, Color=dataColor2, psym = psyms[1]
      XYOUTS, title_pos1b[0], title_pos1b[1],  info.y1title, Color=dataColor
      Oplot, [title_pos1a[0]], [title_pos1a[1]], Color=dataColor, psym = abs(psyms[0])
      XYOUTS, title_pos2b[0], title_pos2b[1], info.y2title, Color=dataColor2
      Oplot, [title_pos2a[0]], [title_pos2a[1]], Color=dataColor2, psym = abs(psyms[1])
  ENDIF
  IF n_elements(info.dep3) EQ n_elements(info.dep) THEN BEGIN
      OPlot, info.indep, info.dep3, Color=dataColor3, psym = psyms[2]
      XYOUTS, title_pos3b[0], title_pos3b[1], info.y3title, Color=dataColor3
      Oplot, [title_pos3a[0]], [title_pos3a[1]], Color=dataColor3, psym = abs(psyms[2])
  ENDIF
  IF n_elements(info.dep4) EQ n_elements(info.dep) THEN BEGIN
      OPlot, info.indep, info.dep4, Color=dataColor4, psym = psyms[3]
      XYOUTS, title_pos4b[0], title_pos4b[1], info.y4title, Color=dataColor4
      Oplot, [title_pos4a[0]], [title_pos4a[1]], Color=dataColor4, psym = abs(psyms[3])
  ENDIF
  IF n_elements(info.dep5) EQ n_elements(info.dep) THEN BEGIN
      OPlot, info.indep, info.dep5, Color=dataColor5, psym = psyms[4]
      XYOUTS, title_pos5b[0], title_pos5b[1], info.y5title, Color=dataColor5
      Oplot, [title_pos5a[0]], [title_pos5a[1]], Color=dataColor5, psym = abs(psyms[4])
  ENDIF
;
; Top side
;######################
WSet, info.drawIndex

;############;
; Flags plot ;
;############;
; Set y-range
f_yrange_max=2
IF n_elements(info.flag2) EQ n_elements(info.indep) THEN f_yrange_max=f_yrange_max+1.1
IF n_elements(info.flag3) EQ n_elements(info.indep) THEN f_yrange_max=f_yrange_max+1.1
IF n_elements(info.flag4) EQ n_elements(info.indep) THEN f_yrange_max=f_yrange_max+1.1
IF n_elements(info.flag5) EQ n_elements(info.indep) THEN f_yrange_max=f_yrange_max+1.1
f_yrange = [-1, f_yrange_max]

; Base lavel for the next flag plot
next_flag_base = 0 ; initial level

; Plot 1st flag
IF (n_elements(info.flag1) EQ n_elements(info.indep)) THEN BEGIN
  Plot, info.indep, info.flag1, position=pos2, ytickformat="(A1)", $
     Background=backColor, Color=faxisColor, /NoData, _Extra=*info.extraKeywords, $
     XRange=info.xrange, XStyle=info.xstyle, Xticklen=info.ticklen, Xgridstyle=(info.ticklen NE 0.02) ? 1:0, $
     YRange=f_yrange,    YStyle=info.ystyle, Yticklen=info.ticklen, Ygridstyle=(info.ticklen NE 0.02) ? 1:0
  Plot, info.indep, info.flag1, position=pos2, xstyle=info.xstyle+4, ystyle=info.ystyle+4, $
     XRange=info.xrange, YRange=f_yrange, $
     thick=1, psym=psyms[0], /noerase, Color=dataColor
  next_flag_base = next_flag_base + 1.1
ENDIF

; Standard position of legends
title_posF1a = CONVERT_COORD([0.85, 0.15], /NORMAL, /TO_DATA)  ; for symbol
title_posF1b = CONVERT_COORD([0.86, 0.15], /NORMAL, /TO_DEVICE); for label string
title_posF1b = CONVERT_COORD(title_posF1b[0],title_posF1b[1]-5, /DEVICE, /TO_DATA)

title_posF2a = CONVERT_COORD([0.85, 0.19], /NORMAL, /TO_DATA)  ; for symbol
title_posF2b = CONVERT_COORD([0.86, 0.19], /NORMAL, /TO_DEVICE); for label string
title_posF2b = CONVERT_COORD(title_posF2b[0],title_posF2b[1]-5, /DEVICE, /TO_DATA)

title_posF3a = CONVERT_COORD([0.85, 0.23], /NORMAL, /TO_DATA)  ; for symbol
title_posF3b = CONVERT_COORD([0.86, 0.23], /NORMAL, /TO_DEVICE); for label string
title_posF3b = CONVERT_COORD(title_posF3b[0],title_posF3b[1]-5, /DEVICE, /TO_DATA)

title_posF4a = CONVERT_COORD([0.85, 0.27], /NORMAL, /TO_DATA)  ; for symbol
title_posF4b = CONVERT_COORD([0.86, 0.27], /NORMAL, /TO_DEVICE); for label string
title_posF4b = CONVERT_COORD(title_posF4b[0],title_posF4b[1]-5, /DEVICE, /TO_DATA)

title_posF5a = CONVERT_COORD([0.85, 0.31], /NORMAL, /TO_DATA)  ; for symbol
title_posF5b = CONVERT_COORD([0.86, 0.31], /NORMAL, /TO_DEVICE); for label string
title_posF5b = CONVERT_COORD(title_posF5b[0],title_posF5b[1]-5, /DEVICE, /TO_DATA)

; Show the title of flag1
XYOUTS, title_posF1b[0], title_posF1b[1],  info.f1title, Color=dataColor

; Plot other flags
IF (n_elements(info.flag2) EQ n_elements(info.indep)) THEN BEGIN
  Plot, info.indep, info.flag2+next_flag_base,  position=pos2, xstyle=info.xstyle+4, ystyle=info.ystyle+4, $
    XRange=info.xrange, YRange=f_yrange, $
    thick=1, psym=psyms[0], /noerase, Color=dataColor2
  next_flag_base = next_flag_base + 1.1
  XYOUTS, title_posF2b[0], title_posF2b[1],  info.f2title, Color=dataColor2
ENDIF
IF (n_elements(info.flag3) EQ n_elements(info.indep)) THEN BEGIN
  Plot, info.indep, info.flag3+next_flag_base,  position=pos2, xstyle=info.xstyle+4, ystyle=info.ystyle+4, $
    XRange=info.xrange, YRange=f_yrange, $
    thick=1, psym=psyms[0], /noerase, Color=dataColor3
  next_flag_base = next_flag_base + 1.1
  XYOUTS, title_posF3b[0], title_posF3b[1],  info.f3title, Color=dataColor3
ENDIF
IF (n_elements(info.flag4) EQ n_elements(info.indep)) THEN BEGIN
  Plot, info.indep, info.flag4+next_flag_base,  position=pos2, xstyle=info.xstyle+4, ystyle=info.ystyle+4, $
    XRange=info.xrange, YRange=f_yrange, $
    thick=1, psym=psyms[0], /noerase, Color=dataColor4
  next_flag_base = next_flag_base + 1.1
  XYOUTS, title_posF4b[0], title_posF4b[1],  info.f4title, Color=dataColor4
ENDIF
IF (n_elements(info.flag5) EQ n_elements(info.indep)) THEN BEGIN
  Plot, info.indep, info.flag5+next_flag_base,  position=pos2, xstyle=info.xstyle+4, ystyle=info.ystyle+4, $
    XRange=info.xrange, YRange=f_yrange, $
    thick=1, psym=psyms[0], /noerase, Color=dataColor5
  next_flag_base = next_flag_base + 1.1
  XYOUTS, title_posF5b[0], title_posF5b[1],  info.f5title, Color=dataColor5
ENDIF

;############;
; Data Plot  ;
;############;
  Plot, info.indep, info.dep, $
     XRange=info.xrange, XStyle=info.xstyle, Xticklen=info.ticklen, Xgridstyle=(info.ticklen NE 0.02) ? 1:0, $
     YRange=info.yrange, YStyle=info.ystyle, Yticklen=info.ticklen, Ygridstyle=(info.ticklen NE 0.02) ? 1:0, $
     XLog=info.xlog, YLog=info.ylog, $
     Background=backColor, Color=axisColor, /NoData, _Extra=*info.extraKeywords, $
     position = pos1, xtickformat="(A1)", /noerase
  OPlot, info.indep, info.dep, Color=dataColor, psym = psyms[0]
  XYOUTS, title_pos1b[0], title_pos1b[1],  info.y1title, Color=dataColor
  Oplot, [title_pos1a[0]], [title_pos1a[1]], Color=dataColor, psym = abs(psyms[0])

;#
  IF n_elements(info.dep2) EQ n_elements(info.dep) THEN BEGIN
      OPlot, info.indep, info.dep2, Color=dataColor2, psym = psyms[1]
      XYOUTS, title_pos2b[0], title_pos2b[1], info.y2title, Color=dataColor2
      Oplot, [title_pos2a[0]], [title_pos2a[1]], Color=dataColor2, psym = abs(psyms[1])
  ENDIF
  IF n_elements(info.dep3) EQ n_elements(info.dep) THEN BEGIN
      OPlot, info.indep, info.dep3, Color=dataColor3, psym = psyms[2]
      XYOUTS, title_pos3b[0], title_pos3b[1], info.y3title, Color=dataColor3
      Oplot, [title_pos3a[0]], [title_pos3a[1]], Color=dataColor3, psym = abs(psyms[2])
  ENDIF
  IF n_elements(info.dep4) EQ n_elements(info.dep) THEN BEGIN
      OPlot, info.indep, info.dep4, Color=dataColor4, psym = psyms[3]
      XYOUTS, title_pos4b[0], title_pos4b[1], info.y4title, Color=dataColor4
      Oplot, [title_pos4a[0]], [title_pos4a[1]], Color=dataColor4, psym = abs(psyms[3])
  ENDIF
  IF n_elements(info.dep5) EQ n_elements(info.dep) THEN BEGIN
      OPlot, info.indep, info.dep5, Color=dataColor5, psym = psyms[4]
      XYOUTS, title_pos5b[0], title_pos5b[1], info.y5title, Color=dataColor5
      Oplot, [title_pos5a[0]], [title_pos5a[1]], Color=dataColor5, psym = abs(psyms[4])
  ENDIF
  ; ## DISPLAY END##

   ; Update system parameters.

info.x = !X
info.y = !Y
info.p = !P

Widget_Control, event.top, Set_UValue=info, /No_Copy

    end
  endcase

END ; of ZfPLOT_RESIZE **********************************************************


PRO ZfPLOT_PROCESS_EVENTS, event

   ; This event handler ONLY responds to button down events from the
   ; draw widget. If it gets a DOWN event, it does three things: (1) sets
   ; the static and dynamic corners of the arrow box, (2) changes the
   ; event handler for the draw widget to ZPLOT_DRAWBOX and turns on MOTION
   ; events, and (3) update the user's color table vectors.

possibleEventTypes = [ 'DOWN', 'UP', 'MOTION', 'SCROLL' ]
thisEvent = possibleEventTypes(event.type)
IF thisEvent NE 'DOWN' THEN RETURN

    ; Must be DOWN event to get here, so get info structure.

 Widget_Control, event.top, Get_UValue=info, /No_Copy

    ; Set the static corners of the arrow box to current
    ; cursor location.

 info.xs = event.x
 info.ys = event.y

    ; Change the event handler for the draw widget and turn MOTION
    ; events ON.

 Widget_Control, event.id, Event_Pro='ZfPLOT_DRAWBOX', $
    Draw_Motion_Events=1

   ; Put the info structure back into its storage location.

Widget_Control, event.top, Set_UValue=info, /No_Copy
END ; of ZPLOT_PROCESS_EVENTS *****************************************************



PRO ZfPLOT_DRAWBOX, event

   ; This event handler continuously draws and erases the arrow box until it
   ; receives an UP event from the draw widget. Then it turns draw widget motion
   ; events OFF and changes the event handler for the draw widget back to
   ; ZPLOT_PROCESS_EVENTS.

   ; Get the info structure out of the top-level base.

Widget_Control, event.top, Get_UValue=info, /No_Copy

   ;  Set up colors for line plot.
   backColor = GetColor('charcoal', !D.Table_Size-2)
   dataColor = GetColor('yellow', !D.Table_Size-3)
   axisColor = GetColor('green', !D.Table_Size-4)
   boxColor = GetColor('beige', !D.Table_Size-5)
   ;#
   dataColor2 = GetColor('pink', !D.Table_Size-6)
   dataColor3 = GetColor('aqua', !D.Table_Size-7)
   dataColor4 = GetColor('magenta', !D.Table_Size-8)
   dataColor5 = GetColor('gray', !D.Table_Size-9)
   ;#
   faxisColor = GetColor('aquamarine', !D.Table_Size-9)

   ;  Set up properties (line and symbol) of line plot.
   get_prop_plot, info, PSYMS=psyms

  ; Plot position
pos1 = [0.13,0.4,0.965,0.9] ; for normal data
pos2 = [0.13,0.1,0.965,0.4] ; for flag data

   ; What type of an event is this?

possibleEventTypes = [ 'DOWN', 'UP', 'MOTION', 'SCROLL' ]
thisEvent = possibleEventTypes(event.type)

IF thisEvent EQ 'UP' THEN BEGIN

      ; If this is an UP event, you need to erase the zoombox, restore
      ; the user's color table, turn motion events OFF, set the
      ; draw widget's event handler back to ZPLOT_PROCESS_EVENTS, and
      ; draw the "zoomed" plot in both the draw widget and the pixmap.

      ; Erase the arrow box one final time by copying the plot from the pixmap.

   WSet, info.drawIndex
   Device, Copy = [0, 0, info.zxsize, info.zysize, 0, 0, info.pixIndex]

      ; Turn motion events off and redirect the events to ZPLOT_PROCESS_EVENTS.

    Widget_Control, event.id, Draw_Motion_Events=0, $
       Event_Pro='ZfPLOT_PROCESS_EVENTS'

      ; Draw the "zoomed" plot. Start by getting the new limits to the plot
      ; (i.e., the LAST zoom box outline).

   x = [info.xs, event.x]
   y = [info.ys, event.y]

;#
   ; Make sure the x values are ordered as [min, max].
   IF info.xs GT event.x THEN x = [event.x, info.xs]

   ; Don't want exact style if we are drawing entire plot.
   ;IF info.xs EQ event.x THEN xstyle = 0 ELSE xstyle = 1
   ;info.xstyle = xstyle

;#
   ; Make sure the y values are ordered as [min, max].
   IF info.ys GT event.y THEN y = [event.y, info.ys]

   ; Don't want exact style if we are drawing entire plot.
   ;IF info.ys EQ event.y THEN ystyle = 0 ELSE ystyle = 1
   ;info.ystyle = ystyle


   ; Restore plot system variables.
   !X = info.x
   !Y = info.y
   !P = info.p


   ; Convert the x device coordinates to data coordinates.
   coords = Convert_Coord(x, y, /Device, /To_Data)


   x1 = coords(0,0)
   x2 = coords(0,1)

   ;#####altered code#####
   y1 = coords(1,0)
   y2 = coords(1,1)


   ; ## Set new ranges ##
   IF ((x1 NE x2) and (y1 NE y2)) THEN BEGIN
     info.xrange = [x1,x2]
     info.yrange = [y1,y2]
     info.xstyle=1
     info.ystyle=1
   ; When click without drag, does not change the ranges.
   ; But, print out the coordinates of the point.
   ENDIF ELSE BEGIN
     print, 'X:', x1, ',   Y:', y1
   ENDELSE

   ;######################


      ; Draw the "zoomed" plot in both the draw widget and the pixmap.

;# Top side
;############;
; Flags plot ;
;############;
; Set y-range
f_yrange_max=2
IF n_elements(info.flag2) EQ n_elements(info.indep) THEN f_yrange_max=f_yrange_max+1.1
IF n_elements(info.flag3) EQ n_elements(info.indep) THEN f_yrange_max=f_yrange_max+1.1
IF n_elements(info.flag4) EQ n_elements(info.indep) THEN f_yrange_max=f_yrange_max+1.1
IF n_elements(info.flag5) EQ n_elements(info.indep) THEN f_yrange_max=f_yrange_max+1.1
f_yrange = [-1, f_yrange_max]

; Base lavel for the next flag plot
next_flag_base = 0 ; initial level

; Plot 1st flag
IF (n_elements(info.flag1) EQ n_elements(info.indep)) THEN BEGIN
  Plot, info.indep, info.flag1, position=pos2, ytickformat="(A1)", $
     Background=backColor, Color=faxisColor, /NoData, _Extra=*info.extraKeywords, $
     XRange=info.xrange, XStyle=info.xstyle, Xticklen=info.ticklen, Xgridstyle=(info.ticklen NE 0.02) ? 1:0, $
     YRange=f_yrange,    YStyle=info.ystyle, Yticklen=info.ticklen, Ygridstyle=(info.ticklen NE 0.02) ? 1:0
  Plot, info.indep, info.flag1, position=pos2, xstyle=info.xstyle+4, ystyle=info.ystyle+4, $
     XRange=info.xrange, YRange=f_yrange, $
     thick=1, psym=psyms[0], /noerase, Color=dataColor
  next_flag_base = next_flag_base + 1.1
ENDIF

; Standard position of legends
title_posF1a = CONVERT_COORD([0.85, 0.15], /NORMAL, /TO_DATA)  ; for symbol
title_posF1b = CONVERT_COORD([0.86, 0.15], /NORMAL, /TO_DEVICE); for label string
title_posF1b = CONVERT_COORD(title_posF1b[0],title_posF1b[1]-5, /DEVICE, /TO_DATA)

title_posF2a = CONVERT_COORD([0.85, 0.19], /NORMAL, /TO_DATA)  ; for symbol
title_posF2b = CONVERT_COORD([0.86, 0.19], /NORMAL, /TO_DEVICE); for label string
title_posF2b = CONVERT_COORD(title_posF2b[0],title_posF2b[1]-5, /DEVICE, /TO_DATA)

title_posF3a = CONVERT_COORD([0.85, 0.23], /NORMAL, /TO_DATA)  ; for symbol
title_posF3b = CONVERT_COORD([0.86, 0.23], /NORMAL, /TO_DEVICE); for label string
title_posF3b = CONVERT_COORD(title_posF3b[0],title_posF3b[1]-5, /DEVICE, /TO_DATA)

title_posF4a = CONVERT_COORD([0.85, 0.27], /NORMAL, /TO_DATA)  ; for symbol
title_posF4b = CONVERT_COORD([0.86, 0.27], /NORMAL, /TO_DEVICE); for label string
title_posF4b = CONVERT_COORD(title_posF4b[0],title_posF4b[1]-5, /DEVICE, /TO_DATA)

title_posF5a = CONVERT_COORD([0.85, 0.31], /NORMAL, /TO_DATA)  ; for symbol
title_posF5b = CONVERT_COORD([0.86, 0.31], /NORMAL, /TO_DEVICE); for label string
title_posF5b = CONVERT_COORD(title_posF5b[0],title_posF5b[1]-5, /DEVICE, /TO_DATA)

; Show the title of flag1
XYOUTS, title_posF1b[0], title_posF1b[1],  info.f1title, Color=dataColor

; Plot other flags
IF (n_elements(info.flag2) EQ n_elements(info.indep)) THEN BEGIN
  Plot, info.indep, info.flag2+next_flag_base,  position=pos2, xstyle=info.xstyle+4, ystyle=info.ystyle+4, $
    XRange=info.xrange, YRange=f_yrange, $
    thick=1, psym=psyms[0], /noerase, Color=dataColor2
  next_flag_base = next_flag_base + 1.1
  XYOUTS, title_posF2b[0], title_posF2b[1],  info.f2title, Color=dataColor2
ENDIF
IF (n_elements(info.flag3) EQ n_elements(info.indep)) THEN BEGIN
  Plot, info.indep, info.flag3+next_flag_base,  position=pos2, xstyle=info.xstyle+4, ystyle=info.ystyle+4, $
    XRange=info.xrange, YRange=f_yrange, $
    thick=1, psym=psyms[0], /noerase, Color=dataColor3
  next_flag_base = next_flag_base + 1.1
  XYOUTS, title_posF3b[0], title_posF3b[1],  info.f3title, Color=dataColor3
ENDIF
IF (n_elements(info.flag4) EQ n_elements(info.indep)) THEN BEGIN
  Plot, info.indep, info.flag4+next_flag_base,  position=pos2, xstyle=info.xstyle+4, ystyle=info.ystyle+4, $
    XRange=info.xrange, YRange=f_yrange, $
    thick=1, psym=psyms[0], /noerase, Color=dataColor4
  next_flag_base = next_flag_base + 1.1
  XYOUTS, title_posF4b[0], title_posF4b[1],  info.f4title, Color=dataColor4
ENDIF
IF (n_elements(info.flag5) EQ n_elements(info.indep)) THEN BEGIN
  Plot, info.indep, info.flag5+next_flag_base,  position=pos2, xstyle=info.xstyle+4, ystyle=info.ystyle+4, $
    XRange=info.xrange, YRange=f_yrange, $
    thick=1, psym=psyms[0], /noerase, Color=dataColor5
  next_flag_base = next_flag_base + 1.1
  XYOUTS, title_posF5b[0], title_posF5b[1],  info.f5title, Color=dataColor5
ENDIF

;############;
; Data Plot  ;
;############;
  Plot, info.indep, info.dep, $
     XRange=info.xrange, XStyle=info.xstyle, Xticklen=info.ticklen, Xgridstyle=(info.ticklen NE 0.02) ? 1:0, $
     YRange=info.yrange, YStyle=info.ystyle, Yticklen=info.ticklen, Ygridstyle=(info.ticklen NE 0.02) ? 1:0, $
     XLog=info.xlog, YLog=info.ylog, $
     Background=backColor, Color=axisColor, /NoData, _Extra=*info.extraKeywords, $
     position = pos1, xtickformat="(A1)", /noerase

  ; Standard position of legends
  title_pos1a = CONVERT_COORD([0.85, 0.90], /NORMAL, /TO_DATA)  ; for symbol
  title_pos1b = CONVERT_COORD([0.86, 0.90], /NORMAL, /TO_DEVICE); for label string
  title_pos1b = CONVERT_COORD(title_pos1b[0],title_pos1b[1]-5, /DEVICE, /TO_DATA)

  title_pos2a = CONVERT_COORD([0.85, 0.85], /NORMAL, /TO_DATA)  ; for symbol
  title_pos2b = CONVERT_COORD([0.86, 0.85], /NORMAL, /TO_DEVICE); for label string
  title_pos2b = CONVERT_COORD(title_pos2b[0],title_pos2b[1]-5, /DEVICE, /TO_DATA)

  title_pos3a = CONVERT_COORD([0.85, 0.80], /NORMAL, /TO_DATA)  ; for symbol
  title_pos3b = CONVERT_COORD([0.86, 0.80], /NORMAL, /TO_DEVICE); for label string
  title_pos3b = CONVERT_COORD(title_pos3b[0],title_pos3b[1]-5, /DEVICE, /TO_DATA)

  title_pos4a = CONVERT_COORD([0.85, 0.75], /NORMAL, /TO_DATA)  ; for symbol
  title_pos4b = CONVERT_COORD([0.86, 0.75], /NORMAL, /TO_DEVICE); for label string
  title_pos4b = CONVERT_COORD(title_pos4b[0],title_pos4b[1]-5, /DEVICE, /TO_DATA)

  title_pos5a = CONVERT_COORD([0.85, 0.70], /NORMAL, /TO_DATA)  ; for symbol
  title_pos5b = CONVERT_COORD([0.86, 0.70], /NORMAL, /TO_DEVICE); for label string
  title_pos5b = CONVERT_COORD(title_pos5b[0],title_pos5b[1]-5, /DEVICE, /TO_DATA)

  OPlot, info.indep, info.dep, Color=dataColor, psym = psyms[0]
  XYOUTS, title_pos1b[0], title_pos1b[1],  info.y1title, Color=dataColor
  Oplot, [title_pos1a[0]], [title_pos1a[1]], Color=dataColor, psym = abs(psyms[0])

;#
  IF n_elements(info.dep2) EQ n_elements(info.dep) THEN BEGIN
      OPlot, info.indep, info.dep2, Color=dataColor2, psym = psyms[1]
      XYOUTS, title_pos2b[0], title_pos2b[1], info.y2title, Color=dataColor2
      Oplot, [title_pos2a[0]], [title_pos2a[1]], Color=dataColor2, psym = abs(psyms[1])
  ENDIF
  IF n_elements(info.dep3) EQ n_elements(info.dep) THEN BEGIN
      OPlot, info.indep, info.dep3, Color=dataColor3, psym = psyms[2]
      XYOUTS, title_pos3b[0], title_pos3b[1], info.y3title, Color=dataColor3
      Oplot, [title_pos3a[0]], [title_pos3a[1]], Color=dataColor3, psym = abs(psyms[2])
  ENDIF
  IF n_elements(info.dep4) EQ n_elements(info.dep) THEN BEGIN
      OPlot, info.indep, info.dep4, Color=dataColor4, psym = psyms[3]
      XYOUTS, title_pos4b[0], title_pos4b[1], info.y4title, Color=dataColor4
      Oplot, [title_pos4a[0]], [title_pos4a[1]], Color=dataColor4, psym = abs(psyms[3])
  ENDIF
  IF n_elements(info.dep5) EQ n_elements(info.dep) THEN BEGIN
      OPlot, info.indep, info.dep5, Color=dataColor5, psym = psyms[4]
      XYOUTS, title_pos5b[0], title_pos5b[1], info.y5title, Color=dataColor5
      Oplot, [title_pos5a[0]], [title_pos5a[1]], Color=dataColor5, psym = abs(psyms[4])
  ENDIF

;
; Back side
;######################
WSet, info.pixIndex

  Plot, info.indep, info.dep, $
     XRange=info.xrange, XStyle=info.xstyle, Xticklen=info.ticklen, Xgridstyle=(info.ticklen NE 0.02) ? 1:0, $
     YRange=info.yrange, YStyle=info.ystyle, Yticklen=info.ticklen, Ygridstyle=(info.ticklen NE 0.02) ? 1:0, $
     XLog=info.xlog, YLog=info.ylog, $
     Color=axisColor, /NoData, _Extra=*info.extraKeywords, $
     position = pos1
  OPlot, info.indep, info.dep, Color=dataColor, psym = psyms[0]

;#
  IF n_elements(info.dep2) EQ n_elements(info.dep) THEN BEGIN
      OPlot, info.indep, info.dep2, Color=dataColor2, psym = psyms[1]
      XYOUTS, title_pos1b[0], title_pos1b[1],  info.y1title, Color=dataColor
      Oplot, [title_pos1a[0]], [title_pos1a[1]], Color=dataColor, psym = abs(psyms[0])
      XYOUTS, title_pos2b[0], title_pos2b[1], info.y2title, Color=dataColor2
      Oplot, [title_pos2a[0]], [title_pos2a[1]], Color=dataColor2, psym = abs(psyms[1])
  ENDIF
  IF n_elements(info.dep3) EQ n_elements(info.dep) THEN BEGIN
      OPlot, info.indep, info.dep3, Color=dataColor3, psym = psyms[2]
      XYOUTS, title_pos3b[0], title_pos3b[1], info.y3title, Color=dataColor3
      Oplot, [title_pos3a[0]], [title_pos3a[1]], Color=dataColor3, psym = abs(psyms[2])
  ENDIF
  IF n_elements(info.dep4) EQ n_elements(info.dep) THEN BEGIN
      OPlot, info.indep, info.dep4, Color=dataColor4, psym = psyms[3]
      XYOUTS, title_pos4b[0], title_pos4b[1], info.y4title, Color=dataColor4
      Oplot, [title_pos4a[0]], [title_pos4a[1]], Color=dataColor4, psym = abs(psyms[3])
  ENDIF
  IF n_elements(info.dep5) EQ n_elements(info.dep) THEN BEGIN
      OPlot, info.indep, info.dep5, Color=dataColor5, psym = psyms[4]
      XYOUTS, title_pos5b[0], title_pos5b[1], info.y5title, Color=dataColor5
      Oplot, [title_pos5a[0]], [title_pos5a[1]], Color=dataColor5, psym = abs(psyms[4])
  ENDIF


  ; ## DISPLAY END##


      ; Update the plot system variables.

   info.x = !X
   info.y = !Y
   info.p = !P

      ; Put the info structure back into its storage location and then, out of here!

   Widget_Control, event.top, Set_UValue=info, /No_Copy
   RETURN
ENDIF ; thisEvent = UP


   ; Most of the action in this event handler occurs here while we are waiting
   ; for an UP event to occur. As long as we don't get it, keep erasing the
   ; old zoom box and drawing a new one.

   ; Erase the old zoom box.

WSet, info.drawIndex
Device, Copy = [0, 0, info.zxsize, info.zysize, 0, 0, info.pixIndex]

   ; Update the dynamic corner of the zoom box to the current cursor location.

 info.xd = event.x
 info.yd = event.y

   ; Restore plot system variables.

!X = info.x
!Y = info.y
!P = info.p

   ; Convert the x device coordinates to data coordinates.

x = [info.xs, event.x]
y = [info.ys, event.y]
coords = Convert_Coord(x, y, /Device, /To_Data)


x1 = coords(0,0)
x2 = coords(0,1)
y1 = coords(1,0)
y2 = coords(1,1)

   ; Draw the arrow box.
IF info.dbly EQ 1 THEN $
   Arrow, x1, y1, x2, y1, /Data, Color=boxColor, /Solid, HSize=12 $
ELSE $
   Arrow, x1, y1, x2, y2, /Data, Color=boxColor, /Solid, HSize=12

IF info.ylog THEN BEGIN
  IF info.dbly EQ 1 THEN BEGIN
   PlotS, [x1, x1], [10^!Y.CRange[0], 10^!Y.CRange[1]], Color=boxColor
   PlotS, [x2, x2], [10^!Y.CRange[0], 10^!Y.CRange[1]], Color=boxColor
   PlotS, [x1, x2], [10^!Y.CRange[0], 10^!Y.CRange[0]], Color=boxColor
   PlotS, [x1, x2], [10^!Y.CRange[1], 10^!Y.CRange[1]], Color=boxColor
  ENDIF ELSE BEGIN
   PlotS, [x1, x1], [y1, y2], Color=boxColor
   PlotS, [x2, x2], [y1, y2], Color=boxColor
   PlotS, [x1, x2], [y1, y1], Color=boxColor
   PlotS, [x1, x2], [y2, y2], Color=boxColor
  ENDELSE
ENDIF ELSE BEGIN
  IF info.dbly EQ 1 THEN BEGIN
   PlotS, [x1, x1], [!Y.CRange[0], !Y.CRange[1]], Color=boxColor
   PlotS, [x2, x2], [!Y.CRange[0], !Y.CRange[1]], Color=boxColor
   PlotS, [x1, x2], [!Y.CRange[0], !Y.CRange[0]], Color=boxColor
   PlotS, [x1, x2], [!Y.CRange[1], !Y.CRange[1]], Color=boxColor
  ENDIF ELSE BEGIN
   PlotS, [x1, x1], [y1, y2], Color=boxColor
   PlotS, [x2, x2], [y1, y2], Color=boxColor
   PlotS, [x1, x2], [y1, y1], Color=boxColor
   PlotS, [x1, x2], [y2, y2], Color=boxColor
  ENDELSE
ENDELSE

   ; Put the info structure back into its storage location.

Widget_Control, event.top, Set_UValue=info, /No_Copy
END ; of ZfPLOT_DRAWBOX ******************************************************************



PRO ZPLOT_FLAG, x, y, Y2DATA=y2, Y3DATA=y3, Y4DATA=y4, Y5DATA=y5, $
   Y1TITLE=y1title, Y2TITLE=y2title, Y3TITLE=y3title, Y4TITLE=y4title, Y5TITLE=y5title, $
   Zoom_XSize=zxsize, Zoom_YSize=zysize, XRANGE=xrange, $
   Group_Leader=group, XLog=xlog, YLog=ylog, PSYM=psym, FILENAME=filename, EXTBTN_MODE=extbtn_mode, $
   XOffset=xoffset, YOffset=yoffset, $
   FLAG1=flag1, FLAG2=flag2, FLAG3=flag3, FLAG4=flag4, FLAG5=flag5, $
   F1TITLE=f1title, F2TITLE=f2title, F3TITLE=f3title, F4TITLE=f4title, F5TITLE=f5title, $
   _Extra=extra

;## 'y' must not be flag (0/1) data.

   ; This procedure allows the user to "zoom" into the data plot
   ; by drawing an arrow box around the part of the data to zoom into.
   ; The arrow box will be drawn and erased by using a pixmap and
   ; the "Device Copy" technique.

   ; On an error condition, return to the main level of IDL.

On_Error, 1

   ; Was data passed into the procedure? If not, create some.

CASE N_Params() OF

   0: BEGIN
   dep = Findgen(101)
   dep = Sin(dep/5) / Exp(dep/50)
   indep = Findgen(101)
   ENDCASE

   1: BEGIN
   dep = x
   indep = Findgen(N_Elements(dep))
   ENDCASE

   2: BEGIN
   dep = y
   indep = x
   ENDCASE

ENDCASE

   ;  Set up colors for line plot.

backColor = GetColor('charcoal', !D.Table_Size-2)
dataColor = GetColor('yellow', !D.Table_Size-3)
axisColor = GetColor('green', !D.Table_Size-4)
;#
dataColor2 = GetColor('pink', !D.Table_Size-5)
dataColor3 = GetColor('aqua', !D.Table_Size-6)
dataColor4 = GetColor('magenta', !D.Table_Size-7)
dataColor5 = GetColor('gray', !D.Table_Size-8)
;#
faxisColor = GetColor('aquamarine', !D.Table_Size-9)

   ; Check for keywords. Set defaults if necessary.

IF N_Elements(zxsize) EQ 0 THEN zxsize = 400
IF N_Elements(zysize) EQ 0 THEN zysize = 400
xlog = Keyword_Set(xlog)
ylog = Keyword_Set(ylog)
;#
IF N_Elements(y2) EQ 0 THEN y2 = mean(dep)
IF N_Elements(y3) EQ 0 THEN y3 = mean(dep)
IF N_Elements(y4) EQ 0 THEN y4 = mean(dep)
IF N_Elements(y5) EQ 0 THEN y5 = mean(dep)
IF N_Elements(y1title) EQ 0 THEN y1title='Data1'
IF N_Elements(y2title) EQ 0 THEN y2title='Data2'
IF N_Elements(y3title) EQ 0 THEN y3title='Data3'
IF N_Elements(y4title) EQ 0 THEN y4title='Data4'
IF N_Elements(y5title) EQ 0 THEN y5title='Data5'
;#
IF N_Elements(flag1) LE 1 THEN flag1 = make_array(n_elements(indep), /integer, value=-2)
IF N_Elements(flag2) EQ 0 THEN flag2 = -1
IF N_Elements(flag3) EQ 0 THEN flag3 = -1
IF N_Elements(flag4) EQ 0 THEN flag4 = -1
IF N_Elements(flag5) EQ 0 THEN flag5 = -1
IF N_Elements(f1title) EQ 0 THEN f1title='Flag1'
IF N_Elements(f2title) EQ 0 THEN f2title='Flag2'
IF N_Elements(f3title) EQ 0 THEN f3title='Flag3'
IF N_Elements(f4title) EQ 0 THEN f4title='Flag4'
IF N_Elements(f5title) EQ 0 THEN f5title='Flag5'

IF N_Elements(xoffset) EQ 0 THEN xoffset=0.0
IF N_Elements(yoffset) EQ 0 THEN yoffset=0.0

IF (Keyword_Set(psym) NE 1) THEN psym=0
line = 1 ; initialize as default
IF (Keyword_Set(filename) NE 1) THEN filename=''
IF (Keyword_Set(extbtn_mode) NE 1) THEN extbtn_mode=0 ; If extbtn_mode=1 then display extra buttons

; Create a top-level base for this program. No resizing of this base.
tlb = Widget_Base(Title='FISv Plot Window: '+filename, TLB_Size_Events=1, column=1, UNAME='PlotWin', $
                  XOffset=xoffset, YOffset=yoffset)

   ; The only thing in the top-level base is a draw widget.
   ; The draw widget will have its own event handler.

drawID = Widget_Draw(tlb, XSize=zxsize, YSize=zysize, $
   Button_Events=1, Event_Pro='ZfPLOT_PROCESS_EVENTS')

;#####Start of making control button base#####
; Create buttons
ctrlbase = Widget_Base(tlb, column=1)

DispBtnbase = Widget_Base(ctrlbase, row=1, /frame, xpad=0, ypad=0)

  chkecbox_base = Widget_Base(DispBtnbase, row=1, /NONEXCLUSIVE)
    SymbBtn = Widget_Button(chkecbox_base, VALUE='Symbol', UNAME='SymbBtn', Tooltip='Symbol On/Off')
    IF (psym EQ 0) THEN Widget_Control, SymbBtn, SET_BUTTON=0 ELSE Widget_Control, SymbBtn, /SET_BUTTON

    IF (extbtn_mode EQ 1) THEN BEGIN
      sybm_status = Widget_Info(SymbBtn, /BUTTON_SET)
      LineBtn = Widget_Button(chkecbox_base, VALUE='Line', UNAME='LineBtn', Tooltip='Line On/Off', $
                              Sensitive=sybm_status)
      Widget_Control, LineBtn, /SET_BUTTON
    ENDIF

    GridBtn = Widget_Button(chkecbox_base, VALUE='Grid', UNAME='GridBtn', Tooltip='Grid On/Off')

    dbly=0
;    IF (n_elements(y2) EQ n_elements(indep)) and (n_elements([y3,y4,y5]) EQ 3) THEN BEGIN
;      DblYBtn = Widget_Button(chkecbox_base, VALUE='Double Y', UNAME='DblYBtn', Tooltip='Double Y-axis On/Off')
;    ENDIF

  btn1base = Widget_Base(DispBtnbase, row=1)
    ResetBtn = Widget_Button(btn1base, xsize=40, value='Reset', uname='ResetBtn', Tooltip='Reset range')
    ASBtn = Widget_Button(btn1base, xsize=40, value='Auto', uname='ASBtn', Tooltip='Auto Scale')

    ZinBtn = Widget_Button(btn1base, xsize=30, value='ZoomIn.bmp', /bitmap, uname='ZinBtn', Tooltip='Zoom in')
    ZoutBtn = Widget_Button(btn1base, xsize=30, value='ZoomOut.bmp', /bitmap, uname='ZoutBtn', Tooltip='Zoom Out')

    IF (extbtn_mode EQ 1) THEN BEGIN
      xZinBtn = Widget_Button(btn1base, xsize=30, value='x+.bmp', /bitmap, uname='xZinBtn', Tooltip='X-axis Zoom in')
      xZoutBtn = Widget_Button(btn1base, xsize=30, value='x-.bmp', /bitmap, uname='xZoutBtn', Tooltip='X-axis Zoom Out')
      yZinBtn = Widget_Button(btn1base, xsize=30, value='y+.bmp', /bitmap, uname='yZinBtn', Tooltip='Y-axis Zoom in')
      yZoutBtn = Widget_Button(btn1base, xsize=30, value='y-.bmp', /bitmap, uname='yZoutBtn', Tooltip='Y-axis Zoom Out')
    ENDIF

    ScrLBtn = Widget_Button(btn1base, xsize=25, value='left_arr.bmp', /bitmap, uname='ScrLBtn', Tooltip='Scroll to Left')
    ScrRBtn = Widget_Button(btn1base, xsize=25, value='right_arr.bmp', /bitmap, uname='ScrRBtn', Tooltip='Scroll to Right')

    IF (extbtn_mode EQ 1) THEN BEGIN
      ScrUBtn = Widget_Button(btn1base, xsize=25, value='up_arr.bmp', /bitmap, uname='ScrUBtn', Tooltip='Scroll Up')
      ScrDBtn = Widget_Button(btn1base, xsize=25, value='down_arr.bmp', /bitmap, uname='ScrDBtn', Tooltip='Scroll Down')
    ENDIF

    IF (extbtn_mode EQ 0) THEN BEGIN
      ExtBtn = Widget_Button(btn1base, xsize=17, value='>>', uname='ExtBtn', Tooltip='Extra Mode')
    ENDIF ELSE BEGIN
      ExtBtn = Widget_Button(btn1base, xsize=17, value='<<', uname='ExtBtn', Tooltip='Simple Mode')
    ENDELSE

buttbase = Widget_Base(ctrlbase, /frame, row=1)
  PrintBtn = Widget_Button(buttbase, xsize=45, value='Print', uname='PrintBtn', Tooltip='Print Out')
  PSBtn = Widget_Button(buttbase, value='PS', xsize=45, uname='PSBtn', Tooltip='Save as a PostScript file')
  StatBtn = Widget_Button(buttbase, value='STAT', xsize=45, uname='StatBtn', Tooltip='Calculate statistics')

  ; If the called zplot_alt is used with TSD object.
  ; (Diff and FFT buttons can be used only when zplot_alt was called from FISv.)
  COMMON Share, tsd
  IF OBJ_VALID(tsd) EQ 1 THEN BEGIN
    DiffBtn = Widget_Button(buttbase, value='DIFF', xsize=45, uname='DiffBtn', Tooltip='Calculate Differentiation')
    FFTBtn = Widget_Button(buttbase, value='FFT', xsize=45, uname='FFTBtn', Tooltip='FFT')
  ENDIF

buttbase2 = Widget_Base(ctrlbase, row=1)
  ClsBtn = Widget_Button(buttbase2, value='Close', xsize=70, uname='ClsBtn', Tooltip='Colse window')

;#####End of making control button base#####

   ; Realize the program.

Widget_Control, tlb, /Realize

   ; Get the window index number of the draw widget.
   ; Make the draw widget the current graphics window
   ; and draw the plot of the data in it. Make the
   ; X data range exactly fit the data.

Widget_Control, drawID, Get_Value=drawIndex
WSet, drawIndex
IF N_Elements(xrange) EQ 0 THEN $
  xrange = [Min(indep), Max(indep)]
yrange = [Min([min(dep),min(y2),min(y3),min(y4),min(y5)]) $
           ,Max([max(dep),max(y2),max(y3),max(y4),max(y5)])]
xrange_def = xrange
yrange_def = yrange
y2range = [Min(y2),Max(y2)]

; Plot position
pos1 = [0.13,0.4,0.965,0.9] ; for normal data
pos2 = [0.13,0.1,0.965,0.4] ; for flag data

; Top side
;############;
; Flags plot ;
;############;
; Set y-range
f_yrange_max=2
IF n_elements(flag2) EQ n_elements(indep) THEN f_yrange_max=f_yrange_max+1.1
IF n_elements(flag3) EQ n_elements(indep) THEN f_yrange_max=f_yrange_max+1.1
IF n_elements(flag4) EQ n_elements(indep) THEN f_yrange_max=f_yrange_max+1.1
IF n_elements(flag5) EQ n_elements(indep) THEN f_yrange_max=f_yrange_max+1.1
f_yrange = [-1, f_yrange_max]

; Base lavel for the next flag plot
next_flag_base = 0 ; initial level

; Plot 1st flag
IF (n_elements(flag1) EQ n_elements(indep)) THEN BEGIN
  Plot, indep, flag1, position=pos2, ytickformat="(A1)", yrange=f_yrange, $
     Background=backColor, Color=faxisColor, /NoData, _Extra=extra
  Plot, indep, flag1, position=pos2, xstyle=4, ystyle=4, yrange=f_yrange, $
     thick=1, psym=(psym NE 0) ? -1:0, /noerase, Color=dataColor
  next_flag_base = next_flag_base + 1.1
ENDIF

; Standard position of legends
title_posF1a = CONVERT_COORD([0.85, 0.15], /NORMAL, /TO_DATA)  ; for symbol
title_posF1b = CONVERT_COORD([0.86, 0.15], /NORMAL, /TO_DEVICE); for label string
title_posF1b = CONVERT_COORD(title_posF1b[0],title_posF1b[1]-5, /DEVICE, /TO_DATA)

title_posF2a = CONVERT_COORD([0.85, 0.19], /NORMAL, /TO_DATA)  ; for symbol
title_posF2b = CONVERT_COORD([0.86, 0.19], /NORMAL, /TO_DEVICE); for label string
title_posF2b = CONVERT_COORD(title_posF2b[0],title_posF2b[1]-5, /DEVICE, /TO_DATA)

title_posF3a = CONVERT_COORD([0.85, 0.23], /NORMAL, /TO_DATA)  ; for symbol
title_posF3b = CONVERT_COORD([0.86, 0.23], /NORMAL, /TO_DEVICE); for label string
title_posF3b = CONVERT_COORD(title_posF3b[0],title_posF3b[1]-5, /DEVICE, /TO_DATA)

title_posF4a = CONVERT_COORD([0.85, 0.27], /NORMAL, /TO_DATA)  ; for symbol
title_posF4b = CONVERT_COORD([0.86, 0.27], /NORMAL, /TO_DEVICE); for label string
title_posF4b = CONVERT_COORD(title_posF4b[0],title_posF4b[1]-5, /DEVICE, /TO_DATA)

title_posF5a = CONVERT_COORD([0.85, 0.31], /NORMAL, /TO_DATA)  ; for symbol
title_posF5b = CONVERT_COORD([0.86, 0.31], /NORMAL, /TO_DEVICE); for label string
title_posF5b = CONVERT_COORD(title_posF5b[0],title_posF5b[1]-5, /DEVICE, /TO_DATA)

; Show the title of flag1
XYOUTS, title_posF1b[0], title_posF1b[1],  f1title, Color=dataColor

; Plot other flags
IF (n_elements(flag2) EQ n_elements(indep)) THEN BEGIN
  Plot, indep, flag2+next_flag_base,  position=pos2, xstyle=4, ystyle=4, yrange=f_yrange, $
    thick=1, psym=(psym NE 0) ? -1:0, /noerase, Color=dataColor2
  next_flag_base = next_flag_base + 1.1
  XYOUTS, title_posF2b[0], title_posF2b[1],  f2title, Color=dataColor2
ENDIF
IF (n_elements(flag3) EQ n_elements(indep)) THEN BEGIN
  Plot, indep, flag3+next_flag_base,  position=pos2, xstyle=4, ystyle=4, yrange=f_yrange, $
    thick=1, psym=(psym NE 0) ? -1:0, /noerase, Color=dataColor3
  next_flag_base = next_flag_base + 1.1
  XYOUTS, title_posF3b[0], title_posF3b[1],  f3title, Color=dataColor3
ENDIF
IF (n_elements(flag4) EQ n_elements(indep)) THEN BEGIN
  Plot, indep, flag4+next_flag_base,  position=pos2, xstyle=4, ystyle=4, yrange=f_yrange, $
    thick=1, psym=(psym NE 0) ? -1:0, /noerase, Color=dataColor4
  next_flag_base = next_flag_base + 1.1
  XYOUTS, title_posF4b[0], title_posF4b[1],  f4title, Color=dataColor4
ENDIF
IF (n_elements(flag5) EQ n_elements(indep)) THEN BEGIN
  Plot, indep, flag5+next_flag_base,  position=pos2, xstyle=4, ystyle=4, yrange=f_yrange, $
    thick=1, psym=(psym NE 0) ? -1:0, /noerase, Color=dataColor5
  next_flag_base = next_flag_base + 1.1
  XYOUTS, title_posF5b[0], title_posF5b[1],  f5title, Color=dataColor5
ENDIF

;############;
; Data Plot  ;
;############;
Plot, indep, dep, XRange=xrange, YRange=yrange, XLog=xlog, YLog=ylog, $
   Color=axisColor, /NoData, _Extra=extra, $
   position = pos1, xtickformat="(A1)", /noerase

; Standard position of legends
title_pos1a = CONVERT_COORD([0.85, 0.90], /NORMAL, /TO_DATA)  ; for symbol
title_pos1b = CONVERT_COORD([0.86, 0.90], /NORMAL, /TO_DEVICE); for label string
title_pos1b = CONVERT_COORD(title_pos1b[0],title_pos1b[1]-5, /DEVICE, /TO_DATA)

title_pos2a = CONVERT_COORD([0.85, 0.85], /NORMAL, /TO_DATA)  ; for symbol
title_pos2b = CONVERT_COORD([0.86, 0.85], /NORMAL, /TO_DEVICE); for label string
title_pos2b = CONVERT_COORD(title_pos2b[0],title_pos2b[1]-5, /DEVICE, /TO_DATA)

title_pos3a = CONVERT_COORD([0.85, 0.80], /NORMAL, /TO_DATA)  ; for symbol
title_pos3b = CONVERT_COORD([0.86, 0.80], /NORMAL, /TO_DEVICE); for label string
title_pos3b = CONVERT_COORD(title_pos3b[0],title_pos3b[1]-5, /DEVICE, /TO_DATA)

title_pos4a = CONVERT_COORD([0.85, 0.75], /NORMAL, /TO_DATA)  ; for symbol
title_pos4b = CONVERT_COORD([0.86, 0.75], /NORMAL, /TO_DEVICE); for label string
title_pos4b = CONVERT_COORD(title_pos4b[0],title_pos4b[1]-5, /DEVICE, /TO_DATA)

title_pos5a = CONVERT_COORD([0.85, 0.70], /NORMAL, /TO_DATA)  ; for symbol
title_pos5b = CONVERT_COORD([0.86, 0.70], /NORMAL, /TO_DEVICE); for label string
title_pos5b = CONVERT_COORD(title_pos5b[0],title_pos5b[1]-5, /DEVICE, /TO_DATA)

OPlot, indep, dep, Color=dataColor, psym=psym
XYOUTS, title_pos1b[0], title_pos1b[1],  y1title, Color=dataColor
Oplot, [title_pos1a[0]], [title_pos1a[1]], Color=dataColor, psym = (psym NE 0) ? psym:0

;#
IF n_elements(y2) EQ n_elements(indep) THEN BEGIN
    OPlot, indep, y2, Color=dataColor2, psym=(psym NE 0) ? -1:0
    XYOUTS, title_pos2b[0], title_pos2b[1], y2title, Color=dataColor2
    Oplot, [title_pos2a[0]], [title_pos2a[1]], Color=dataColor2, psym = (psym NE 0) ? 1:0
ENDIF
IF n_elements(y3) EQ n_elements(indep) THEN BEGIN
    OPlot, indep, y3, Color=dataColor3, psym=(psym NE 0) ? -4:0
    XYOUTS, title_pos3b[0], title_pos3b[1], y3title, Color=dataColor3
    Oplot, [title_pos3a[0]], [title_pos3a[1]], Color=dataColor3, psym = (psym NE 0) ? 4:0
ENDIF
IF n_elements(y4) EQ n_elements(indep) THEN BEGIN
    OPlot, indep, y4, Color=dataColor4, psym=(psym NE 0) ? -2:0
    XYOUTS, title_pos4b[0], title_pos4b[1], y4title, Color=dataColor4
    Oplot, [title_pos4a[0]], [title_pos4a[1]], Color=dataColor4, psym = (psym NE 0) ? 2:0
ENDIF
IF n_elements(y5) EQ n_elements(indep) THEN BEGIN
    OPlot, indep, y5, Color=dataColor5, psym=(psym NE 0) ? -6:0
    XYOUTS, title_pos5b[0], title_pos5b[1], y5title, Color=dataColor5
    Oplot, [title_pos5a[0]], [title_pos5a[1]], Color=dataColor5, psym = (psym NE 0) ? 6:0
ENDIF

; Back side
  ; Create a pixmap window the same size as the draw widget window.
  ; Store its window index number in a local variable. Draw the same
  ; plot you just put in the draw widget in the pixmap window.

Window, /Free, XSize=zxsize, YSize=zysize, /Pixmap
pixIndex = !D.Window

; Data plot
Plot, indep, dep, XRange=xrange, YRange=yrange, XLog=xlog, YLog=ylog, $
   Color=axisColor, /NoData, _Extra=extra, $
   position = pos1
OPlot, indep, dep, Color=dataColor, psym=psym
XYOUTS, title_pos1b[0], title_pos1b[1],  y1title, Color=dataColor
Oplot, [title_pos1a[0]], [title_pos1a[1]], Color=dataColor, psym = (psym NE 0) ? psym:0

;#
IF n_elements(y2) EQ n_elements(indep) THEN BEGIN
    OPlot, indep, y2, Color=dataColor2, psym=(psym NE 0) ? -1:0
    XYOUTS, title_pos2b[0], title_pos2b[1], y2title, Color=dataColor2
    Oplot, [title_pos2a[0]], [title_pos2a[1]], Color=dataColor2, psym = (psym NE 0) ? 1:0
ENDIF
IF n_elements(y3) EQ n_elements(indep) THEN BEGIN
    OPlot, indep, y3, Color=dataColor3, psym=(psym NE 0) ? -4:0
    XYOUTS, title_pos3b[0], title_pos3b[1], y3title, Color=dataColor3
    Oplot, [title_pos3a[0]], [title_pos3a[1]], Color=dataColor3, psym = (psym NE 0) ? 4:0
ENDIF
IF n_elements(y4) EQ n_elements(indep) THEN BEGIN
    OPlot, indep, y4, Color=dataColor4, psym=(psym NE 0) ? -2:0
    XYOUTS, title_pos4b[0], title_pos4b[1], y4title, Color=dataColor4
    Oplot, [title_pos4a[0]], [title_pos4a[1]], Color=dataColor4, psym = (psym NE 0) ? 2:0
ENDIF
IF n_elements(y5) EQ n_elements(indep) THEN BEGIN
    OPlot, indep, y5, Color=dataColor5, psym=(psym NE 0) ? -6:0
    XYOUTS, title_pos5b[0], title_pos5b[1], y5title, Color=dataColor5
    Oplot, [title_pos5a[0]], [title_pos5a[1]], Color=dataColor5, psym = (psym NE 0) ? 6:0
ENDIF


IF N_Elements(extra) EQ 0 THEN extraKeywords = Ptr_New(/Allocate_Heap) ELSE extraKeywords = Ptr_New(extra)

   ; Create an info structure to hold information required by the program.

info = { $
   dep:dep, $                       ; The dependent data to be plotted.
   dep2:y2, $                       ;# Added member (2003/04/15)
   dep3:y3, $                       ;#
   dep4:y4, $                       ;#
   dep5:y5, $                       ;#
   y1title:y1title, $               ;#
   y2title:y2title, $               ;#
   y3title:y3title, $               ;#
   y4title:y4title, $               ;#
   y5title:y5title, $               ;#
   flag1:flag1, $                   ;# Added member (2004/02/20)
   flag2:flag2, $                   ;#
   flag3:flag3, $                   ;#
   flag4:flag4, $                   ;#
   flag5:flag5, $                   ;#
   f1title:f1title, $               ;#
   f2title:f2title, $               ;#
   f3title:f3title, $               ;#
   f4title:f4title, $               ;#
   f5title:f5title, $               ;#
   indep:indep, $                   ; The independent data to be plotted.
   zxsize:zxsize, $                 ; The X size of the draw widget.
   zysize:zysize, $                 ; The Y size of the draw widget.
   drawID:drawID, $                 ; The widget identifier of the draw widget.
   drawIndex:drawIndex, $           ; The draw window index number.
   pixIndex:pixIndex, $             ; The pixmap window index number.
   xrange:xrange, $                 ; The current X range of the plot.
   yrange:yrange, $                 ; The current Y range of the plot.
   xrange_def:xrange_def, $         ;# default X range (2003/05/27)
   yrange_def:yrange_def, $         ;# default Y range (2003/04/24)
   xlog:xlog, $                     ; A flag for X logarithmic axis style.
   ylog:ylog, $                     ; A flag for Y logarithmic axis style.
   xstyle:0, $                      ; The setting for the XStyle keyword.
   ystyle:0, $                      ;# Added member (2003/04/24)
   ticklen:0.02, $                  ;# (2003/05/27)
   ;xmargin:xmargin, $               ;# (2004/04/20)
   xs:0d, $                         ; X static corner of the zoom box.
   ys:0d, $                         ; Y static corner of the zoom box.
   xd:0, $                          ; X dynamic corner of the zoom box.
   yd:0, $                          ; Y dynamic corner of the zoom box.
   x:!X, $                          ; The !X system variable after plot.
   y:!Y, $                          ; The !Y system variable after plot.
   p:!P, $                          ; The !P system variable after plot.
   psym:psym, $                     ;# Added member (2003/03/27)
   line:line, $                     ;# (2005/03/18)
   dbly:dbly, $                     ;# (2003/04/22)
   filename:filename, $             ;# (2003/06/05)
   extbtn_mode:extbtn_mode, $       ;# To display extra buttons (2005/03/17)
   extraKeywords:extraKeywords }    ; Extra plot keywords pointer.

   ; Store the info structure in the user value of the top-level base.

Widget_Control, tlb, Set_UValue=info, /No_Copy

   ; Register this program and set up the event loop.

XManager, 'ZPLOT_FLAG', tlb, Cleanup='ZfPLOT_Cleanup', Group_Leader=group, /No_Block, $
   Event_Handler='ZfPLOT_Resize'
END ; of ZPLOT ****************************************************************************
