;#> frin_select.dc3
; Identifier	frin_select
;
; Purpose	select data from struct according to fringe-id
;
; Synopsis	result = frin_select( struct, frid, wran=wran )
;
; Arguments	Name	I/O    Type:	Description
;		------------------------------------------------------------
;		struct	 I     struct	SPD or AAR or CAL25
;		frid	 I     struct	fringe-id (see fringes.pro)
;		wran	 I     fltarr	wavelength range.
;					default: automatic
;
; Returns	result	 O     struct
;		  { w1	       fltarr	wavelengths
;		    x1	       fltarr	fluxes
;		    r1	       intarr	indices to restore
;		    n1	       int }	length
;
; Description	selection is done on det (or band), aperture, order,
;		up/down and wavelength range.
;
;
;
; Comment
;
;
; Example	--
;
; Category	UTIL ( module: fringes )
;
; Filename	frin_select.pro
;
; Author	Do Kester
;
; Version	1.3
;
; History	1.0  07-03-2001 DK, first implementation of frin_select.pro
;		1.1  05-04-2001 DK, remove compile_opt
;		1.2  16-05-2001 DK, option to select for more than one detector
;               1.3  19-02-2003 FL, enable a non-SWS aar to be processed
;#<
;===========================================================================

function frin_select, struct, frid, wran=wran

; compile_opt idl2, hidden

forward_function isspd, isaar, apband_to_aot, append

if ( n_elements( wran ) NE 2 ) then begin

; set (coarse) wavelength ranges per aotnumber
;	       1a   1b	 1d   1e  2a  2b  2c  3a  3c  3d  3e   4  4a  4c  4d
  wrlo = [ 0, 2.2, 2.5, 2.8, 3.4, 3.8, 5,  6, 11, 15, 18, 27, 28, 11, 15, 18 ]
  wrhi = [ 0, 2.7, 3.2, 3.7, 4.5, 5.6, 8, 13, 17, 21, 29, 30, 46, 17, 21, 28 ]
  aotname = [ '1A', '1B', '1D', '1E', '2A', '2B', '2C', $
	      '3A', '3C', '3D', '3E',  '4', '4A', '4C', '4D' ]

  band = ( frid.det[0] LT 51 ) ? ( frid.det[0] - 1 ) / 12 + 1 : 6
  aot = apband_to_aot( band, frid.aper, frid.order, /silent )
  aotnr = where( aotname EQ aot ) + 1

  wrange = [ wrlo[aotnr], wrhi[aotnr] ]
endif else wrange = wran

isspd = is_spd( struct )
isaar = is_aar( struct )
instrument = ''
stat = read_fits_key( struct.header, 'INSTRUME', instrument, comm )
instrument = strupcase(strtrim(instrument,2))

case frid.updown of
  ''   : begin
	   count_bud = n_elements( struct.data )
	   bud = lindgen( count_bud )
	 end
  ' u' : bud = where( test_status( struct, /swdown ) EQ 1, count_bud )
  ' d' : bud = where( test_status( struct, /swdown ) EQ 0, count_bud )
endcase

if ( count_bud EQ 0 ) then return, 0

nn = 0
orderflag = frid.order * 32L
ndet = n_elements( frid.det )

for kd = 0, ndet - 1 do begin
  det  = frid.det[kd]
  sdet = det - 1

  if ( isspd ) then begin					; SPD
    r1 = where( ( struct.data[bud].status AND 3L ) EQ frid.aper AND $
		( struct.data[bud].det[sdet].flag AND 224L ) EQ orderflag AND $
		( struct.data[bud].det[sdet].flag AND frid.fmask ) EQ 0 AND $
		  struct.data[bud].det[sdet].wave GT wrange[0] AND $
		  struct.data[bud].det[sdet].wave LT wrange[1], n1 )
    if ( n1 EQ 0 ) then goto, end_loop

    r1 = bud[r1]

; select the data: wavelengths and fluxes
    w1 = struct.data[r1].det[sdet].wave
    x1 = struct.data[r1].det[sdet].flux
    d1 = intarr( n1 ) + sdet

  endif else if ( isaar ) then begin				; AAR
    IF instrument EQ 'SWS' THEN BEGIN
      r1 = where( ( struct.data[bud].status AND 3L ) EQ frid.aper AND $
		  ( struct.data[bud].flag AND 224L ) EQ orderflag AND $
		  ( struct.data[bud].flag AND frid.fmask ) EQ 0 AND $
		    struct.data[bud].det	EQ det AND $
		    struct.data[bud].wave GT wrange[0] AND $
		    struct.data[bud].wave LT wrange[1], n1 )
    ENDIF ELSE BEGIN
      r1 = where( struct.data[bud].det        EQ det AND $
	          ( struct.data[bud].flag AND frid.fmask ) EQ 0 AND $
                  struct.data[bud].wave GT wrange[0] AND $
                  struct.data[bud].wave LT wrange[1], n1 )
    ENDELSE
    if ( n1 EQ 0 ) then goto, end_loop

    r1 = bud[r1]

; select the data: wavelengths and fluxes
    w1 = struct.data[r1].wave
    x1 = struct.data[r1].flux
    d1 = intarr( n1 ) + sdet

  endif else begin						; CAL25
    bdet = 3 * ( sdet MOD 12 )
    r1 = where( struct.data.(bdet) GE wrange[0] AND $
		struct.data.(bdet) LE wrange[1], n1 )
    if ( n1 EQ 0 ) then goto, end_loop

    w1 = struct.data[r1].(bdet)
    x1 = struct.data[r1].(bdet+1)
    d1 = intarr( n1 ) + bdet + 1
  endelse

  ww = append( ww, w1 )
  xx = append( xx, x1 )
  rr = append( rr, r1 )
  dd = append( dd, d1 )
  nn = nn + n1

  end_loop :
endfor

if ( nn EQ 0 ) then return, 0

select = { w1 : ww, $
	   x1 : xx, $
	   r1 : rr, $
	   d1 : dd, $
	   n1 : nn }

return, select
end
