;#> frin_smooth.dc3
; Identifier	frin_smooth
;
; Purpose	make a smooth and clipped version of the flux
;
; Synopsis	result = frin_smooth( w1, x1, midcycle, mask=mask, $
;                                     suppress=suppress, supwid=supwid
;                                     box=box, plot=plot )
;
; Arguments	Name	I/O    Type:	Description
;		------------------------------------------------------------
;		w1	 I     fltarr	wavelengths
;		x1	 I     fltarr	fluxes
;		midcycle I     float	mid of fringe search range
;		mask	 O     fltarr	mask
;               suppress I     fltarr   set of wavelengths to disregard during
;                                       fringe fitting (eg. at emission lines)
;               supwid   I     float    width of the suppression
;                                       default : 0.01
;		box	 I     int	smooth box. default=51
;		plot	 I     keyword	do some plotting
;
; Returns	result	 O     fltarr	smoothed background
;
; Description
;
;
;
;
; Comment
;
;
; Example	--
;
; Category	UTIL ( module: fringes )
;
; Filename	frin_smooth.pro
;
; Author	Do Kester
;
; Version	1.6
;
; History	1.0  07-03-2001 DK, first implementation of frin_smooth.pro
;		1.1  22-03-2001 DK, first subtract straight line
;		1.2  10-05-2001 DK, mainly stability issues
;		1.3  21-06-2001 FL, return mask as output, more carefull
;				    smoothing, clipping and polynomial fitting
;		1.4  27-09-2001 DK, use running stdev for obtaining mask
;               1.5  22-11-2001 DK, smooth needs box < length; stupid IDL
;               1.6  05-06-2003 FL, suppress defined lines in smoothing
;#<
;===========================================================================

function frin_smooth, w0, x0, midcycle, mask=mask, box=box_in, plot=plot, $
	              suppress=suppress, supwid=supwid

; compile_opt idl2, hidden

nx = n_elements( x0 )
if ( nx LE 3 ) then return, x0

; if ( n_elements( box ) EQ 0 ) then box = 51
; box = box < ( ( nx - 1 ) / 12 )

nc = midcycle * ( max(1./w0) - min(1./w0) )

srt = sort( w0 )
w1 = w0[srt]
x1 = x0[srt]

dw = ( mean( w1[1,*]-w1[0,*] ) )[0]
box = long( (max(w1)-min(w1))/dw / nc ) > long( nx / nc )
box = 1 + 2*(box/2)

if n_elements(box_in) gt 0 then begin
	if box_in gt 1 then box = box_in
endif

;print,midcycle,nx,nc,box

ss = fltarr( nx )

pos = [ 0.1, 0.4, 0.9, 0.9 ]

for kk = 1, 2 do begin

; make smoothed and clipped version of flux (x1) in s1
  s0 = x1
  ws = 70

  mask = fltarr( nx ) + 1.0
  ns = n_elements(suppress)
  if ns gt 0 then begin
    case n_elements(supwid) of
     0 : sw = replicate(0.01,ns)
     1 : sw = replicate(supwid[0],ns)
     else : begin
	     if n_elements(suppress) eq ns then $
	          sw = supwid $
	     else sw = replicate(supwid[0],ns)
	  end
    endcase
  endif
  for s=0, ns-1 do begin
    mask = temporary(mask) and $
           ( w1 lt suppress[s]-0.5*sw[s] or $
             w1 gt suppress[s]+0.5*sw[s] )
  endfor

  for kz = 0, 1 do begin
;
; FL: in the first iteration do first a median then a smooth to
;     nicely eliminate strong spectral features which will be
;     clipped out for the second iteration. This greatly improves
;     the masking of 'real' spectral features
;
    n = ( box < n_elements(mask)/5 > 5)
;print,n
    if ( kz eq 0 ) then begin
      sm = smooth( float( mask ), n, /edge_truncate )
      s0 = smooth( median( x1 * mask, n ), n, /edge_truncate ) / $
	   ( sm > 0.01 )
    endif else begin
      sm = smooth( float( mask ) , n, /edge_truncate )
      s0 = smooth( smooth( x1 * mask, n, /edge_truncate ) / ( sm > 0.01 ), 5 )
    endelse

; standard deviation of (masked) residual
    n = 10*box < n_elements(mask)/10
;print,n
    ws = 4.0 * smooth( abs( mask*(x1-s0) ), n < nx-1, /edge_truncate ) / $
	       smooth( mask, 10*box < nx-1, /edge_truncate )

    mask = float( finite( s0 ) AND abs( x1 - s0 ) LT ws )

; clip extra points around extremes
    for ms =1,3 do $
      mask = mask AND shift( mask, ms ) AND shift( mask, -ms )

  endfor

  if ( total(mask) EQ 0. ) then mask[*] = 1.0

; extract straight line first (to get the edges proper)
;
; FL: use poly_fit instead of polyfitw for more control in
;     case of failed/bad polynomial fits
; FL: combine points with the same wavelength before fitting
;     a polynomial instead of modifying weights

  b = where( mask GT 0 )
  w2 = w1[b] & x2 = x1[b]
  wave_combine,w2,x2,/mean

  ppar = poly_fit( w2, x2, 1, /double, status=status )
  if ( status GT 0 ) then begin
    error,'W','bad polynomial fit, will use median level'
    xfit = replicate( median(x1), nx )
  endif else xfit = poly( w1, ppar )

  if ( keyword_set( plot ) ) then begin
    winset, 3
    minx = min( x1 ) < 0
    medx = median( x1 ) * 0.1
    yr = cliprange( x1, [0.01,0.99] )
    yr[0] = yr[0] < 0
    plot, w1, x1, yrange=yr, xst=3, yst=3, position=pos, noerase=(kk EQ 2)
    if ( kk EQ 1 ) then oplot, w1, mask * medx, ps=-3
; oplot,w1, !y.crange[1] * mask / max(mask), ps=0, thick=2, col=3
    oplot, w1, xfit, color=3
    oplot, w1, s0 + ws, ps=3, color=2
    oplot, w1, s0 - ws, ps=3, color=2

    pos = [ 0.1, 0.1, 0.9, 0.4 ]
  endif

  s1 = x1 - xfit
  s0 = s0 - xfit
  cran = 1.0 / midcycle
;  mask = float( mask ) > 0.1			; avoid zero's
  for kz = 0, 4 do begin
    ws = stdev( s0 - s1 ) * 3
    s0 = s1 < ( s0 + ws ) > ( s0 - ws )
    tt = fltarr( nx )
    for n = 0L, nx - 1 do begin
      wlo = w1[n] / ( 1 + cran * w1[n] )
      whi = w1[n] / ( 1 - cran * w1[n] )
      bw = where( w1 GT wlo AND w1 LT whi, nw )
      if ( nw GT 0 ) then begin
	tm = total( mask[bw] )
	tt[n] = tm
	if ( tm GT 0 ) then s1[n] = total( s0[bw] * mask[bw] ) / tm $
	else s1[n] = 0.0
      endif
    endfor

    mtt = mean( tt )

    if ( keyword_set( plot ) ) then begin
      oplot, w1, s1+xfit, color=kz+4
      oplot, w1, tt, color=5
    endif

    if ( mtt LT  4 ) then cran = 4.0 / midcycle

  endfor

  ss = ss + s1 + xfit
  x1 = x1 - ss
endfor

ss = ss[sort( srt )]					; unsort
mask = mask[sort( srt )]				; FL, unsort mask output

return, ss
end
