; $Id: gb_get_readout_mode.pro,v 1.2 2006/01/16 00:01:23 baba Exp $
;
; Copyright (c) 2002, ASTRO-F Data Reduction Team.  All rights reserved.
;
;+
; NAME:
;	gb_get_readout_mode.pro
;
; PURPOSE:
;       This function returns the values according to the packetid.
;       mode: readout mode
;             0: CDS mode
;             1: simple sampling mode
;           > 1: co-add mode with number of accumlated samplings
;           < 0: undefined packet ID
;
; CATEGORY:
;	Function. Green Box library in ASTRO-F pipeline.
;
; CALLING SEQUENCE:
;	Result = gb_get_readout_mode (packetid)
;
; INPUTS:
;
; OUTPUTS:
;
; RETURN:
;       This function returns the values according to the packetid.
;       mode: readout mode
;             0: CDS mode
;             1: simple sampling mode
;           > 1: co-add mode with number of accumlated samplings
;           < 0: undefined packet ID
;
; RESTRICTIONS:
;	Currently, readout modes and corresponding packetid are
;           0x40 and 0x50    simple
;           0x41 and 0x51    co-added
;           0x43 and 0x53    CDS
;
;       This module uses IDL default library "WHERE".
; PROCEDURE:
;
; EXAMPLE:
;       The following process will returen the values
;       for 'infile.fits'.
;
;       tsd=obj_new('tsd')
;       ret = tsd->read_tsd('infile.fits')
;       packetid = tsd->get_value_tsd('fis_obs','packetid')
;       mode = gb_get_readout_mode(packetid,count)
;       
; REFERENCE:
;	Baba et al. 2002 Nov. 21, ``ASTRO-F/FIS Survey Data Structure"
;
; MODIFICATION HISTORY:
;       Nov. 2004    IY Written.
;       Nov. 2004    AK Converted to GB library format
;-

FUNCTION gb_get_readout_mode, PacketID
; set Readout mode (simple / CDS / co-add) by packet ID
; input
;  PacketID: array of packet ID
; output
;  mode: readout mode
;     0: CDS mode
;     1: simple sampling mode
;   > 1: co-add mode with number of accumlated samplings
;   < 0: undefined packet ID

   mode = replicate(-1, n_elements(packetid))      ; integer

   idx = where ( packetid EQ '40'X OR packetid EQ '50'X, cnt)
   if (cnt GT 0) then mode[idx] = 1

   idx = where ( packetid EQ '43'X OR packetid EQ '53'X, cnt)
   if (cnt GT 0) then mode[idx] = 0

   idx = where ( packetid EQ '41'X OR packetid EQ '51'X OR $
                 packetid EQ '42'X OR packetid EQ '52'X, cnt)
   if (cnt GT 0) then mode[idx] = 6

return, mode
end



