; $Id: gb_dark_subtraction.pro,v 1.28 2006/10/05 04:22:04 makiuti Exp $
;
; Copyright (c) 2002-2006, ASTRO-F Data Reduction Team.  All rights reserved.
;
;+
;FUNCTION NAME:
;       GB_Dark_Subtraction.pro
;
;PURPOSE:
;       The dark current level is estimated and subtracted by referring
;       to dark data in the file and to the dark table previously derived
;       by the procedure, "GB_Make_Dark_Table."
;
;CATEGORY:
;       Function. Green Box procedure in ASTRO-F pipeline.
;
;CALLING SEQUENCE:
;       Result = GB_Dark_Subtraction(tsd)
;
;INPUTS:
;       tsd             TSD object reference.
;       dark_table      Dark table made in the previous procedure,
;                       It is automatically searched.
;
;OUTPUTS:
;       tsd             TSD object reference. Dark-sbtracted TSD.
;
;RETURN:
;       !SUCCEED if function ended successfully.
;       !FAIL in failure.
;
;RESTRICTIONS:
;       NONE
;
;PROCEDURE:
;       1. Extract detector data from TSD.
;       2. Read various status and construct dark data set.
;       3. Search dark table and read dark data corresponding to both ends
;          of the dark data.
;       4. Check validity of the dark data.
;       5. Using exponential curve, transient effect is roughly corrected.
;       6. Linearly interpolate the transient-corrected dark data and
;          subtract these values from the input data.
;       7. Write a history in TSD header.
;       8. Output TSD object reference.
;       9. Write a summary of this function.
;
;AUXILIARY PROCEDURE:
;       GB_DARK_FIT.PRO - used for approximate correction of transient effect
;
;EXAMPLE:
;
;REFERENCE:
;       Baba et al., ``ASTRO-F/FIS Survey Data Structure"
;
;MODIFICATION HISTORY:
;       Oct. 2002    Written by A. Kawamura.
;       Jun. 2004    modified by YS (correction of transient effect,
;                    and many error handlings)
;       Oct. 2006    Last modified by Makiuti
;
;-

FUNCTION GB_Dark_Subtraction, tsd, DISPLAY = display
; ON_ERROR, 2

  ;;; Starting GB_Dark_Subtraction ;;;
  LMESSAGE, 5, 'Executing GB_Dark_Subtraction...', /CONTINUE

  ;;; Preliminary setting that need to be modified when dealing with REAL data
  fldiv=1.D5  ;  seconds, table file is generated at this rate
;  idura=5     ;  in sampling unit, duration of pulse cal (cal A)
;  idurb=21    ;  in sampling unit, duration of cal B
  dk_int=60.D ;  seconds, interval between two dark measurements just before
               ;  and after cal.  Two data set within this time interval will
               ;  be averaged.

  ;;; Extract values from TSD object reference ;;;
  in_flux = tsd->Get_Value_TSD('FIS_OBS','Flux')
  ret     = af_finite(in_flux, flux)
  verr    = tsd->Get_Value_TSD('FIS_OBS','FErr')
  ti      = tsd->Get_Value_TSD('FIS_OBS','AFtime')
  ; number of channels ;
    si = SIZE(flux)
    num_ch = si[1]
    num_data = si[2]
      corr_flux=flux

  ;;; Define duration of lighting by CalA for pulse(short) Cal.
  ;   ## This values need to be optimized. ##
  IF (num_ch EQ 75) THEN BEGIN ;; LW
    calAtime = 0.5     ; (05/11/26 since IT#4)
    idura = fix(calAtime*16.86)+1 ; Sampling rate is 16.86Hz in Survey mode
  ENDIF ELSE IF (num_ch EQ 100) THEN BEGIN ;; SW
    ;calAtime = 0.2     ; Old default
    ;calAtime = 0.1399  ; Crossing time of point source in survey obs.
    calAtime = 0.16    ; (05/11/26 since IT#4)
    idura = fix(calAtime*25.28)+1 ; Sampling rate is 25.28Hz in Survey mode
  ENDIF

  ; Make an array for the 'No_dk_corr' flag
  no_dk_flg = bytarr(num_ch, num_data)

  ;;; Read status and check if data are valid ;;;
  op_shutter = tsd->Get_Status_TSD('FIS_OBS','ShtOp')  ; 1 for open shutter
  cl_shutter = 1 - op_shutter

  pid     = tsd->Get_Value_TSD('FIS_OBS','PacketID')

  IF (num_ch EQ 75) THEN BEGIN
    on_calA = tsd->Get_Status_TSD('FIS_OBS','CalALon')
    calA_pls_cnt = tsd->Get_Value_TSD('FIS_OBS','CalPlsCntAL')
    on_boost = tsd->Get_Status_TSD('FIS_OBS','LWBoostOn')
    of_boost = 1 - on_boost
  ENDIF ELSE BEGIN
    on_calA = tsd->Get_Status_TSD('FIS_OBS','CalASon')
    calA_pls_cnt = tsd->Get_Value_TSD('FIS_OBS','CalPlsCntAS')
    on_boost = tsd->Get_Status_TSD('FIS_OBS','SWBoostOn')
    of_boost = 1 - on_boost
  ENDELSE

  on_calB = tsd->get_status_tsd('FIS_OBS','CalBOn')
  of_calB = 1 - on_calB
  calB_pls_cnt = tsd->get_value_tsd('FIS_OBS','CalPlsCntB')


;;-----------------------
;; Get quality indicators
;;-----------------------
  qual_dk_data = tsd->Get_Quality_TSD('FIS_OBS','Qual_dk_data')
  qual_dk_param = tsd->Get_Quality_TSD('FIS_OBS','Qual_dk_param')
  qual_dk_table = tsd->Get_Quality_TSD('FIS_OBS','Qual_dk_table')

  ;;-----------------------
  ;; Correct status bit of 'CalAS(L)on' using 'CalAPlsCnt' and duration
  ;;   - Status of pulse (short) Cal is to be set again because the timing
  ;;     is supposed to be different from the real data.
  ;;   - Status of continuous Cal is not changed.
  ;;   - Pulse Cal is regarded to be done during shutter open, while continuous Cal
  ;;     is done during shutter close.
  ;;-----------------------

  ; Find out positions of frames at which CalA turned on
  idum = where(calA_pls_cnt - shift(calA_pls_cnt, 1) NE 0, cnt)

  IF (cnt GE 1) THEN BEGIN ; If there are pulse CalA signals

    IF (on_calA[0]*op_shutter[0] EQ 1) THEN BEGIN ; If data begins during A Cal pulse On.
      dum = where(on_calA[0:idura-1] EQ 1, cntp)
      on_calA[0:cntp-1] = 1B
      on_calA[cntp:num_data-1L] = 0B
    ENDIF ELSE BEGIN
      tmp_idx = where(on_calA*op_shutter EQ 1, cnt_tmp) ; CalA ON but not continuous Cal
      IF (cnt_tmp GT 0) THEN on_calA[tmp_idx] = 0B
    ENDELSE

    ist=0 & IF (idum[0] EQ 0) THEN ist=1
    FOR i = ist, cnt-1 DO BEGIN

      ;ib = idum[i] ; until IT#4
      IF (num_ch EQ 100) THEN BEGIN
        ib = idum[i]+3 ; new for SW
        IF (ib GE num_data) THEN continue
      ENDIF ELSE BEGIN
        ib = idum[i] ; new for LW
      ENDELSE

      IF (cl_shutter[ib] EQ 1) THEN continue ; If not pulse but continuous
      ;ie = min([num_data, ib+idura-1L]) ; until IT#4
      ie = min([num_data-1, ib+idura]) ; new
      on_calA[ib:ie] = 1B
    ENDFOR

  ENDIF ELSE BEGIN
    LMESSAGE, 7, 'No pulse Cal in the data', /CONTINUE
;    return, !SUCCEED
  ENDELSE

  of_calA = 1 - on_calA

  ;;; to discard dark data taken at transition period to continous (B) cal.,
  ;;; we superficially extend continuous (B) cal.
  ;
  idx_c=where(on_calB EQ 1, cnt)
  idx_b=where(idx_c - shift(idx_c, 1) NE  1)
  idx_e=where(idx_c - shift(idx_c,-1) NE -1)
  on_calB[idx_c[idx_b]-1] = 1   &  on_calB[idx_c[idx_b]-2] = 1
  on_calB[idx_c[idx_e]+1] = 1   &  on_calB[idx_c[idx_e]+2] = 1

  of_calB = 1 - on_calB

  st_bad   = tsd->Get_Pix_Flag_TSD('FIS_OBS','Bad')
  no_bad   = 1 - st_bad

  ; probably we need more to check, e.g., bad pix and ....
  st_blank = tsd->Get_Flag_TSD('FIS_OBS','Blank')
  no_blank = 1 - st_blank
  in_in_saa= tsd->Get_Flag_TSD('FIS_OBS','In_SAA')
  no_in_saa= 1 - in_in_saa


;;----------------------------------------------------------
;; Read dark table files
;;----------------------------------------------------------

  ;;-----------------------
  ;; Identify table files to be read
  ;;-----------------------

  ;; If there are any file to supplement data before and after the time
  ;    range in the current TSD file, those are also read.
  flnm = intarr(4)
  timin = min(ti, max=timax) ; ti is AF-time

  ; Find files ; Up to 50000sec before the current TSD file
  flnm[0] = fix(timin/fldiv)
  IF fix(timin/fldiv) NE fix((timin-50000.D)/fldiv) THEN BEGIN
    flnm[1] = fix((timin-50000.D)/fldiv)
  ENDIF

  ; Find files : Up to 50000sec after the current TSD file
  flnm[2]=fix(timax/fldiv)
  IF fix(timax/fldiv) NE fix((timax+50000.D)/fldiv) THEN BEGIN
    flnm[3] = fix((timax+50000.D)/fldiv)
  ENDIF

  ;; Find files : For the current file
  ; Assortment (Find ID numbers for dark files)
  tidn = fix(ti[21, *]/fldiv) ; ti is in AF-time; fldiv is an interval of the file.
                              ; tidn is the ID number of dark files to be read.
  idx_idc = where(tidn - shift(tidn, 1) NE 0, cntds) ; index# of changing points of tidn
                                            ; cntds is the number of changing points of tidn

  IF (cntds EQ 0) THEN BEGIN ; If all data in the current file are to be in one file,
    cntds = 1                ;   only one file will be read.
    idx_idc = [0, N_elements(tidn)]
  ENDIF

  FOR i = 0, cntds-1 DO BEGIN
    fileID = tidn[idx_idc[i]]
  ENDFOR

  ;; Sort and uniq fileID numbers
  fileID = [flnm, fileID]
  fileID = fileID[uniq(fileID, sort(fileID))] ; File ID numbers (including 0)

  ;;-----------------------
  ;; Read table files
  ;;-----------------------
  dmy_flg=0
  ;; Select a band name
  IF (num_ch EQ 75)  THEN bnd='_LW_'
  IF (num_ch EQ 100) THEN bnd='_SW_'

  FOR i = 0, n_elements(fileID)-1 DO BEGIN

    ;; In case that the data of +-50000sec are in the same table file as others.
    ;; i.e. if flnm[1](flnm[3])=0
    IF (fileID[i] EQ 0) THEN Continue

    dk_file = './DARKTABLE/DARK'+bnd+string(fileID[i],format='(i4.4)')+'.fits'
    ret = file_test(dk_file)

    IF (ret EQ 0) THEN BEGIN ; If the file does not exist
      LMessage, 5, 'Info: dark table ('+dk_file+') does not exist. ', /Continue
      continue
    ENDIF

    IF (ret NE 0) THEN BEGIN ; If the file does exist
      LMessage, 5, dk_file+' is read.', /Continue
      dk = MRDFITS(dk_file, 1, STATUS=status)

      ; Omit the data which are apart from the time of the current file more than 50000sec.
      IF (status GE 0) THEN   $
        ti_idx = where((dk.aftime GT timin-50000.D) AND (dk.aftime LT timax+50000.D), cnt)

      IF (cnt GE 1) THEN BEGIN
        IF (dmy_flg EQ 0) THEN BEGIN ; for the first file
          ti_tab     = dk[ti_idx].aftime
          flux_tab   = dk[ti_idx].flux
          err_tab    = dk[ti_idx].err
          no_bad_tab = 1-dk[ti_idx].bad
          pid_tab    = dk[ti_idx].pid
          dmy_flg = 1
        ENDIF ELSE BEGIN ; for 2nd or the later files
          ti_tab     = [dk[ti_idx].aftime, ti_tab]
          flux_tab   = transpose([transpose(dk[ti_idx].flux),  transpose(flux_tab)])
          err_tab    = transpose([transpose(dk[ti_idx].err),   transpose(err_tab)])
          no_bad_tab = transpose([transpose(1-dk[ti_idx].bad), transpose(no_bad_tab)])
          pid_tab    = [dk[ti_idx].pid, pid_tab]
        ENDELSE
      ENDIF

    ENDIF
  ENDFOR

  ;;; Sort and uniq dark data
  IF (n_elements(ti_tab) GT 0) THEN BEGIN
    idx_tmp = where(ti_tab GT 0.D, cnt)
    IF (cnt GT 0) THEN BEGIN
      idx_tmp    = uniq(ti_tab[sort(ti_tab)])
      ti_tab     = ti_tab[idx_tmp]
      flux_tab   = flux_tab[*,idx_tmp]
      err_tab    = err_tab[*,idx_tmp]
      no_bad_tab = no_bad_tab[*,idx_tmp]
      pid_tab    = pid_tab[idx_tmp]
    ENDIF
  ENDIF


;;----------------------------------------------------------
;; Extract dark value from the current TSD file
;;----------------------------------------------------------
;;; This part can be omitted when the gb_make_dark_table module was performed
;;;   before this module.

  ;;-----------------------
  ;; Check the data
  ;;-----------------------
  ok1_dat = op_shutter * of_calA * of_calB * of_boost * no_blank * no_in_saa
  ok2_dat = bytarr(num_ch, num_data)
  FOR i = 0, num_ch-1 DO BEGIN
    ok2_dat[i,*] = ok1_dat[*] * no_bad[i,*]
  ENDFOR

  ; though not sufficient condition....
  idx_dat = where(ok1_dat gt 0.D, cnt)
  IF (cnt EQ 0) THEN BEGIN
    LMESSAGE, 3, 'Not sufficient obs. data...Go back...', /CONTINUE
    RETURN, !SUCCEED
  ENDIF

  ;;-----------------------
  ;; Identify useful dark data frames with flags
  ;;-----------------------

  ; Avoid after effect of continuous CAL
  ;  - make usable flag which are set after continuous CAL
  calAstr = where(fix(on_calA) - shift(on_calA, 1) EQ  1, cnt_calAstr)
  calAend = where(fix(on_calA) - shift(on_calA, 1) EQ -1, cnt_calAend)

  IF (cnt_calAstr GT 0) AND (cnt_calAend GT 0) THEN BEGIN

    ; If file starts Cal On then void first calAend.
    IF (calAend[0] LT calAstr[0]) THEN calAend = calAend[1:N_elements(calAend)-1]
    ; If file ends Cal On then void last calAstr.
    IF (calAstr[N_elements(calAstr)-1] GT calAend[N_elements(calAend)-1]) THEN $
            calAstr = calAstr[0:N_elements(calAstr)-2]

    ; Frame# to be regarded as unusable for dark data after Cal ON
       ; Cal On more than 16 frames are regarded as continuous cal
       cont_cal_idx = where((calAend - calAstr) GT 16, cnt_unuse)
       IF (cnt_unuse GT 0) THEN BEGIN ; If there are continuous cal
         unuse_str = calAend[cont_cal_idx]
         ; 2000 frames after continuous cal are regarded as unusable for dark
         unuse_end = unuse_str + 2000

         unusable_dk = make_array(N_elements(on_calA), /BYTE)
         FOR i=0, N_elements(unuse_str)-1 DO BEGIN
           IF unuse_end[i] GE N_elements(unusable_dk) THEN unuse_end[i] = N_elements(unusable_dk)-1
           unusable_dk[unuse_str[i]:unuse_end[i]] = 1
         ENDFOR
       usable_dk = 1 - unusable_dk
       ENDIF

  ENDIF

  IF N_Elements(usable_dk) EQ 0 THEN usable_dk = make_array(N_elements(on_calA), value=1, /BYTE)

  ok1_dk = cl_shutter * of_calA * usable_dk * of_calB * of_boost * no_in_saa
  ok2_dk = ok1_dk * no_blank
  ok3_dk = bytarr(num_ch,num_data)
  FOR i = 0, num_ch-1 DO BEGIN
    ok3_dk[i,*] = ok2_dk[*] * no_bad[i,*]
  ENDFOR

  ;;-----------------------
  ;; Make index of dark data & Check them
  ;;-----------------------
  idx_dk = where(ok2_dk NE 0.D, dk_cnt)
           ; idx_dk is an index of dark data frames
           ; dk_cnt is the number of dark data frames

  IF (dk_cnt GT 5) THEN BEGIN ;; Now, needs more than 5 frames of dark data
    inum_str = where(idx_dk - shift(idx_dk, 1) NE  1, cnt_dkb) ; index# of dark start
    inum_end = where(idx_dk - shift(idx_dk,-1) NE -1         ) ; index# of dark end
                                                    ; cnt_dkb is a number of dark blocks

    ; Packet ID for each dark
    dk_pid = pid[idx_dk[inum_str]]

    IF (cnt_dkb LE 2) THEN BEGIN
    ;; Originally, needs more than 2 sets of dark data in the file.
    ;; -> Now, allow single dark measurement.
      ;LMESSAGE, 3, 'Not sufficient dark data with reasonable intervals...Go back...', /CONTINUE
      ;RETURN, !SUCCEED
      LMESSAGE, 3, 'Dark measurement is only once in the file.', /CONTINUE
    ENDIF
  ENDIF ELSE BEGIN
    LMESSAGE, 3, 'Not sufficient dark data in the current TSD file.', /CONTINUE
    ;RETURN, !SUCCEED
  ENDELSE


;;----------------------------------------------------------
;; Derive dark values and subtract
;;----------------------------------------------------------
FOR ch = 0, num_ch-1 DO BEGIN  ; loop for channel (pixel)

  ;;-----------------------
  ;; Estimate a dark value at each dark data set in the current TSD file.
  ;;-----------------------
  ;;; This part can be omitted when the gb_make_dark_table module was performed
  ;;;   before this module.
  IF (dk_cnt GT 5) THEN BEGIN ; There is valid dark data in the current file.

    dk_flux = dblarr(cnt_dkb) ; cnt_dkb is the number of dark data set in the file
    dk_time = dblarr(cnt_dkb)

    FOR j = 0, cnt_dkb-1 DO BEGIN

      ;; not to use data at transition period
      IF (num_ch EQ 75) AND (ch EQ 0) THEN BEGIN ; If pixel with strong after effect
                                                 ;;;  now special pixel is LW#1
        nouse_aft_pcal = 160
      ENDIF ELSE BEGIN
        nouse_aft_pcal = 10
      ENDELSE
      ii_str = idx_dk[inum_str[j]] + nouse_aft_pcal ; start frame# of 'j'th dark data
      ii_end = idx_dk[inum_end[j]] - 2              ; end   frame#

      IF (ii_end LE ii_str) THEN Continue ; avoid an error due to invalid dark range

      ;idx_ok = where(flux[ch,ii_str:ii_end]*ok3_dk[ch, ii_str:ii_end] ne 0.D, cntp)
      idx_ok   = where(ok3_dk[ch, ii_str:ii_end] gt 0.D, cntp)
      idx_ok_f = where(ok2_dk[ii_str:ii_end] gt 0.D) ; This index isn't affected by pixel bad flag.
      ; check if whole data in a block are bad
      IF (cntp LE 5) THEN Continue

      ;; Set dark value for 'j'th dark data
      dk_flux[j] = gb_dark_fit(flux[ch, idx_ok + ii_str], ti[idx_ok], CHISQ=chisq)
                                                         ;ti[idx_ok+ii_str] is also acceptable
      dk_time[j] = mean(ti[idx_ok_f + ii_str])

      ; To set quality (tentative code at 050609, the definition is groundless)
      ;print, ch, ii_str, chisq; #for test
      IF ((chisq GT 2e-7) AND (chisq LE 5e-7)) THEN BEGIN
        ;print, 'QUAL1', ch, ii_str, chisq; #for test
        IF (j GE 1) THEN BEGIN
          qual_dk_data[ch, idx_dk[inum_str[j-1]]+2:ii_str]=1
          qual_dk_table[ch, idx_dk[inum_str[j-1]]+2:ii_str]=1
        ENDIF ELSE IF (j EQ 0) THEN BEGIN
          qual_dk_data[ch, 0:ii_str]=1
          qual_dk_table[ch, 0:ii_str]=1
        ENDIF
      ENDIF ELSE IF (chisq GT 5e-7) THEN BEGIN
        ;print, 'QUAL2', ch, ii_str, chisq; #for test
        IF (j GE 1) THEN BEGIN
          qual_dk_data[ch, idx_dk[inum_str[j-1]]+2:ii_str]=2
          qual_dk_table[ch, idx_dk[inum_str[j-1]]+2:ii_str]=2
        ENDIF ELSE IF (j EQ 0) THEN BEGIN
          qual_dk_data[ch, 0:ii_str]=2
          qual_dk_table[ch, 0:ii_str]=2
        ENDIF
      ENDIF

    ENDFOR
  ENDIF ELSE BEGIN ; remove variables, dk_flux, df_time, which may contain table values of previous pixel
    dk_flux = 1
    dummy = temporary(dk_flux)
    dk_time = 1
    dummy = temporary(dk_time)
  ENDELSE

  ;;----------------------------------------------------------
  ;; Get final dark values: merge and sort dark data
  ;;----------------------------------------------------------
  ; Merge new dark data (obtained from the current file) with dark table values
  IF (n_elements(ti_tab) GT 0) AND (n_elements(dk_time) GT 0) THEN BEGIN
    dk_flux  = [dk_flux, reform(flux_tab[ch,*])]
    dk_time  = [dk_time, reform(ti_tab[*])]
    dk_pid   = [dk_pid, reform(pid_tab[*])]
  ENDIF ELSE IF (n_elements(ti_tab) GT 0) THEN BEGIN
    dk_flux  = [reform(flux_tab[ch,*])]
    dk_time  = [reform(ti_tab[*])]
    dk_pid   = [reform(pid_tab[*])]
  ENDIF

  IF (n_elements(dk_time) GT 0) THEN BEGIN
    ; Sort and uniq
    idx_temp = uniq(dk_time, sort(dk_time))
    ;; Get dark data set for the current file and channel
    dk_time  = dk_time[idx_temp]
    dk_flux  = dk_flux[idx_temp]
    dk_pid   = dk_pid[idx_temp]

    ; Cancel out the data of other than co-add mode
    idx_coadd42 = where(dk_pid EQ 66, cnt_coadd)
    IF (cnt_coadd GT 0) THEN BEGIN
      dk_time = dk_time[idx_coadd42]
      dk_flux = dk_flux[idx_coadd42]
      dk_pid  = dk_pid[idx_coadd42]
    ENDIF

  ENDIF ELSE BEGIN
    IF (ch EQ 0) THEN LMESSAGE, 3, 'No dark data available in both table and the current files.'
       ; ^ this massage is shown only once.
    no_dk_flg[ch, *]     = 1
    qual_dk_table[ch, *] = 3
    Continue
    ;RETURN, !FAIL
  ENDELSE

  ; To eliminate blocks where data in the blocks are all bad
  dk_idx = where(dk_flux GT 0.D, cntp)

  IF (cntp EQ 0) THEN BEGIN
  ;; If there is no available dark, cannot make dark subtraction.
  ;
  ;if (cntp LT 4) then begin    ; not sufficient dark data with reasonable intervals.
  ;; Originally, needs more than 2x2 sets of dark data in the file.
  ;; (There may be two dark data in one shutter close sequence.)
    no_dk_flg[ch, *]     = 1
    qual_dk_table[ch, *] = 3
    LMESSAGE, 7, 'Dark data unavailable. (ch'+strcompress(string(ch))+')', /CONTINUE
    Continue
  ENDIF

  ;; Get dark data set available to the current channel.
  dk_fin = dk_flux[dk_idx]
  ti_fin = dk_time[dk_idx]

  ; To reduce fluctuation
  ;; There may be two dark data in one shutter close sequence.
  ;; These two dark will be averaged.
  FOR j=1, cntp-1 DO BEGIN
    IF (abs(ti_fin[j] - ti_fin[j-1]) LE dk_int) THEN BEGIN
      dk_fin[j-1] = (dk_fin[j-1]+dk_fin[j])/2.D
      dk_fin[j]   = 9999.D
      ti_fin[j-1] = (ti_fin[j-1]+ti_fin[j])/2.D
      j = j+1
    ENDIF
  ENDFOR

  ; sort again
  dk_idx = where(dk_fin NE 9999.D, cntp)
  IF (cntp EQ 0) THEN BEGIN
  ;if cntp lt 3 then begin  ; not sufficient dark data with reasonable intervals.
    no_dk_flg[ch,*]     = 1
    qual_dk_table[ch,*] = 3
    LMESSAGE, 7, 'Dark data unavailable. (ch'+strcompress(string(ch))+')', /CONTINUE
    Continue
  ENDIF

  ;; Get final dark data set available to the current channel.
  dk_fin = dk_fin[dk_idx]
  ti_fin = ti_fin[dk_idx]

  ;;----------------------------------------------------------
  ;; Interpolate and subtract dark
  ;;----------------------------------------------------------
  min_ti = min(ti) ; TI at the file start
  max_ti = max(ti) ; TI at the file end
  ch_str = 'ch.'+strcompress(string(ch))+': '

  IF (total(where(ti_fin GT min_ti) EQ where(ti_fin LT max_ti)) EQ 0) THEN BEGIN
    LMessage, 7, ch_str+'There are NO dark data in the current file.'
    IF (total(where(ti_fin LT min_ti)) NE -1) THEN BEGIN
      ;LMessage, 7, ch_str+'Dark table data available from earlier data than the current file.'
      dk_intp = make_array(n_elements(flux[ch,*]))
      dk_intp[*] = dk_fin[n_elements(dk_fin)-1] ; dark is constant
    ENDIF ELSE IF (total(where(ti_fin GT max_ti)) NE -1) THEN BEGIN
      ;LMessage, 7, ch_str+'Dark table data available from later data than the current file.'
      dk_intp = make_array(n_elements(flux[ch,*]))
      dk_intp[*] = dk_fin[0] ; dark is constant
    ENDIF ELSE BEGIN
      ;LMessage, 7, 'Dark table data available from both earlier and later than the current file.'
      dk_intp = interpol(dk_fin,ti_fin,ti) ; linear interpolation
    ENDELSE
  ENDIF ELSE BEGIN
    ;LMessage, 7, 'There are dark data in the current file.'
    IF (n_elements(dk_fin) LE 1) THEN BEGIN
      LMessage, 7, 'There is only once dark data in the current file.'
      dk_intp = make_array(n_elements(flux[ch,*]))
      dk_intp[*] = dk_fin ; dark is constant
    ENDIF ELSE BEGIN
      ;LMessage, 7, 'There are more then once dark data in the current file.'
      dk_intp = interpol(dk_fin,ti_fin,ti) ; linear interpolation
    ENDELSE
  ENDELSE

  ; Dark subtraction
  corr_flux[ch,*] = flux[ch,*] - dk_intp[*]

  ;; Display for check
  IF keyword_set(display) THEN BEGIN
;    IF (ch EQ 4) THEN BEGIN
      ;; Plot dark data and interpolated results
;      plot, ti_fin, dk_fin, yra=[-0.005, 0.003], /nodata
;      oplot, ti_fin, dk_fin, psym=-4, color='FF0000'XL
;      oplot, ti, dk_intp, psym=-5, color='0000FF'XL
      ;
      ;; Plot flux and subtructed results
      plot, ti, corr_flux[ch,*], yra=[-0.002, 0.01], title='pixel#'+strcompress(string(ch+1)), psym=3
      oplot, ti, flux[ch,*], color='00FF00'XL, psym=3
      oplot, ti, dk_intp[*], color='0000FF'XL
      oplot, ti_fin, dk_fin, color='FF0000'XL, psym=4
      ;
;    ENDIF
    wait, 0.3
  ENDIF

ENDFOR  ; End of loop for channel

;;----------------------------------------------------------
;; Set quality indicators
;;----------------------------------------------------------
; tentative version
  ret = tsd->Set_Quality_TSD('FIS_OBS','Qual_dk_data', qual_dk_data)
  ret = tsd->Set_Quality_TSD('FIS_OBS','Qual_dk_param', qual_dk_param)
  ret = tsd->Set_Quality_TSD('FIS_OBS','Qual_dk_table', qual_dk_table)

;;----------------------------------------------------------
;; Write TSD
;;----------------------------------------------------------
   dummy = tsd->Set_Value_TSD('FIS_OBS','Flux', corr_flux)
   LMESSAGE, 7, 'Writing dark subtracted TSD...', /CONTINUE

   dummy = tsd->Set_Pix_Flag_TSD('FIS_OBS', 'No_dk_corr', no_dk_flg)
 
   idx_bad = where(st_bad+no_dk_flg GT 0, cnt)
   IF (cnt GT 0) THEN st_bad[idx_bad]=1
   dummy = tsd->Set_Pix_Flag_TSD('FIS_OBS', 'Bad', st_bad)

   LMESSAGE, 7, 'Setting NO_DK_CORR, BAD flag,...', /CONTINUE

;;----------------------------------------------------------
;; Write HISTORY in TSD header
;;----------------------------------------------------------
   LMESSAGE, 7, 'Writing HISTORY', /CONTINUE
   name_ver = gb_get_name_ver(infile='gb_dark_subtraction')
   hist_message = STRCOMPRESS(' '+name_ver+' done. CCF= ')

   IF KEYWORD_SET(ccf) NE 1 THEN BEGIN
      hist_message = hist_message + 'NONE'
   ENDIF ELSE BEGIN
      hist_message = hist_message+ccf
   ENDELSE

   dummy = tsd_fxaddhist(tsd, hist_message, /prres)
   IF dummy NE 0 THEN $
     LMESSAGE, 3, 'Unable to write HISTORY', /CONTINUE

   return, !SUCCEED

END
