; $Id: gb_ramp_curve_corr.pro,v 1.14 2007/01/21 23:42:47 yamamura Exp $
;
; Copyright (c) 2002, ASTRO-F Data Reduction Team.  All rights reserved.
;
;+
; NAME:
;   GB_Ramp_Curve_Corr.pro
;
; PURPOSE:
;   The output of the cryogenic readout electronics (CRE) has
;   non-linearity characteristics depending on an output voltage.
;   This function corrects this non-linearity, assuming that the
;       degree of deviation from an ideal ramp curve is a function of
;       output voltage.
;
; CATEGORY:
;   Function. Green Box procedure in ASTRO-F pipeline.
;
; CALLING SEQUENCE:
;   Result = GB_Ramp_Curve_Corr (tsd
;   [, CCF_FILE_RCTB = ccf_file_rctb
;    , NOPLOT = noplot])
;
; INPUTS:
;   tsd              TSD object reference.
;   CCF_FILE_RCTB    CCF file if given externally
;   /NOPLOT         Suppress plotting during the process
;
; OUTPUTS:
;   tsd     TSD object reference. CRE non-linearity
;      corrected data.
;   (SUMMARY = summary)   Summary of the non-linearity correction,
;      e.g., examination of non-linearity correction. How
;       to manage the summaries is under considerations.
;
; RETURN:
;   !SUCCEED if function ended successfully.
;   !FAIL in failure.
;
; RESTRICTIONS:
;   no correction applyed (just multiply 1.0 for all data in -0.1--1.0)
;   when CCF='nocorrection' is given. Useful for tests. Correction
;   table created internally so no need of CCF file.
;
; PROCEDURE:
;   1. Extract detector data from TSD.
;   2. Read correction parameters from CCF.
;   3. Check flags for invalid data.
;   4. Extract ramp curves.
;   5. Select a region to be adopted from the ramp curve.
;   6. Carry out a correction; method is TBD.
;   7. Examine a result.
;   8. Write a history in TSD header.
;   9. Output TSD object reference.
;   10. Write a summary of this function.
;
;   The contents of the summary with its management are TBD.
;   Items to be discussed and determined:
;     1. How to find the reset. Libraries to find the reset and
;     to extract the data taken between resets can be prepared
;     for the pipeline.
;     2. Correction method and correction parameters.
;     These are to be determined from the pre-launch
;     calibration measurements.
;     3. Correction of the data containing saturated parts.
;
; EXAMPLE:
;
; REFERENCE:
;   Baba et al. 2002 Nov. 21, ``ASTRO-F/FIS Survey Data Structure"
;
; MODIFICATION HISTORY:
;   Oct. 2002       Written by A. Kawamura.
;   Dec. 2002     Header is added.
;   Dec. 02, 2003 IY  Implementaion of real algorithm.
;   Dec. 12, 2003 IY  Polishing up the code to adjust TSD data.
;   Dec. 18, 2003 IY  FLUX is treated as double.
;   May. 28, 2003 IY  SW/LW automatic selection. CCF preliminary.
;   Nov. 02, 2004 IY  TSD3 implemented. PacketID recognition.
;   Nov. 03, 2004 IY  Use external function for PacketID recognition.
;          /NOCORRECTION -> CCF='nocorrection'
;          Create_RCTbl_of_Constant() updated.
;   Jan. 11, 2005 IY  CCF new format including offset to avoid
;                               zero-cross. Old CCF will not be used.
;   Jan. 27, 2005 SM  Support the latest sampling mode by using /lib/smpl_mode.pro
;   Mar. 15, 2005 IY  Create_RCTbl_of_Constant() updated for new rctbl struct.
;   Jun. 09, 2005 IY  CCF implemented.
;   Jun. 12, 2005 IY  bad_val_def() moved to external routine.
;                     Narrow and Wide are treated at the same time.
;   Jun. 15, 2005 IY  MESSAGE replaced with LMessage.
;   Oct. 11, 2006 IY  Correction changed from multiple to additive.
;                     Quadratic interpolation instead of spline.
;   Nov. 11, 2006 IY  Accept CCF_RTBL option = specify CCF file directly.
;   Jan. 04, 2007 IY  PixBad flag not used as input
;                 IY  Skip dead pixels as they are mostly flaged as bad and
;                     cause unsatisactorily result in periodic noise reduction
;   Jan. 13, 2007 IY  /QUADRATIC in INTERPOL not used
;   Jan. 13, 2007 IY  y_bad_val_def() not used. BADLIM is used.
;-

; main starts below
; ==============================================================
FUNCTION Do_Ramp_Curve_Corr, $
   flux,       $ ; array of detector data (flux[ndet, sampling])
   packetid,   $ ; Packet ID (readout mode) (packetid[sampling])
   aftime,     $ ; AFTIME ((aftime[sampling])
   inst,       $ ; Type of data ('FIS' or 'FISVI')
   badpix,     $ ; BAD pixel flag (badpix[ndet, sampling])
   no_rp_corr, $ ; Ramp curve correction unavailable flag (returned, [ndet, sampling])
   ctbl,       $ ; correction table ctbl[ndet].v/c
   dead,       $ ; Dead pixel list from CCF
   drange        ; detector pixel range to be applied [start, end]

; (returned)     ; corrected ramps
;
; 02/10/31 I.Yamamura
; 03/03/25 avoid crush when no bad data exists
;          handle v-range evaluation more realistically
; 03/12/02 Included in the GB module
; 03/12/12 treat one dimensional data
; 04/05/19 add drange[] to treat WIDE and NARROW band separately
; 04/10/30 migrating to TSD3. result flag badpix -> no_rp_corr
; 07/01/04 consider dead pixel (ignore for the moment)

   f_dim = size(flux, /DIMENSIONS)
   if (n_elements(f_dim) NE 2) then begin
      LMESSAGE, 0, "Incorrect dimension of flux (expect flux[ndet, sampling])."+STRING(f_dim,/PRINT), /CONTINUE
      return, !FAIL
   endif

   b_dim = size(badpix, /DIMENSIONS)
   if (n_elements(b_dim) NE 2 OR (b_dim[0] NE f_dim[0]) OR (b_dim[1] NE f_dim[1])) then begin
      LMESSAGE, 0, "Incorrect dimension of badpix (expect ["+STRING(f_dim,/PRINT)+"]).", /CONTINUE
      return, !FAIL
   endif

   n_dim = size(no_rp_corr, /DIMENSIONS)
   if (n_elements(n_dim) NE 2 OR (n_dim[0] NE f_dim[0]) OR (n_dim[1] NE f_dim[1])) then begin
      LMESSAGE, 0, "Incorrect dimension of no_rp_corr (expect ["+STRING(f_dim,/PRINT)+"]).", /CONTINUE
      return, !FAIL
   endif

   p_dim = size(packetid, /DIMENSIONS)
   if (n_elements(p_dim) NE 1 OR p_dim[0] NE f_dim[1]) then begin
      LMESSAGE, 0, "Incorrect dimension of packetid (expect ["+STRING(f_dim[1],/PRINT)+"]).", /CONTINUE
      return, !FAIL
   endif

   t_dim = size(ctbl, /DIMENSIONS)
   if (n_elements(t_dim) NE 1 OR t_dim[0] NE f_dim[0]) then begin
      LMESSAGE, 0, "Incorrect dimension of ctbl (expect [ndet].v/c)"+STRING(t_dim,/PRINT), /CONTINUE
stop
      return, !FAIL
   endif

;   BAD = y_Bad_Val_Def()
   BADLIM = -9    ; arbitrary small number

; set readout mode
   ;rmode = gb_get_readout_mode(packetid)
   rmode = smpl_mode(packetid, AFTIME=aftime[0], TYPE=inst)

; set data range
   if (drange[0] LT 0) then begin
      drange[0] = 0
   endif else if (drange[1] GE f_dim[0]) then begin
      drange[1] = r_dim[0] - 1
   endif else if (drange[1] GE t_dim[0]) then begin
      drange[1] = t_dim[0] - 1
   endif

   for idet=drange[0], drange[1] do begin

; 07/01/04 skip dead pixels (temorary needed because they are mostly flagged as bad)
   if (dead[idet] EQ !TRUE) then begin
      msg = STRING(FORMAT='(A, I3, A)', 'Skip dead pixel #', idet+1, '.')
      lmessage, 5, msg, /CONTINUE
      continue
   endif

; use data points between inner max and min
      v = ctbl[idet].v
      c = ctbl[idet].c
;      e = ctbl[idet].e
      o = ctbl[idet].o

; select only valid values
      idx = Where (v GE BADLIM AND c GE BADLIM, cnt)
      if (cnt EQ 0) then begin
         no_rp_corr[idet, *] = no_rp_corr[idet, *] OR !TRUE
         continue
      endif

      v = v[idx]
      c = c[idx]
;      e = e[idx]

; sort as v being ascending order
      idx = Uniq(v, sort(v))
      v = v[idx]
      c = c[idx]
;      e = e[idx]

; choose the most inside pair
      vmax = Max(v, imax)
      vmin = Min(v, imin)
      imin = Max(imin)
      imax = Min(imax)

      if (imax - imin LT cnt / 3) then begin
         no_rp_corr[idet, *] = no_rp_corr[idet, *] OR !TRUE
         continue
      endif

; final sets of correction table data
      v = v[imin:imax]
      c = c[imin:imax]
;      e = e[imin:imax]

      vmin = Min(v, MAX=vmax)

; do correction for individual ramps
;      t = SPL_INIT(v, c, /DOUBLE)
;      cflux[idet,*] = flux[idet,*] * SPL_INTERP(v, c, t, flux[idet,*], /DOUBLE)
      f = reform(flux[idet, *], f_dim[1])
      b = reform(badpix[idet, *], b_dim[1])
      n = reform(no_rp_corr[idet, *], n_dim[1])
;      f = f - o
      idx = Where ( (NOT b) AND (NOT n) AND (f GE vmin) AND (f LE vmax) AND ((rmode NE 1) AND (rmode NE 9)), cnt, COMPLEMENT=idxc, NCOMPLEMENT=ncnt)
      if (cnt GT 0) then begin
         idy = Sort(f[idx])
;         f[idx[idy]] = f[idx[idy]] * SPLINE(v, c, f[idx[idy]])
         f[idx[idy]] = f[idx[idy]] + INTERPOL(c, v, f[idx[idy]], /QUADRATIC)
;         f[idx[idy]] = f[idx[idy]] + INTERPOL(c, v, f[idx[idy]])
      endif
      if (ncnt GT 0) then n[idxc] = n[idxc] OR !TRUE

      ; Set no_rp_corr flag as 0 on CDS frames
      idx_cds = Where(rmode EQ 1, cnt_cds)
      IF (cnt_cds GT 0) THEN n[idx_cds] = 0

;      flux[idet, *] = f + o
      flux[idet, *] = f
      no_rp_corr[idet, *] = n

;;; qualities
;;;


   endfor

   v = 0B & c = 0B & r = 0B & b = 0B & n = 0B & t = 0B

   return, !SUCCEED
end

; ==============================================================
FUNCTION Create_RCTbl_of_Constant, $
   ndet,    $  ; Number of detectors
   ngrid,   $  ; table grid size
   range,   $  ; [min, max] range of the adopted FLUX range.
   factor      ; a constant factor to be applied.
; create a rctbl that does not change data (for test purpose)
; 05/03/15 updated for latest rctbl structure (o field added)

   if (N_Elements(range) NE 2) then begin
      LMESSAGE, 0, 'range must be [min, max]', /CONTINUE
      return, !FAIL
   endif
   vmin = Min(range, MAX = vmax)
   vstep = (vmax - vmin) / DOUBLE(ngrid)

   tbl = { v: dblarr(ngrid), c: dblarr(ngrid), e: dblarr(ngrid), o: 0D }
   rctbl = replicate(tbl, ndet)
   v_tmp = indgen(ngrid, /DOUBLE) * vstep + vmin
   c_tmp = dblarr(ngrid) & c_tmp[*] = double(factor)
   e_tmp = dblarr(ngrid) & e_tmp[*] = 0D
; for test of quality indicator
   idx = WHERE(v_tmp GT 0.07D, cnt)
   if (cnt GT 0) then e_tmp[idx] = 0.01D
   idx = WHERE(v_tmp GT 0.10D, cnt)
   if (cnt GT 0) then e_tmp[idx] = 0.02D

   rctbl[*].v = v_tmp
   rctbl[*].c = c_tmp
   rctbl[*].e = e_tmp
   rctbl[*].o = 0D
   v_tmp = 0B & c_tmp = 0B & e_tmp = 0B

   return, rctbl
end

; ==============================================================
FUNCTION GB_Ramp_Curve_Corr, tsd, CCF_FILE_RCTB = ccf_file_rctb, NOPLOT=noplot

;ON_ERROR 2

; define default CCF file name here
   CCF_RCTB_S = 'RCTBS'
   CCF_RCTB_L = 'RCTBL'

;;; Starting GB_Ramp_Curve_Corr ;;;
   LMESSAGE, 5,'Executing GB_Ramp_Curve_Corr...', /CONTINUE

; set colour table
if ( NOT Keyword_Set(NOPLOT) ) then LoadCT, 39

; SW or LW
   datatype = tsd->Get_DataType()
   if (datatype EQ 'FIS_SW') then begin
      ndet = 100
   endif else if (datatype EQ 'FIS_LW') then begin
      ndet =  75
   endif else begin
      LMESSAGE, 0, "Unknown DataType ("+datatype+").", /CONTINUE
      return, !FAIL
   endelse
   drange = [ 0, ndet-1]

; read the correction table (preliminary I/F)
   if ( Keyword_Set(CCF_FILE_RCTB) ) then begin
      if ( ccf_file_rctb EQ 'nocorrection' ) then begin
         LMESSAGE, 6, "No Correction mode.", /CONTINUE
         ccf_rctb = Create_RCTbl_of_Constant(ndet, 500, [-0.1D, 1.0D], 1.0D)
      endif else begin
         if ( File_Test(ccf_file_rctb, /READ, /REGULAR) ) then begin
;            restore, ccf_file_rctb     ; this part have to be be updated
             ccf_rctb = CCF_Read_CCF_FILE(ccf_file_rctb, ERROR=err, HEADER=hdr)
         endif else begin
            LMESSAGE, 0, ccf_file_rctb + ': No CCF File found ('+ccf_file_rctb+').', /CONTINUE
            return, !FAIL
         endelse
      endelse
   endif else begin

           if (ndet EQ 100) then ccf_file = CCF_RCTB_S $
      else if (ndet EQ  75) then ccf_file = CCF_RCTB_L $
      else begin
         LMESSAGE, 0, "Invalid Array size (",STRING(ndet),").", /CONTINUE
         return, !FAIL
      endelse

      ccf_rctb = CCF_Read_CCF(ccf_file, tsd, ERROR=err, HEADER=hdr)

      if (err EQ !FAIL) then begin
         LMESSAGE, 0, 'Cannot access CCF File ('+ccf_file+')', /CONTINUE
         return, !FAIL
      endif
   endelse

; read DeadPix CCF file
        if (ndet EQ 100) then ccf_file = 'DEADS' $
   else if (ndet EQ  75) then ccf_file = 'DEADL' $
   else begin
      LMESSAGE, 0, "Invalid Array size (",STRING(ndet),").", /CONTINUE
      return, !FAIL
   endelse

   ccf_dead = CCF_Read_CCF(ccf_file, tsd, ERROR=err, HEADER=hdr)

   if (err EQ !FAIL) then begin
      LMESSAGE, 0, 'Cannot access CCF File ('+ccf_file+')', /CONTINUE
      return, !FAIL
   endif


;  check Correction Table data size
   nrc = size(ccf_rctb)
   if (nrc[1] NE ndet) then begin
      msg = STRING('RCTbl size incompatible with the array size', nrc[1],  ndet)
      LMESSAGE, 0, msg, /CONTINUE
      return, !FAIL
   endif

; read data from TSD
   inst=tsd->get_header_keyword_tsd('PRIMARY', 'INSTRUME')

   aftime=tsd->get_value_tsd('FIS_OBS', 'AFTIME')

   flux = tsd->Get_Value_TSD('FIS_OBS', 'FLUX')

   packetid = tsd->Get_Value_TSD('FIS_OBS', 'PACKETID')

   undef_anom = tsd->Get_Pix_Flag_TSD('FIS_OBS', 'UNDEF_ANOM')
   arith_err = tsd->Get_Pix_Flag_TSD('FIS_OBS', 'ARITH_ERR')
   saturate = tsd->Get_Pix_Flag_TSD('FIS_OBS', 'SATURATE')
   badpix = undef_anom OR arith_err OR saturate

   no_rp_corr = tsd->Get_Pix_Flag_TSD('FIS_OBS', 'NO_RP_CORR')

; plot data before correction
   if ( NOT Keyword_Set(NOPLOT) ) then Plot, flux[12, *], psym=3

; initialize no_rp_corr
   no_rp_corr = no_rp_corr AND 0B      ; clear up

; do correction
   ret = Do_Ramp_Curve_Corr(flux, packetid, aftime, inst, badpix, no_rp_corr, ccf_rctb, ccf_dead, drange)
   if (ret NE !SUCCEED) then begin
      LMESSAGE, 0, 'Correction Failed . Exiting...', /CONTINUE
      return, !FAIL
   endif

; plot data after correction
   if ( NOT Keyword_Set(NOPLOT) ) then OPlot, flux[12, *], psym=3, color=160

;; update badpix flag
   badpix_org = tsd->Get_Pix_Flag_TSD('FIS_OBS', 'BAD')

   badpix = badpix OR no_rp_corr OR badpix_org

;;; Set quality indicator
   qd = tsd->Get_Quality_TSD('FIS_OBS', 'QUAL_RP_DATA')
   qp = tsd->Get_Quality_TSD('FIS_OBS', 'QUAL_RP_PARAM')
   qt = tsd->Get_Quality_TSD('FIS_OBS', 'QUAL_RP_TABLE')


;;; write back in TSD
   ret = tsd->Set_Value_TSD('FIS_OBS', 'FLUX', flux)
   ret = tsd->Set_Pix_Flag_TSD('FIS_OBS', 'BAD', badpix)
   ret = tsd->Set_Pix_Flag_TSD('FIS_OBS', 'NO_RP_CORR', no_rp_corr)

   ret = tsd->Set_Quality_TSD('FIS_OBS', 'QUAL_RP_DATA', qd)
   ret = tsd->Set_Quality_TSD('FIS_OBS', 'QUAL_RP_PARAM', qp)
   ret = tsd->Set_Quality_TSD('FIS_OBS', 'QUAL_RP_TABLE', qt)

; clean up
   vv = 0B & v = 0B & cc = 0B & c = 0B
   idx = 0B & idx_dlm = 0B & idx_dlm_l = 0B & idr = 0B
   flux = 0B
   badpix = 0B & badpix_org = 0B

;;; Write HISTORY in header ;;;

   LMESSAGE, 7,'Writing HISTORY',/CONTINUE

   name_ver=Gb_Get_Name_Ver(infile='gb_ramp_curve_corr')
   hist_message = StrCompress(' '+name_ver+' done. CCF= ')

   if (Keyword_Set(ccf) NE 1) then begin
      hist_message = hist_message + 'NONE'
   endif else begin
      hist_message = hist_message+ccf
   endelse

;;; Write a summary of this function
   dummy = TSD_FxAddHist(tsd, hist_message, /PRRES)
   if (dummy NE 0) then LMESSAGE, 1,'Unable to write HISTORY',/CONTINUE

   return, !SUCCEED
END
