; $Id: gb_run_gb.pro,v 1.22 2007/01/27 11:19:55 makiuti Exp $
;
; Copyright (c) 2002-2007, ASTRO-F Data Reduction Team.  All rights reserved.
;
;+
; NAME:
;   GB_Run_GB.pro
;
; PURPOSE:
;   This function is used to run and control the Green Box (Pipeline) modules
;   for ASTRO-F/FIS.
;
; CATEGORY:
;   Function to run the Green Box procedures of ASTRO-F pipeline.
;
; CALLING SEQUENCE:
;   Result = GB_Run_GB(infile('TSD FITS file name' or 'LDS')
;              [, OUTFILE=outfile,
;               , GB_MODULE=gb_module, Enable_GB=enable_gb
;               , /LOG, /Elapsed_time, /Save_at_all_process
;               , /GB_LIST, Other_options=other_options])
;
; INPUTS:
;   infile            Name of a TSD File or 'LDS' or TSD object
;
; OUTPUTS:
;   outfile               TSD output file (FITS binary table). Default name is "out.fits".
;                         If the file with the same name exists, it will be replaced.
;                         When the option 'outfile=-1' is set, no output file is created.
;   /Save_at_all_process  TSD output file at each process.
;   /LOG                  Log (Journal) file.
;   /Elapsed_time         Elapsed time record file. "GB_elapsed_time.txt"
;                         If the file exists, new records will be appended at the end of the file.
;
; EXAMPLE:
;
;   >ret = gb_run_gb('tsdfile.fits', /log, /elapsed)
;   >ret = gb_run_gb('tsdfile.fits', GB_MOD='gb_conv_to_volt')
;   >ret = gb_run_gb(tsd_obj, Ena=[1,3,5], /save)
;   >ret = gb_run_gb('LDS', time='2006-04-15 17:10:00', span='60', format='tsd4_sw', mode='Fli')
;
;   >ret = gb_run_gb(/GB_LIST) ; to see the numbering of the modules

;   ##  To know how to use the gb_run_gb() in detail,
;   ##  please see the /reduction/gb/doc/README_GB_Run_GB.txt
;
; BASIC PROCEDURES:
;   1)  Read TSD.
;   2)  Apply (selected) GB modules
;       -  GB_Flag_Bad_Data
;       -  GB_Set_Flag_Rstanom
;       -  GB_Conv_to_Volt
;       -  GB_Ramp_Curve_Corr
;       -  GB_Differentiation
;       -  GB_Periodic_Noise ;; added at Oct. 2006
;       -  GB_Set_Glitch_Status_MMT
;       -  GB_Set_Glitch_Status_GP
;       -  GB_Make_DC_Cal_Lamp_Table
;       -  GB_Make_Dark_Table
;       -  GB_Dark_Subtraction
;       -  GB_DC_Responsivity_Corr
;       (-  GB_Transient_Corr) ;; disabled at 050530
;       - GB_Flux_Calib
;   3)  Write HISTORY to header
;   4)  Write TSD
;
; RESTRICTIONS:
;
; REFERENCE:
;   Baba et al., "ASTRO-F/FIS Survey Data Structure"
;   Kawamura et al. 2002 Nov. "ASTRO-F/FIS Survey Data Structure"
;
; MODIFICATION HISTORY:
;   Oct. 2002     Written by A. Kawamura.
;   Jan. 2007     Last modified by Makiuti
;
;-
FUNCTION GB_Run_GB, $
    infile, $           ; Input file name. When 'LDS', query to LDS. A TSD object is also acceptable.
    outfile=outfile, $  ; Output file name, 'out.fits' by default. If -1 then no output.
    TIME=stime, SPAN=span, FORMAT=format, MODE=mode, $; Keywords to query LDS
    ; GB_Flag_Bad_Data
    CCF_DeadPix = ccf_deadpix, $
    CCF_SatPix = ccf_satpix, $
    CCF_Moon = ccf_moon, $
    CCF_SAA = ccf_saa, $
    ; GB_Set_Flag_Rstanom
    CCF_Rstanom = ccf_rstanom, $
    ; GB_Conv_to_Volt
    CCF_Thresh = ccf_thresh, $
    C2V_FERR = c2v_ferr, $
    ; GB_Ramp_Curve_Corr
    CCF_RCTB = ccf_rctb, $
    RC_NOPLOT = rc_noplot, $
    ; GB_Differentiation
    Diff_Mod = diff_mod, $
    ShiftVl = shiftvl, $
    Diff_Disp = diff_disp, $
    Diff_rdvdet = rdvdet, $
    ; GB_Periodic_Noise
    ;
    ; GB_Set_Glitch_Status_MMT
    Output_Glitch = output_glitch, $
    Debug=debug, $
    ; GB_Set_Glitch_Status_GP
    CCF_Glitch_GP = ccf_glitch_gp,$
    GlitchList_GP = glitchlist_gp, $
    Pixel_GP = pixel_gp, $
    Plot_GP = plot_gp, $
    METHOD_GP = method_gb, $
    ; GB_Make_DC_Cal_Lamp_Table, GB_Make_Dark_Table
    ; GB_Dark_Subtraction, GB_DC_Responsivity_Corr
    CCF_DC_Cal_Lamp_Table = ccf_cal_tab, $
    CCF_Dark_Table = ccf_dark_tab, $
    DISP_DARK = disp_dark, $
    RESULT_DC_Resp = result_dc_resp, $
    ;CCF_Dark_Subtraction = ccf_dark, $
    CCF_DC_Responsivity = ccf_responsivity, $
    ; GB_Transient_Corr
    CCF_Transient_Corr = ccf_transient, $
    ; GB_Flux_Calib
    CCF_Calib = ccf_calib, $
    CCF_BadPix = ccf_badpix, $
    ; for GB_Run_GB
    GB_LIST = gb_list, $                         ; Display acceptable GB Modules and number of Enable_GB list
    GB_Module = gb_module, $                     ; Single module is enabled (set by name)
    Enable_GB = enable_gb, $                     ; Multiple modules are enabled (set by number)
    LOG = log, $                                 ; Enable Journal function
    Elapsed_time = elapsed_time, $               ; Enable elapsed-time record
    Save_at_all_process = save_at_all_process, $ ; Save TSD files after each process
    _EXTRA = _extra

;ON_ERROR 2

;;----------------------------------------------------------
;; Read 'VERSION' file and then write the keyword 'PIPELINE' 
;; into the Primary header of a TSD file.
;; 
;; The 'VERSION' file should be manually revised when a GB 
;; module modified.
;;----------------------------------------------------------
VER_PIPELINE = ''

fisdr_root = GETENV('FISDR_ROOT')
ver_file = fisdr_root+'/gb/pro/VERSION'
ret = file_test(ver_file)
IF (ret EQ 1) THEN BEGIN

  num_rows = count_rows(ver_file)
  OPENR, lun, ver_file, /GET_LUN
  verstr = ''

  FOR row = 0, num_rows-1 DO BEGIN
    readf, lun, verstr
  ENDFOR

  dum = Execute(verstr) ; Set a variable 'VER_PIPELINE'
  Close, lun
  FREE_LUN, lun

ENDIF

;;----------------------------------------------------------
;; Help Message
;;----------------------------------------------------------
IF ((Keyword_set(GB_LIST) EQ 1)) THEN BEGIN
  print, "== Acceptable GB_MODULE List =="
  print, "      1. GB_Flag_Bad_Data"
  print, "      2. GB_Set_Flag_Rstanom"
  print, "      3. GB_Conv_to_Volt"
  print, "      4. GB_Ramp_Curve_Corr"
  print, "      5. GB_Differentiation"
  print, "      6. GB_Periodic_Noise"
  print, "      7. GB_Set_Glitch_Status_MMT"
  print, "      8. GB_Set_Glitch_Status_GP"
  print, "      9. GB_Make_DC_Cal_Lamp_Table"
  print, "     10. GB_Make_Dark_Table"
  print, "     11. GB_Dark_Subtraction"
  print, "     12. GB_DC_Responsivity_Corr"
  print, "     13. GB_Flux_Calib"
  print, "Ex.)"
  print, "(for single module)    >ret = gb_run_gb('tsdfile.fits', GB_Mod='GB_Conv_to_Volt')"
  print, "(for multiple modules) >ret = gb_run_gb('tsdfile.fits', Enable_GB=[1,2,3,5])"
  RETURN, -1
ENDIF

IF ((n_params() EQ 0) OR (N_ELEMENTS(infile) EQ 0)) THEN BEGIN
  print, "[## GB_RUN_GB : CALLING SEQUENCE ##]"
  print, "   Result = GB_Run_GB(infile ('TSD file name' or 'LDS')"
  print, "              [, OUTFILE=outfile, CAL_CONST_FILEs=cal_const_files"
  print, "              , GB_MODULE=gb_module, Enable_GB=enable_gb, /LOG, /Elapsed_time, /Save_at_all_process])"
  print, "   e.g."
  print, "   >ret = gb_run_gb('tsdfile.fits', /log, /elapsed)"
  print, "   >ret = gb_run_gb('tsdfile.fits', GB_MOD='gb_conv_to_volt') ;set by a module name"
  print, "   >ret = gb_run_gb('tsdfile.fits', Enable_GB=[1,2,3,5]) ;set by an array of numbers"
  print, "   >ret = gb_run_gb('LDS', time='2006-04-15 17:10:00', span='60', format='tsd4_sw', mode='Fli')"
  print, ""
  print, "   To display List of Acceptable GB Modules and number of Enable_GB"
  print, "   >ret = gb_run_gb(/GB_LIST)"
  RETURN, -1
ENDIF


;;----------------------------------------------------------
;; Journaling
;;----------------------------------------------------------
IF KEYWORD_SET(log) EQ 1 THEN BEGIN

    IF !JOURNAL NE 0 THEN BEGIN
       LMESSAGE, 3, 'Journaling is already in progress', /CONTINUE
       LMESSAGE, 3, 'Stopping JOURNAL...', /CONTINUE
       JOURNAL
    ENDIF

    ;;;  Create output file for log
    logfile = gb_get_logfilename()

    ;;;  Journaling
    ;;; Check directory permission
    IF file_test(logfile) NE 0 THEN BEGIN
       LMESSAGE, 3, 'Logfile already exists. '+logfile, /CONTINUE
       LMESSAGE, 0, 'Green box not started...'
    ENDIF

    journal, logfile
    LMESSAGE, 5, 'Starting log: '+logfile, /CONTINUE

ENDIF

;;----------------------------------------------------------
;; Record elapsed-time
;;----------------------------------------------------------
IF KEYWORD_SET(elapsed_time) EQ 1 THEN BEGIN

  time_elps = systime()
  time_start_sec = systime(1)
  fname_elapsed = 'GB_elapsed_time.txt'
  openw, lun, fname_elapsed, width=100, /get_lun, /append

  IF OBJ_VALID(infile) THEN BEGIN ;; case of TSD object
    mesg = 'GB procedures started (infile is an object)'
  ENDIF ELSE IF (STRMATCH(infile, 'LDS', /FOLD_CASE) NE 1) THEN BEGIN ;; case of a local TSD file
    mesg = 'GB procedures started ('+infile+')'
    IF (KEYWORD_SET(gb_module) NE 0) THEN mesg = 'GB ('+gb_module+') started ('+infile+')'
  ENDIF ELSE BEGIN ;; case of LDS
    IF ((N_ELEMENTS(stime) EQ 0) OR (N_ELEMENTS(span) EQ 0) OR (N_ELEMENTS(format) EQ 0)) THEN BEGIN
      MESSAGE, 'Keywords for LDS are not enough.',/CONTINUE
      MESSAGE, 'Keywords TIME, SPAN, FORMAT are needed. MODE is an option (default "Lab")',/CONTINUE
      MESSAGE, "e.g. >r=gb_run_gb('LDS', time='2006-04-15 17:10:00', span='60', format='tsd4_sw', mode='Fli')",/CONTINUE
      RETURN, -1
    ENDIF
    mesg = 'GB procedures started ('+infile+' '+stime+' '+span+'sec '+format+')'
  ENDELSE

  printf, lun, mesg
  printf, lun, time_elps, time_start_sec - time_start_sec
  free_lun, lun

ENDIF


;;----------------------------------------------------------
;; Read TSD file
;;----------------------------------------------------------
  IF OBJ_VALID(infile) THEN BEGIN
  ;; If infile is not a string but a TSD object reference
    ; Copy TSD object reference
    tsd = infile

  ENDIF ELSE IF (STRMATCH(infile, 'LDS', /FOLD_CASE) EQ 1) THEN BEGIN
  ;;; Open from LDS

    IF ((N_ELEMENTS(stime) EQ 0) OR (N_ELEMENTS(span) EQ 0) OR (N_ELEMENTS(format) EQ 0)) THEN BEGIN
      MESSAGE, 'Keywords for LDS are not enough.',/CONTINUE
      MESSAGE, 'Keywords TIME, SPAN, FORMAT are needed. MODE is an option (default "Lab")',/CONTINUE
      MESSAGE, "e.g. >r=gb_run_gb('LDS', time='2006-04-15 17:10:00', span='60', format='tsd4_sw', mode='Fli')",/CONTINUE
      RETURN, -1
    ENDIF
    IF (N_ELEMENTS(mode) EQ 0) THEN mode='Fli' ; Default
    tsd = Obj_New('tsd')
    ret = tsd->read_tsd_from_lds(time=stime, span=span, format=format, mode=mode, /COMPRESS)

    ; In the case that the LDS cannot find any data.
    IF (size(ret, /type) NE 7) THEN BEGIN
      LMESSAGE, 1, 'LDS could not find any data.', /CONTINUE
      OBJ_DESTROY, tsd
      RETURN, -1
    ENDIF

  ENDIF ELSE BEGIN
  ;;; Open a local file

    openr, 1, infile, error=err
    IF (err NE 0) THEN BEGIN
      LMESSAGE, 1, 'File not found', /CONTINUE
      LMESSAGE, 1, 'Exitting pipeline...', /CONTINUE
      RETURN, -1
    ENDIF
    close, 1

    tsd=Obj_New('tsd', INFILE=infile)

  ENDELSE

  ;; Recording elapsed-times
  IF KEYWORD_SET(elapsed_time) EQ 1 THEN BEGIN
    time_elps = systime()
    time_elps_sec = systime(1)
    openw, lun, fname_elapsed, width=100, /get_lun, /append
    printf, lun, time_elps, time_elps_sec - time_start_sec, ' File opened'
    free_lun, lun
  ENDIF


;;----------------------------------------------------------
;; Start procedures
;;    Show a message and add header strings (PIPELINE and HISTORY)
;;----------------------------------------------------------
;;; Running each procedure
   LMESSAGE, 5, 'Now, starting Green Box modules...', /CONTINUE
   name_ver = gb_get_name_ver(infile='gb_run_gb')
   hist_message = STRCOMPRESS(name_ver+' started')

   dummy = tsd_fxaddhist(tsd, hist_message, /curtime)
   IF dummy NE 0 THEN LMESSAGE, 3, 'Unable to write "HISTORY" in the header', /CONTINUE

   dummy = tsd->set_header_keyword_tsd('Primary', 'Pipeline', VER_PIPELINE)
   IF dummy NE 0 THEN LMESSAGE, 3, 'Unable to write "PIPELINE" in the header', /CONTINUE

;;----------------------------------------------------------
;; Initialize Enable_GB option
;;----------------------------------------------------------
  IF N_Elements(enable_gb) EQ 0 THEN BEGIN
    enable_gb = [1,2,3,4,5,6,7,8,9,10,11,12];; all module are enabled by default
  ENDIF

;;----------------------------------------------------------
;; If 'GB_Module' option was set, run in the single module mode
;; A selected pipeline module is performed.
;;----------------------------------------------------------
IF KEYWORD_SET(gb_module) EQ 1 THEN BEGIN

  IF STRMATCH(gb_module, '*Bad*', /Fold_Case) THEN BEGIN
    enable_gb=[1]
  ENDIF ELSE IF STRMATCH(gb_module, '*Rstanom*', /Fold_Case) THEN BEGIN
    enable_gb=[2]
  ENDIF ELSE IF STRMATCH(gb_module, '*Conv*', /Fold_Case) THEN BEGIN
    enable_gb=[3]
  ENDIF ELSE IF STRMATCH(gb_module, '*Ramp*', /Fold_Case) THEN BEGIN
    enable_gb=[4]
  ENDIF ELSE IF STRMATCH(gb_module, '*Diff*', /Fold_Case) THEN BEGIN
    enable_gb=[5]
  ENDIF ELSE IF STRMATCH(gb_module, '*Peri*', /Fold_Case) THEN BEGIN
    enable_gb=[6]
  ENDIF ELSE IF STRMATCH(gb_module, '*MMT*', /Fold_Case) THEN BEGIN
    enable_gb=[7]
  ENDIF ELSE IF STRMATCH(gb_module, '*GP*', /Fold_Case) THEN BEGIN
    enable_gb=[8]
  ENDIF ELSE IF STRMATCH(gb_module, '*Lamp_T*', /Fold_Case) THEN BEGIN
    enable_gb=[9]
  ENDIF ELSE IF STRMATCH(gb_module, '*Dark_T*', /Fold_Case) THEN BEGIN
    enable_gb=[10]
  ENDIF ELSE IF STRMATCH(gb_module, '*Dark_S*', /Fold_Case) THEN BEGIN
    enable_gb=[11]
  ENDIF ELSE IF STRMATCH(gb_module, '*Resp*', /Fold_Case) THEN BEGIN
    enable_gb=[12]
  ENDIF ELSE IF STRMATCH(gb_module, '*Flux*', /Fold_Case) THEN BEGIN
    enable_gb=[13]
  ENDIF ELSE BEGIN
    msg = '!!! GB_MODULE NAME (' + gb_module + ') mismatch !!!'
    LMESSAGE, 1, msg, /CONTINUE
    msg = 'Green Box, '+name_ver+' END UP in FAILURE '+'at '+systime()
    LMESSAGE, 1, msg, /CONTINUE

    IF (~OBJ_VALID(infile)) AND (OBJ_VALID(tsd)) THEN OBJ_DESTROY, tsd
    IF !JOURNAL NE 0 THEN JOURNAL ; Stop Journalating

    Return, !FAIL
  ENDELSE

ENDIF

;;==========================================================
;; Pipeline modules are performed in turn.
;;==========================================================
  error_code = 0 ;; Initialization

  IF total(enable_gb EQ 1) THEN BEGIN
    error_code = gb_flag_bad_data(tsd, $
      ccf_deadpix = ccf_deadpix, ccf_satpix = ccf_satpix, $
      ccf_moon=ccf_moon, ccf_saa=ccf_saa, _extra=_extra)
    IF KEYWORD_SET(save_at_all_process) EQ 1 THEN BEGIN
      result = tsd->Write_tsd('gb_flag_bad_data.fits')
      LMESSAGE, 5, 'Processed data written in gb_flag_bad_data.fits', /CONTINUE
    ENDIF
    ;; Recording elapsed-times
    IF KEYWORD_SET(elapsed_time) EQ 1 THEN BEGIN
      time_elps = systime()
      time_elps_sec = systime(1)
      openw, lun, fname_elapsed, width=100, /get_lun, /append
      printf, lun, time_elps, time_elps_sec - time_start_sec, ' GB_Flag_Bad_Data finished'
      free_lun, lun
    ENDIF
  ENDIF

  IF (error_code EQ 0) AND total(enable_gb EQ 2) THEN BEGIN
    error_code = gb_set_flag_rstanom(tsd, ccf_rstanom = ccf_rstanom, _extra=_extra)
    IF KEYWORD_SET(save_at_all_process) EQ 1 THEN BEGIN
      result = tsd->Write_tsd('gb_set_flag_rstanom.fits')
      LMESSAGE, 5, 'Processed data written in gb_set_flag_rstanom.fits', /CONTINUE
    ENDIF
    ;; Recording elapsed-times
    IF KEYWORD_SET(elapsed_time) EQ 1 THEN BEGIN
      time_elps = systime()
      time_elps_sec = systime(1)
      openw, lun, fname_elapsed, width=100, /get_lun, /append
      printf, lun, time_elps, time_elps_sec - time_start_sec, ' GB_Set_Flag_Rstanom finished'
      free_lun, lun
    ENDIF
  ENDIF

  IF (error_code EQ 0) AND total(enable_gb EQ 3) THEN BEGIN
    error_code = gb_conv_to_volt(tsd, ccf_thresh = ccf_thresh, ferr=c2v_ferr, _extra=_extra)
    IF KEYWORD_SET(save_at_all_process) EQ 1 THEN BEGIN
      result = tsd->Write_tsd('gb_conv_to_volt.fits')
      LMESSAGE, 5, 'Processed data written in gb_conv_to_volt.fits', /CONTINUE
    ENDIF
    ;; Recording elapsed-times
    IF KEYWORD_SET(elapsed_time) EQ 1 THEN BEGIN
      time_elps = systime()
      time_elps_sec = systime(1)
      openw, lun, fname_elapsed, width=100, /get_lun, /append
      printf, lun, time_elps, time_elps_sec - time_start_sec, ' GB_Conv_to_Volt finished'
      free_lun, lun
    ENDIF
  ENDIF

  IF (error_code EQ 0) AND total(enable_gb EQ 4) THEN BEGIN
    IF (n_elements(rc_noplot) EQ 0) THEN rc_noplot = 1
    error_code = gb_ramp_curve_corr(tsd, CCF_FILE_RCTB = ccf_rctb, $
        NOPLOT = rc_noplot, _extra=_extra)
    IF KEYWORD_SET(save_at_all_process) EQ 1 THEN BEGIN
      result = tsd->Write_tsd('gb_ramp_curve_corr.fits')
      LMESSAGE, 5, 'Processed data written in gb_ramp_curve_corr.fits', /CONTINUE
    ENDIF
    ;; Recording elapsed-times
    IF KEYWORD_SET(elapsed_time) EQ 1 THEN BEGIN
      time_elps = systime()
      time_elps_sec = systime(1)
      openw, lun, fname_elapsed, width=100, /get_lun, /append
      printf, lun, time_elps, time_elps_sec - time_start_sec, ' GB_Ramp_Curve_Corr finished'
      free_lun, lun
    ENDIF
  ENDIF

  IF (error_code EQ 0) AND total(enable_gb EQ 5) THEN BEGIN
    error_code = gb_differentiation(tsd, $
        diff_mod = diff_mod, shiftvl = shiftvl, disp = diff_disp, $
         rdvdet=rdvdet, _extra=_extra)
    IF KEYWORD_SET(save_at_all_process) EQ 1 THEN BEGIN
      result = tsd->Write_tsd('gb_differentiation.fits')
      LMESSAGE, 5, 'Processed data written in gb_differentiation.fits', /CONTINUE
    ENDIF
    ;; Recording elapsed-times
    IF KEYWORD_SET(elapsed_time) EQ 1 THEN BEGIN
      time_elps = systime()
      time_elps_sec = systime(1)
      openw, lun, fname_elapsed, width=100, /get_lun, /append
      printf, lun, time_elps, time_elps_sec - time_start_sec, ' GB_Differentiation finished'
      free_lun, lun
    ENDIF
  ENDIF

  IF (error_code EQ 0) AND total(enable_gb EQ 6) THEN BEGIN
    error_code = gb_periodic_noise(tsd, _extra=_extra)
    IF KEYWORD_SET(save_at_all_process) EQ 1 THEN BEGIN
      result = tsd->Write_tsd('gb_periodic_noise.fits')
      LMESSAGE, 5, 'Processed data written in gb_periodic_noise.fits', /CONTINUE
    ENDIF
    ;; Recording elapsed-times
    IF KEYWORD_SET(elapsed_time) EQ 1 THEN BEGIN
      time_elps = systime()
      time_elps_sec = systime(1)
      openw, lun, fname_elapsed, width=100, /get_lun, /append
      printf, lun, time_elps, time_elps_sec - time_start_sec, ' GB_Periodic_Noise finished'
      free_lun, lun
    ENDIF
  ENDIF

  IF (error_code EQ 0) AND total(enable_gb EQ 7) THEN BEGIN
    error_code = gb_set_glitch_status_mmt(tsd, Output_Glitch=output_glitch, Debug=debug, _extra=_extra)
    IF KEYWORD_SET(save_at_all_process) EQ 1 THEN BEGIN
      result = tsd->Write_tsd('gb_set_glitch_status_mmt.fits')
      LMESSAGE, 5, 'Processed data written in gb_set_glitch_status_mmt.fits', /CONTINUE
    ENDIF
    ;; Recording elapsed-times
    IF KEYWORD_SET(elapsed_time) EQ 1 THEN BEGIN
      time_elps = systime()
      time_elps_sec = systime(1)
      openw, lun, fname_elapsed, width=100, /get_lun, /append
      printf, lun, time_elps, time_elps_sec - time_start_sec, ' GB_Set_Glitch_Status_MMT finished'
      free_lun, lun
    ENDIF
  ENDIF

  IF (error_code EQ 0) AND total(enable_gb EQ 8) THEN BEGIN
    ;IF (n_elements(glitchlist_gp) EQ 0) THEN glitchlist_gp = 'Glitchlist_GP.dat'
    error_code = gb_set_glitch_status_gp(tsd, Calib_Const_File = ccf_glitch_gp, $
        GlitchList = glitchlist_gp, Pixel = pixel_gp, Plot=plot_gp, METHOD = method_gb, _extra=_extra)
    IF KEYWORD_SET(save_at_all_process) EQ 1 THEN BEGIN
      result = tsd->Write_tsd('gb_set_glitch_status_gp.fits')
      LMESSAGE, 5, 'Processed data written in gb_set_glitch_status_gp.fits', /CONTINUE
    ENDIF
    ;; Recording elapsed-times
    IF KEYWORD_SET(elapsed_time) EQ 1 THEN BEGIN
      time_elps = systime()
      time_elps_sec = systime(1)
      openw, lun, fname_elapsed, width=100, /get_lun, /append
      printf, lun, time_elps, time_elps_sec - time_start_sec, ' GB_Set_Glitch_Status_GP finished'
      free_lun, lun
    ENDIF
  ENDIF

  IF (error_code EQ 0) AND total(enable_gb EQ 9) THEN BEGIN
    error_code = gb_make_dc_cal_lamp_table(tsd, _extra=_extra)
    IF KEYWORD_SET(save_at_all_process) EQ 1 THEN BEGIN
      result = tsd->Write_tsd('gb_make_dc_cal_lamp_table.fits')
      LMESSAGE, 5, 'Processed data written in gb_make_dc_cal_lamp_table.fits', /CONTINUE
    ENDIF
    ;; Recording elapsed-times
    IF KEYWORD_SET(elapsed_time) EQ 1 THEN BEGIN
      time_elps = systime()
      time_elps_sec = systime(1)
      openw, lun, fname_elapsed, width=100, /get_lun, /append
      printf, lun, time_elps, time_elps_sec - time_start_sec, ' GB_Make_DC_Cal_Lamp_Table finished'
      free_lun, lun
    ENDIF
  ENDIF

  IF (error_code EQ 0) AND total(enable_gb EQ 10) THEN BEGIN
    error_code = gb_make_dark_table(tsd, _extra=_extra)
    IF KEYWORD_SET(save_at_all_process) EQ 1 THEN BEGIN
      result = tsd->Write_tsd('gb_make_dark_table.fits')
      LMESSAGE, 5, 'Processed data written in gb_make_dark_table.fits', /CONTINUE
    ENDIF
    ;; Recording elapsed-times
    IF KEYWORD_SET(elapsed_time) EQ 1 THEN BEGIN
      time_elps = systime()
      time_elps_sec = systime(1)
      openw, lun, fname_elapsed, width=100, /get_lun, /append
      printf, lun, time_elps, time_elps_sec - time_start_sec, ' GB_Make_Dark_Table finished'
      free_lun, lun
    ENDIF
  ENDIF

  IF (error_code EQ 0) AND total(enable_gb EQ 11) THEN BEGIN
    error_code = gb_dark_subtraction(tsd, DISPLAY = disp_dark, _extra=_extra)
    IF KEYWORD_SET(save_at_all_process) EQ 1 THEN BEGIN
      result = tsd->Write_tsd('gb_dark_subtraction.fits')
      LMESSAGE, 5, 'Processed data written in gb_dark_subtraction.fits', /CONTINUE
    ENDIF
    ;; Recording elapsed-times
    IF KEYWORD_SET(elapsed_time) EQ 1 THEN BEGIN
      time_elps = systime()
       time_elps_sec = systime(1)
      openw, lun, fname_elapsed, width=100, /get_lun, /append
      printf, lun, time_elps, time_elps_sec - time_start_sec, ' GB_Dark_Subtraction finished'
      free_lun, lun
    ENDIF
  ENDIF

  IF (error_code EQ 0) AND total(enable_gb EQ 12) THEN BEGIN
    error_code = gb_dc_responsivity_corr(tsd, RESULT_OUT = result_dc_resp, _extra=_extra)
    IF KEYWORD_SET(save_at_all_process) EQ 1 THEN BEGIN
      result = tsd->Write_tsd('gb_dc_responsivity_corr.fits')
      LMESSAGE, 5, 'Processed data written in gb_dc_responsivity_corr.fits', /CONTINUE
    ENDIF
    ;; Recording elapsed-times
    IF KEYWORD_SET(elapsed_time) EQ 1 THEN BEGIN
      time_elps = systime()
      time_elps_sec = systime(1)
      openw, lun, fname_elapsed, width=100, /get_lun, /append
      printf, lun, time_elps, time_elps_sec - time_start_sec, ' GB_DC_Responsivity_Corr finished'
      free_lun, lun
    ENDIF
  ENDIF

;  IF error_code EQ 0 THEN BEGIN
;    error_code = gb_transient_corr(tsd, calib_const_file = ccf_transient)
;    IF KEYWORD_SET(save_at_all_process) EQ 1 THEN BEGIN
;      result = tsd->Write_tsd('gb_transient_corr.fits')
;      MESSAGE,'Processed data written in gb_transient_corr.fits',/CONTINUE
;    ENDIF
;  ENDIF

  IF (error_code EQ 0) AND total(enable_gb EQ 13) THEN BEGIN
    error_code = gb_flux_calib(tsd, ccf_calib = ccf_calib, ccf_badpix = ccf_badpix, _extra=_extra)
    IF KEYWORD_SET(save_at_all_process) EQ 1 THEN BEGIN
      result = tsd->Write_tsd('gb_flux_calib.fits')
      LMESSAGE, 5, 'Processed data written in gb_flux_calib.fits', /CONTINUE
    ENDIF
    ;; Recording elapsed-times
    IF KEYWORD_SET(elapsed_time) EQ 1 THEN BEGIN
      time_elps = systime()
      time_elps_sec = systime(1)
      openw, lun, fname_elapsed, width=100, /get_lun, /append
      printf, lun, time_elps, time_elps_sec - time_start_sec, ' GB_Flux_Calib finished'
      free_lun, lun
    ENDIF
  ENDIF
;;==========================================================

;;----------------------------------------------------------
;; Write TSD file
;;----------------------------------------------------------

  IF KEYWORD_SET(outfile) NE 1 THEN BEGIN
     result = tsd->Write_tsd('out.fits')
     LMESSAGE, 5, 'Processed data written in out.fits', /CONTINUE
  ENDIF ELSE IF (size(outfile, /type) NE 7) THEN BEGIN
     IF (outfile EQ -1) THEN BEGIN
       ;; suppress writing an output file
       LMESSAGE, 5, 'Output file has not been saved.', /CONTINUE
     ENDIF
  ENDIF ELSE BEGIN
     IF (size(outfile, /type) NE 7) THEN outfile=strcompress(string(outfile), /remove_all)+'.fits'
     result = tsd->Write_tsd(outfile)
     LMESSAGE, 5, 'Processed data written in '+outfile, /CONTINUE
  ENDELSE

;;----------------------------------------------------------
;; Terminate processing
;;----------------------------------------------------------

  ; Destroy a tsd object. But, does not destroy it when infile is a tsd object.
  IF (~OBJ_VALID(infile)) AND (OBJ_VALID(tsd)) THEN OBJ_DESTROY, tsd

  ; Display a message
  msg='Green Box, '+name_ver+' Done '+'at '+systime()
  LMESSAGE, 5, msg, /CONTINUE

  ; Stop Journaling
  IF KEYWORD_SET(log) EQ 1 THEN BEGIN
    IF !JOURNAL NE 0 THEN JOURNAL
  ENDIF

  ;; Recording elapsed-times
  IF KEYWORD_SET(elapsed_time) EQ 1 THEN BEGIN
    time_elps = systime()
    time_elps_sec = systime(1)
    openw, lun, fname_elapsed, width=100, /get_lun, /append
    printf, lun, time_elps, time_elps_sec - time_start_sec, ' GB procedures finished'
    free_lun, lun
  ENDIF

  Return, !SUCCEED
END
