; $Id: gb_transient_corr.pro,v 1.9 2006/01/18 06:57:05 baba Exp $
;
; Copyright (c) 2002, ASTRO-F Data Reduction Team.  All rights reserved.
;
;+
; NAME:
;	gb_transient_corr.pro
; PURPOSE:
;	The transient effects of the detector and the cosmic-ray
;	after effects are corrected. 
; CATEGORY:
;	Function. Green Box procedure in ASTRO-F pipeline.
; CALLING SEQUENCE:
;	Result = gb_transient_corr(tsd
;	[, CAL_CONST_FILE = cal_const_file, SUMMARY = summary])
; INPUTS:
;	tsd		TSD object reference.
;	cal_const_file	Calibration constants file; 
;			format, usage are TBD (optional.)
; OUTPUTS:
;	tsd		TSD object reference. Transient corrected TSD.
;	(SUMMARY = summary)	Summary of the procedure.
;			How to manage the summaries is under 
;			considerations.
; RETURN:
;	!SUCCEED if function ended successfully.
;	!FAIL in failure.
; RESTRICTIONS:
;	
; PROCEDURE:
;	  In this pipeline process, the model reported by Kaneda 
;	is imported to correct the transient and cross-talk effects. 
;	The model is as follows:
;		 The transient response for the step change of the 
;		incoming flux can be expressed with two-component 
;		exponential function,
;
;		Delta J(t)/Delta ~J = A+B_1(1-exp(-t/tau _1))
;				+ B_2(1-exp(-t/tau _2)).	(1)
;
;		Delta ~J is the response function without transient.
;		The second term with subscript 1 and the third with 
;		2 represent long- and short- term transient component, 
;		respectively. These two components depend on the 
;		irradiation environment. Under the "normal" situation, 
;		tau_1 and tau_2 are approximately several tens of 
;		seconds and several seconds, respectively. B_1 and B_2 
;		are of the order of 15-20 %. A = 1- B_1 - B_2.
;		If the linear response function is assumed, the corrected
;		photocurrent can be described as,
;
;		J^i(t_n) = sum [r^i(t_n - t_k) ~J^i(t_k) Delta t, (2)
;
;		where the summation is carried out from i = 0 to n and 
;		Delta t = t_k - t_k-1.	r^i(t)is the correction function
;		of the i-th detector element, and given by eq.(1),
;		corresponding to the photon environment at t_k. 
;		Solving the above series function from k = 0, i.e. 
;		shutter is closed, to n gives the corrected photocurrent 
;		~J^i(t_k).
;
;		The cross-talk caused by the detector j to the detector 
;		i can be described as 
;
;		J^i(t_n) = 
;		sum_j{sum_{k=0}^{n}[c_ij r^j(t_n-t_k)J^j(t_k)Delta t]}.
;
;
;	1. The procedure is made and tested including the effects as 
;	follows before the launch;
;		1. transient without cross-talk
;		2. transient with stationary cross-talk 
;		3. transient with non-stationary cross-talk.
;	2. It is possible that SAA is passed while a dataset is taken 
;	if a period between the shutter close is long; It is considered 
;	to close shutter after every SAA.
;	3 Items to be determined from the laboratory measurements:
;		1. Model of the correction algorithm
;		2. Implementation of Cross-talk.
;			2.1. Cross-talk parameter, e.g., c_ij in eq. (3)
;			2.2. If the transient and cross-talk can be 
;			treated separately:If the cross-talk has DC-like 
;			behaviour, transient and cross-talk can be 
;			separated.
; EXAMPLE:
;
; REFERENCE:
;	Baba et al. 2002 Nov. 21, ``ASTRO-F/FIS Survey Data Structure"
;	Kaneda, internal document in Japanese on transient and 
;	cross-talk correction(2002 Apr. 23).
; MODIFICATION HISTORY:
; 	Sept. 2002		Written by A. Kawamura.
;	Dec. 2002		Header is added.
;-

FUNCTION gb_transient_corr, tsd, calib_const_file = ccf
COMPILE_OPT IDL2
;ON_ERROR 2
;;; Starting GB_Transient_Corr ;;;
	MESSAGE,'Executing gb_transient_corr...',/CONTINUE

;;; Writing messages ;;;;
;    re = gb_wr_start_msg('gb_transient_corr')

;;; Extract values from TSD object reference ;;;
        in_vdet=tsd->get_value_tsd('fis_obs','flux')
        err=af_finite(in_vdet, vdet)
        ti_ori=tsd->get_value_tsd('fis_obs','aftime')

  ;;; number of channels ;;;
     si = SIZE(vdet)
     num_ch = si[1]
     num_data = si[2]

   cal_vdet=vdet

   bad=tsd->get_pix_flag_tsd('fis_obs','bad')
   
FOR n = 0, 14 DO BEGIN

 ;;; interp. at bad pix position
   ind_bad=WHERE(bad[n,*] NE 0, cnt_bad)

   IF cnt_bad NE 0 THEN BEGIN
      ind_no_bad=WHERE(bad[n,*] EQ 0, cnt_no_bad)
      IF cnt_no_bad EQ 0 THEN BEGIN
         msg='Only unusable data. No data to be corrected...'
         msg=STRCOMPRESS(STRING(n+1)+'-th pixel:'+msg)
         MESSAGE,msg,/CONTINUE
      ENDIF ELSE BEGIN
         vdet_no_bad=reform(vdet[n,ind_no_bad])
         ti_n=ti_ori[ind_no_bad]
         v_interp=INTERPOL(vdet_no_bad, ti_n, ti_ori)
         
    ;;; Correction for each pixel ;;;
         msg=STRCOMPRESS('Correcting '+STRING(n+1)+'-th pixel')
         MESSAGE,msg,/CONTINUE
         cal_vdet[n,*]=gb_tr_hk1(v_interp, num_ch)
      ENDELSE
    ENDIF ELSE BEGIN
      vdetp=dblarr(num_data)
      vdetp[*]=reform(vdet[n,*])
      cal_vdet[n,*]=gb_tr_hk1(vdetp, num_ch)
    ENDELSE
ENDFOR

FOR n = 15, 29 DO BEGIN

 ;;; interp. at bad pix position
   ind_bad=WHERE(bad[n,*] NE 0, cnt_bad)

   IF cnt_bad NE 0 THEN BEGIN
      ind_no_bad=WHERE(bad[n,*] EQ 0, cnt_no_bad)
      IF cnt_no_bad EQ 0 THEN BEGIN
         msg='Only unusable data. No data to be corrected...'
         msg=STRCOMPRESS(STRING(n+1)+'-th pixel:'+msg)
         MESSAGE,msg,/CONTINUE
      ENDIF ELSE BEGIN
         vdet_no_bad=reform(vdet[n,ind_no_bad])
         ti_n=ti_ori[ind_no_bad]
         v_interp=INTERPOL(vdet_no_bad, ti_n, ti_ori)
         
    ;;; Correction for each pixel ;;;
         msg=STRCOMPRESS('Correcting '+STRING(n+1)+'-th pixel')
         MESSAGE,msg,/CONTINUE
         cal_vdet[n,*]=gb_tr_hk1(v_interp, num_ch)
      ENDELSE
    ENDIF ELSE BEGIN
      vdetp=dblarr(num_data)
      vdetp[*]=reform(vdet[n,*])
      cal_vdet[n,*]=gb_tr_hk1(vdetp, num_ch)
    ENDELSE
ENDFOR

FOR n = 30, 59 DO BEGIN

 ;;; interp. at bad pix position
   ind_bad=WHERE(bad[n,*] NE 0, cnt_bad)

   IF cnt_bad NE 0 THEN BEGIN
      ind_no_bad=WHERE(bad[n,*] EQ 0, cnt_no_bad)
      IF cnt_no_bad EQ 0 THEN BEGIN
         msg='Only unusable data. No data to be corrected...'
         msg=STRCOMPRESS(STRING(n+1)+'-th pixel:'+msg)
         MESSAGE,msg,/CONTINUE
      ENDIF ELSE BEGIN
         vdet_no_bad=reform(vdet[n,ind_no_bad])
         ti_n=ti_ori[ind_no_bad]
         v_interp=INTERPOL(vdet_no_bad, ti_n, ti_ori)
         
    ;;; Correction for each pixel ;;;
         msg=STRCOMPRESS('Correcting '+STRING(n+1)+'-th pixel')
         MESSAGE,msg,/CONTINUE
         cal_vdet[n,*]=gb_tr_hk1(v_interp, num_ch)
      ENDELSE
    ENDIF ELSE BEGIN
      vdetp=dblarr(num_data)
      vdetp[*]=reform(vdet[n,*])
      cal_vdet[n,*]=gb_tr_hk1(vdetp, num_ch)
    ENDELSE
ENDFOR

FOR n = 60, 72 DO BEGIN

 ;;; interp. at bad pix position
   ind_bad=WHERE(bad[n,*] NE 0, cnt_bad)

   IF cnt_bad NE 0 THEN BEGIN
      ind_no_bad=WHERE(bad[n,*] EQ 0, cnt_no_bad)
      IF cnt_no_bad EQ 0 THEN BEGIN
         msg='Only unusable data. No data to be corrected...'
         msg=STRCOMPRESS(STRING(n+1)+'-th pixel:'+msg)
         MESSAGE,msg,/CONTINUE
      ENDIF ELSE BEGIN
         vdet_no_bad=reform(vdet[n,ind_no_bad])
         ti_n=ti_ori[ind_no_bad]
         v_interp=INTERPOL(vdet_no_bad, ti_n, ti_ori)
         
    ;;; Correction for each pixel ;;;
         msg=STRCOMPRESS('Correcting '+STRING(n+1)+'-th pixel')
         MESSAGE,msg,/CONTINUE
         cal_vdet[n,*]=gb_tr_hk1(v_interp, num_ch)
      ENDELSE
    ENDIF ELSE BEGIN
      vdetp=dblarr(num_data)
      vdetp[*]=reform(vdet[n,*])
      cal_vdet[n,*]=gb_tr_hk1(vdetp, num_ch)
    ENDELSE
ENDFOR

FOR n = 73, num_ch-1 DO BEGIN

 ;;; interp. at bad pix position
   ind_bad=WHERE(bad[n,*] NE 0, cnt_bad)

   IF cnt_bad NE 0 THEN BEGIN
      ind_no_bad=WHERE(bad[n,*] EQ 0, cnt_no_bad)
      IF cnt_no_bad EQ 0 THEN BEGIN
         msg='Only unusable data. No data to be corrected...'
         msg=STRCOMPRESS(STRING(n+1)+'-th pixel:'+msg)
         MESSAGE,msg,/CONTINUE
      ENDIF ELSE BEGIN
         vdet_no_bad=reform(vdet[n,ind_no_bad])
         ti_n=ti_ori[ind_no_bad]
         v_interp=INTERPOL(vdet_no_bad, ti_n, ti_ori)
         
    ;;; Correction for each pixel ;;;
         msg=STRCOMPRESS('Correcting '+STRING(n+1)+'-th pixel')
         MESSAGE,msg,/CONTINUE
         cal_vdet[n,*]=gb_tr_hk1(v_interp, num_ch)
      ENDELSE
    ENDIF ELSE BEGIN
      vdetp=dblarr(num_data)
      vdetp[*]=reform(vdet[n,*])
      cal_vdet[n,*]=gb_tr_hk1(vdetp, num_ch)
    ENDELSE
ENDFOR



;;; Write TSD ;;;
   MESSAGE,'Writing transient corrected TSD...',/CONTINUE
   dummy = tsd->set_value_tsd('fis_obs','flux', cal_vdet)

     msg="Setting flag, arith_err"
     MESSAGE,msg,/CONTINUE

      ind_nan = WHERE (FINITE(cal_vdet, /NAN), count_nan)
      ind_inf = WHERE (FINITE(cal_vdet, /INFINITY), count_inf)
      msg=STRCOMPRESS('Number of NaN ... '+STRING(count_nan))
      MESSAGE,msg,/CONTINUE
      msg=STRCOMPRESS('Number of INFINITE ... '+STRING(count_inf))
      MESSAGE,msg,/CONTINUE   
      arith_err=bytarr(num_ch,num_data)
      IF count_nan NE 0 THEN arith_err[ind_nan] = 1
      IF count_inf NE 0 THEN arith_err[ind_inf] = 1
      ind = WHERE(arith_err NE 0, cnt)
      IF cnt NE 0 THEN arith_err[ind]=1
      ret = tsd->set_pix_flag_tsd('fis_obs','arith_err',arith_err)

     msg="Setting flag, no_tr_corr..."
     MESSAGE,msg,/CONTINUE

      no_tr_corr=tsd->get_pix_flag_tsd('fis_obs','no_tr_corr')
      dummy = bad + no_tr_corr + arith_err
      ind = WHERE (dummy NE 0, cnt)
      IF cnt NE 0 THEN dummy[ind]=1
      ret=tsd->set_pix_flag_tsd('fis_obs','no_tr_corr',dummy)

     msg="Setting flag, bad..."
     MESSAGE,msg,/CONTINUE

     ret=tsd->set_pix_flag_tsd('fis_obs','bad',dummy)
;;; Write HISTORY in header ;;;

   MESSAGE,'Writing HISTORY',/CONTINUE
   name_ver=gb_get_name_ver(infile='gb_transient_corr')     
   hist_message = STRCOMPRESS(' '+name_ver+' done. CCF= ')
   IF KEYWORD_SET(ccf) NE 1 THEN BEGIN
      hist_message = hist_message + 'NONE'
   ENDIF ELSE BEGIN
      hist_message = hist_message+ccf
   ENDELSE

   dummy = tsd_fxaddhist(tsd, hist_message, /prres)
   IF dummy NE 0 THEN $
     MESSAGE,'Unable to write HISTORY',/CONTINUE


	return, !SUCCEED
END
