; $Id: im_detcoad.pro,v 1.5 2006/05/11 05:39:46 do Exp $
;
; Copyright (c) 2002, ASTRO-F Data Reduction Team.  All rights reserved.
;
;+
; NAME:
;	im_detcoad.pro
; PURPOSE:
;	coadd one detector scan into the mapset struct
; CATEGORY:
;	Function.
; CALLING SEQUENCE:
;	Result = im_detector( flux, x, y, mapset, mapinfo, TBD )
;
; INPUTS:
;	flux		valid flux for this det
;	x, y		pertaining projected coordinates in pixel values
;	mapinfo 	map information
; OUTPUTS:
;	mapset		update of contents
; RETURN
;	!SUCCEED if function ended successfully.
;	!FAIL in failure.
; RESTRICTIONS:
;
; PROCEDURE:
;	Add flux to all pixels that fall within a beam's distance
;	from the nominal positions of flux in the coadd-map.
;	Increment the same pixels in the count-map and add the square
;	of the fluxes into the pixels of the stdev-map
;
; EXAMPLE:
;
; REFERENCE:
;	Baba et al. 2002 Nov. 21, ``ASTRO-F/FIS Survey Data Structure"
;
; MODIFICATION HISTORY:
;	Oct. 2004		Written by Do Kester
;-

FUNCTION im_detcoad, flux, wgt, xpos, ypos, mapset, mapinfo

COMPILE_OPT IDL2

;   ON_ERROR 2

    pixsize = mapinfo.size / mapinfo.pixels
    beam = mapinfo.beam / pixsize
    beam2 = beam[0] * beam[1]

    npt = n_elements( flux )
    if ( npt LE 1 ) then return, !SUCCEED

    dist = ( abs( xpos[0] - xpos[npt-1] ) + $
	     abs( ypos[0] - ypos[npt-1] ) ) / ( 2.0 * npt )
    nd = ceil( 1.0 / dist )

    ni = npt / nd + 1
    ind = lindgen( ni ) * nd

    if ( ind[ni-1] GE npt ) then begin
	ni--
	if ( ni EQ 0 ) then return, !SUCCEED
	ind = ind[0:ni-1]
    endif

    for d = 0, nd-1 do begin
	ff = flux[ind]
	ww = wgt[ind]
	xx = xpos[ind]
	yy = ypos[ind]

	extrem = fix( beam )
	jj = fix( yy - extrem[1] )
	y2 = jj - yy
	dy2 = 2 * y2 - 1
	y2 *= y2
	for j = -extrem[1], extrem[1] do begin
	    kk = fix( xx - extrem[0] )
	    x2 = kk - xx
	    dx2 = 2 * x2 - 1
	    x2 *= x2
	    dist2 = x2 + y2
	    for k = -extrem[0], extrem[0] do begin
		bb = where( dist2 LT beam2 AND $
			kk GE 0 AND kk LT mapinfo.pixels[0] AND $
			jj GE 0 AND jj LT mapinfo.pixels[1], nbb )

		if ( nbb GT 0 ) then begin
		    kb = kk[bb]
		    jb = jj[bb]
		    wb = ww[bb]
		    fb = ff[bb]
		    mapset.count[kb,jb] += 1
		    mapset.mom0[kb,jb] += wb
		    mapset.mom1[kb,jb] += wb * fb
		    mapset.mom2[kb,jb] += wb * fb * fb
		endif
		kk++
		dx2 += 2
		x2 += dx2
		dist2 = x2 + y2
	    endfor
	    jj++
	    dy2 += 2
	    y2 += dy2
	endfor
	ind = ind + 1
    endfor

    return, !SUCCEED

END


