; $Id: im_scan.pro,v 1.4 2006/05/11 05:39:46 do Exp $
;
; Copyright (c) 2002, ASTRO-F Data Reduction Team.  All rights reserved.
;
;+
; NAME:
;	im_scan.pro
; PURPOSE:
;	process one complete scan
; CATEGORY:
;	Function.
; CALLING SEQUENCE:
;	Result = im_scan( tsd, map, TBD )
;
; INPUTS:
;	tsd		TSD object reference.
;	cal_const_file	Calibration constants file;
;			format, usage are TBD (optional.)
; OUTPUTS:
;	map		map
; RETURN
;	!SUCCEED if function ended successfully.
;	!FAIL in failure.
; RESTRICTIONS:
;
; PROCEDURE:
;

;
;
;
; EXAMPLE:
;
; REFERENCE:
;	Baba et al. 2002 Nov. 21, ``ASTRO-F/FIS Survey Data Structure"
;
; MODIFICATION HISTORY:
;	Oct. 2004		Written by Do Kester
;-

FUNCTION im_scan, tsd, mapset, mapinfo, detector=detector, $
	    plot=plot, scannr=scannr,  _extra=_extra

COMPILE_OPT IDL2

;   ON_ERROR 2

;;; Extract values from TSD object reference ;;;
    flux = tsd->get_value_tsd( 'fis_obs', 'flux' )
    if ( mapinfo.errors ) then begin
	wgt  = tsd->get_value_tsd( 'fis_obs', 'ferr' )
	b = where( wgt NE 0, nb )
	if ( nb GT 0 ) then wgt[b] = 1.0 / wgt[b]^2
    endif else wgt = make_array( size=size( flux ), value = 1 )
    lon = tsd->get_value_tsd( 'gads', 'ra' )
    lat = tsd->get_value_tsd( 'gads', 'dec' )
    rol = tsd->get_value_tsd( 'gads', 'rang' )

;    ang = ang / 36000

;    q = indgen(13) * 200
;    print, lon[0], lat[0], ang[0]
;    n = n_elements( lon ) - 1
;    print, lon[n/2], lat[n/2], ang[n/2]
;    print, lon[n], lat[n], ang[n]

    torad = !DPI / 180
    lon *= torad
    lat *= torad
    rol *= torad

;    infl = tsd->get_pix_flag_tsd( 'fis_obs', 'bad' ) OR $
    infl = tsd->get_pix_flag_tsd( 'fis_obs', 'dead' ) OR $
	   tsd->get_pix_flag_tsd( 'fis_obs', 'saturate' ) OR $
	   tsd->get_pix_flag_tsd( 'fis_obs', 'reset' ) OR $
	   tsd->get_pix_flag_tsd( 'fis_obs', 'rstanom' ) OR $
	   tsd->get_pix_flag_tsd( 'fis_obs', 'no_diff' )

    clos = tsd->get_status_tsd( 'fis_obs', 'shtop' ) EQ 0
    lamp = tsd->get_status_tsd( 'fis_obs', 'calalon' ) OR $
	   tsd->get_status_tsd( 'fis_obs', 'calason' ) OR $
	   tsd->get_status_tsd( 'fis_obs', 'calbon' ) OR $
	   tsd->get_status_tsd( 'fis_obs', 'sinalon' ) OR $
	   tsd->get_status_tsd( 'fis_obs', 'sinason' )

    glfl = make_array( size=size( infl ), value=0 )
    if ( mapinfo.gltype EQ 'all' OR mapinfo.gltype EQ 'gp' ) then $
	glfl = glfl OR $
	       tsd->get_pix_flag_tsd( 'fis_obs', 'gpgl_type1' ) OR $
	       tsd->get_pix_flag_tsd( 'fis_obs', 'gpgl_type2' ) OR $
	       tsd->get_pix_flag_tsd( 'fis_obs', 'gpgl_type3' ) OR $
	       tsd->get_pix_flag_tsd( 'fis_obs', 'gpgl_type4' )
    if ( mapinfo.gltype EQ 'all' OR mapinfo.gltype EQ 'mt' ) then $
	glfl = glfl OR im_mtfirstflag( tsd )

    tail = tsd->get_pix_flag_tsd( 'fis_obs', 'gpgl_tail' )

    if ( max( infl ) GT 1 OR max( glfl ) GT 1 ) then stop

    infl += 2 * glfl + 4 * tail

;;; number of channels ;;;
    si = SIZE(flux)
    arr_dim = si[0]

    band = ( si[1] EQ 100 ) ? "SW" : "LW"
    IF ( mapinfo.band EQ "unknown" ) THEN mapinfo.band = band

    if ( n_elements( detector ) EQ 0 ) THEN BEGIN
	CASE mapinfo.band OF
	 "SW" : det = indgen( 60 ) + 1
	 "SN" : det = indgen( 40 ) + 61
	 "LW" : det = indgen( 45 ) + 1
	 "LN" : det = indgen( 30 ) + 46
	 else : print, "Unknown band : " + band
	ENDCASE
    ENDIF ELSE det = detector
    ndet = n_elements( det )
    num_data = si[2]

    if ( mapinfo.flat ) then im_flatfield, flux, infl, clos, lamp, det=det

    stat = clos OR lamp OR shift( lamp, 2 )	; 2 unusable point after lamp

    nq = uniqer( clos, start=qs, length=ql )

    for kq = 0, nq-1 do begin
	if ( clos[qs[kq]] ) then continue	    ; shutter closed

	b = qs[kq] + lindgen( ql[kq] )
	spheric = { $
	    coslon : cos( lon[b] ), $
	    sinlon : sin( lon[b] ), $
	    coslat : cos( lat[b] ), $
	    sinlat : sin( lat[b] ), $
	    cosrol :  cos( rol[b] ), $
	    sinrol : sin( rol[b] ) }
;	    sinrol : -sin( rol[b] ) }	; to make it similar to the IRAS def.
	if ( max( abs( rol[b] ) ) EQ 0 ) then $
		print, "Roll angle probably not set"

	FOR k = 0, ndet - 1 DO BEGIN
	    idet = det[k]
	    kd = idet - 1
	    if ( n_elements( plot ) EQ 2 ) then $
		do_plot = ( plot[0] EQ scannr AND plot[1] EQ det[k] ) ? 2 : 0 $
	    else do_plot = plot

	    res = im_detector( idet, flux[kd,b], wgt[kd,b], $
			       infl[kd,b] OR stat[b], spheric, mapset, $
			       mapinfo, scannr=scannr, $
			       plot=do_plot, _extra=_extra )
	ENDFOR

    ENDFOR

    return, !SUCCEED

END


