;+
; NAME:
;   SS_INIT_PROC
;
; PURPOSE:
;    Initial procedure for linear-fitting to the ramps for a given integration time
;     and making calibration/correction tables (flat, dark, cal and after-effect).
;    Glitch removal is also done in the linear-fitting process.
;    Procedure is basically the same as previous program "ss_mk_cal_table.pro"
;
; INPUT: '*.fits.gz' (TSD)
;
; OUTPUT:
;    Cal tables (Text): '*_(flat|dark|cal|aftcal).txt'
;    linear-fit result (Text): '*_fit_pr.txt'
;    linear-fit result and used parameters (IDL): '*_pr.sav'
;    * "pr" denotes Pre-Reduction
;
; CALLING SEQUENCE:
;
; MODIFICATION HISTORY:
;   2006/12/27 Written by S. Matsuura, based on ss_line_fit.pro
;   2006/12/29 Makiuti, S., Changed the way of parameter passing. 
;                Added N_RAMP_DIV, PLOT_PIXEL, /NO_DISPLAY options.
;                Removed n_int and plot_pix arguments instead.
;   2007/01/05 Makiuti, S., Improved error handling. Added ERROR_STATUS option argument to 
;                return error status.
;                Changed the order of getting packet ID part.
;   2007/01/18 S.Matsuura, included 'P12_FIS02_SAA' to the choice list of AOT
;   2007/01/19 S.Matsuura, modified the AOT recognition part for SAA special operation.
;   2007/01/22 S.Matsuura, included GB-reset anomaly flags.
;   2007/01/23 M.Shirahata, change output file names (.dat --> .txt)
;   2007/01/25 S.Matsuura, changed cal_on/off_idx loop for SAA condition.
;   2007/01/29 Makiuti, S. Clear all pixel flags before GB processes.
;                          Added /USE_BAD option.
;   2007/01/30 S.Matsuura, one can specify time range of data taken for flat fielding.
;   2007/02/03 S.Matsuura, fixed bug in a part of time range setting for local flat data.
;   2007/02/05 S.Matsuura, fixed bug in time setting for local flat data.
;   2007/02/07 M.Shirahata, change option names (t_flt_st -> t_flat_start, t_flt_en -> t_flat_end)
;
;-
pro ss_init_proc, infile, $
        N_RAMP_DIV = n_int, PLOT_PIXEL = plot_pix, $
        RAMP_COR_OFF = ramp_cor_off, $
        GLITCH_RMV_OFF = glitch_rmv_off, $
        AOCU_ADS = aocu_ads, $
        NO_DISPLAY = no_display, $
        ERROR_STATUS = error_status, $
        USE_BAD = use_bad, $ ; Use bad pixel-flags to mask the data
	T_FLAT_START = t_flat_start, T_FLAT_END = t_flat_end

; Display the usage
    IF (n_params() EQ 0) THEN BEGIN
      print, "# USAGE #"
      print, "  ss_init_proc, tsd_filename [, N_RAMP_DIV = value, PLOT_PIXEL = value, ...]"
      print, "  ex.) IDL> ss_init_proc, 'FIS_SW_20061101180345.fits.gz'"
      error_status = 1 ; set error status
      return
    ENDIF

; Check the parameters and initialize
    IF (n_elements(n_int) EQ 0) THEN BEGIN
      n_int = 1
      print, 'N_RAMP_DIV(n_int) was set to 1.'
    ENDIF
    IF (n_elements(plot_pix) EQ 0) THEN BEGIN
      plot_pix = 5
      print, 'PLOT_PIXEL(plot_pix) was set to 5'
    ENDIF

; read input TSD file
    MESSAGE, 'Reading input file ('+infile+').', /CONTINUE
    tsd = obj_new('tsd')
    ret = tsd->Read_TSD(infile)

           if (ret NE !SUCCEED) then begin
               MESSAGE, "TSD File not found."+infile, /CONTINUE
               error_status = 1 ; set error status
               return
           endif

; make filename

    str = strsplit(infile,'.',/extract)
    file_header = str[0]


; get packet ID and remove minor part

    pktid = tsd->get_value_tsd('FIS_OBS','PacketID')
    if(pktid[0] eq '41'X) then print, 'FIS mode (initial): CDS'
    if(pktid[0] ne '41'X) then print, 'FIS mode (initial): Integration'
    
    ; in case of integration mode
    norm_idx = where(pktid ne '41'X, norm_idx_cnt)
    print, 'N elements of integration mode: ', norm_idx_cnt

    ; in case of CDS
    cds_idx = where(pktid eq '41'X, cds_idx_cnt)
    print, 'N elements of CDS mode: ', cds_idx_cnt

    ; If FTS mode data are contained, then exit
    pid_indicator = smpl_mode(pktid)
    dummy = where(pid_indicator EQ 4, cnt_fts)
    IF (cnt_fts GT 1000) THEN BEGIN
      message, 'Data may be FTS data. This tool cannot process FTS data. Process stopped.', /continue
      error_status = 1 ; set error status
      return
    ENDIF

    ; Define sampling mode (Integration or CDS)
    if (norm_idx_cnt gt cds_idx_cnt) then begin
        smpl_mode = 'Integration'
        if (cds_idx_cnt ne 0) then begin
            subset = tsd->Subset_TSD(norm_idx)
            tsd = subset
        endif
    endif else begin
        smpl_mode = 'CDS'
    endelse
    print, 'SAMPLING MODE: ', smpl_mode

; If /USE_BAD option was set, keep pixel bad flag.
    IF keyword_set(use_bad) THEN BEGIN
      pf_bad = tsd->Get_pix_flag_tsd('FIS_OBS','BAD')
    ENDIF

; Clear pixel-flags in FIS_OBS branch first.
    Clear_All_Flag, tsd

; GB-Setting the flags to bad data
    ret = gb_flag_bad_data(tsd)
    if (ret) then begin
       error_status = 1 ; set error status
       message,'Setting the flags to bad data was failed', /continue
       return
    endif

; GB-Reset anomaly flags
    ret = gb_set_flag_rstanom(tsd)
    if (ret) then begin
       error_status = 1 ; set error status
       message,'Setting the flags to reset anomaly was failed', /continue
       return
    endif

; GB-Conversion to volt 
    ret = gb_conv_to_volt(tsd, ferr=0)
    if (ret) then begin
       error_status = 1 ; set error status
       message,'Conversion to volt was failed', /continue
       return
    endif

; GB-Ramp curve correction
    if keyword_set(ramp_cor_off) then goto, ramp_cor_skip
        ret = gb_ramp_curve_corr(tsd, NOPLOT=no_display)
        if (ret) then begin
           error_status = 1 ; set error status
           message,'Conversion to volt was failed', /continue
           return
        endif
    ramp_cor_skip:


; data type
    data_type = tsd->Get_Datatype()
    print, 'Data type: ', data_type

    if(data_type eq 'FIS_SW') then begin
        n_pix = 100 ; total pix number
        w_pix = 60  ; wide band
        smpl_freq = 25.28D; [Hz]
        pix_omega = 2.1154e-8 ; str
        capacitance = 7.0D ; pF
        spk_thrld = 4.0D
    endif else begin
        n_pix = 75  ;total pix number
        w_pix = 45  ; wide band
        smpl_freq = 16.86D; [Hz]
        pix_omega = 5.8761e-8 ; str
        capacitance = 3.0D ; pF
        spk_thrld = 5.0D
    endelse


; get detector data

    det = tsd->get_value_tsd('FIS_OBS','flux')
    n_sample = n_elements(det)/n_pix
    print, 'N elements in total: ', n_sample

    if (smpl_mode eq 'CDS') then cds = det


; Get bad data pixel flags
    pf_undef_anom = tsd->Get_pix_flag_tsd('FIS_OBS','Undef_Anom')
    pf_satur = tsd->Get_pix_flag_tsd('FIS_OBS','Saturate')
    pf_reset = tsd->Get_pix_flag_tsd('FIS_OBS','Reset')
    pf_rstanom = tsd->Get_pix_flag_tsd('FIS_OBS','RstAnom')
    pf_no_rp_corr = tsd->Get_pix_flag_tsd('FIS_OBS','No_RP_Corr')

    if keyword_set(ramp_cor_off) then begin
        pf_ng_total = pf_reset + pf_rstanom + pf_satur + pf_undef_anom
    endif else begin
        pf_ng_total = pf_reset + pf_rstanom + pf_satur + pf_undef_anom $
            + pf_no_rp_corr ; total bad frame flag for each pixel
    endelse

; If /USE_BAD was set, pixel bad flag are used as pf_ng_total.
    IF keyword_set(use_bad) THEN BEGIN
      pf_ng_total = pf_ng_total + pf_bad
    ENDIF


; get reset, shutter and cal_lamp status
    if(data_type eq 'FIS_SW') then begin
        rstcnt = tsd->get_value_tsd('FIS_OBS','rstcntwides')
        rstcnt = fix(rstcnt)
        shtop = tsd->Get_status_tsd('FIS_OBS','shtop')
        shtop = fix(shtop)
        calon = tsd->Get_status_tsd('FIS_OBS','calason')
        calon = fix(calon)
        calon = shift(calon, 2)
        cal_dac = tsd->Get_value_tsd('FIS_OBS','daccalas')
    endif else begin
        rstcnt = tsd->get_value_tsd('FIS_OBS','rstcntwidel')
        rstcnt = fix(rstcnt)
        shtop = tsd->Get_status_tsd('FIS_OBS','shtop')
        shtop = fix(shtop)
        calon = tsd->Get_status_tsd('FIS_OBS','calalon')
        calon = fix(calon)
        cal_dac = tsd->Get_value_tsd('FIS_OBS','daccalal')
    endelse


; get AOCS data - Q parameter
    if keyword_set(AOCU_ADS) then begin
        q = tsd->get_value_tsd('AOCU','AOCU_ADS_Q')
        print, 'Attitude data: AOCU_ADS_Q'
    endif else begin
        q = tsd->get_value_tsd('GADS','ATTITUDE')
        print, 'Attitude data: GADS'
        ; Check data availability
        dummy = where(q NE 0, cnt_gads)
        if (cnt_gads LE 1000) then begin
          message, 'No attitude information available in GADS. Try /AOCU_ADS option.', /continue
          error_status = 1 ; set error status
          return
        endif
    endelse

; release tsd obj
      Obj_Destroy, tsd

; Reset interval
    if (smpl_mode eq 'Integration') then begin
        rstcnt_d = rstcnt - shift(rstcnt, 1)
        rst_idx = where(rstcnt_d ne 0, rcount)
    rst_discr_range = where(rst_idx gt 630*smpl_freq and rst_idx lt 1350*smpl_freq)
        rst_discr = rst_idx[rst_discr_range]
        rst_int = median(rst_discr-shift(rst_discr,1))/smpl_freq
        print, 'Reset interval [s]:', rst_int
    endif else begin
        rst_int = 0.0
    endelse


; Shutter index
    sht_idx = where(shtop eq 0, cnt)
    sht_d = shtop - shift(shtop, 1)
    sht_op_idx = where(sht_d eq 1, sht_op_cnt)
    sht_cls_idx = where(sht_d eq -1, sht_cls_cnt)

    if (sht_op_idx[0] lt 180.0D*smpl_freq) then begin
        sht_op_idx = sht_op_idx[1:sht_op_cnt-1]
        sht_op_cnt -= 1
    endif
    if (sht_cls_idx[0] lt 180.0D*smpl_freq) then begin
        sht_cls_idx = sht_cls_idx[1:sht_cls_cnt-1]
        sht_cls_cnt -= 1
    endif
    while (sht_op_idx[sht_op_cnt-1] gt 1580.0D*smpl_freq) do begin
        sht_op_idx = sht_op_idx[0:sht_op_cnt-2]
        sht_op_cnt -= 1
    end
    while (sht_cls_idx[sht_cls_cnt-1] gt 1580.0D*smpl_freq) do begin
        sht_cls_idx = sht_cls_idx[0:sht_cls_cnt-2]
        sht_cls_cnt -= 1
    end

    print, 'shutter open count: ', sht_op_cnt
    print, '    sht_op_idx: ', sht_op_idx
    print, 'shutter close count: ', sht_cls_cnt
    print, '    sht_cls_idx: ', sht_cls_idx


; CAL lamp index
    cal_idx = where((calon ne 0) and (cal_dac ne 0), cnt)
    cal_d = calon - shift(calon, 1)
    cal_on_idx = where((cal_d eq 1) and (cal_dac ne 0), cal_on_cnt)
    cal_off_idx = where((cal_d eq -1) and (cal_dac ne 0), cal_off_cnt)

    while (cal_on_idx[0] lt (sht_op_idx[0]-410.0D*smpl_freq)) do begin
        cal_on_idx = cal_on_idx[1:cal_on_cnt-1]
        cal_on_cnt -= 1
    end
    while (cal_off_idx[0] lt (sht_op_idx[0]-410.0D*smpl_freq)) do begin
        cal_off_idx = cal_off_idx[1:cal_off_cnt-1]
        cal_off_cnt -= 1
    end
    while (cal_on_idx[cal_on_cnt-1] gt (sht_cls_idx[sht_cls_cnt-1]+410.0D*smpl_freq)) do begin
        cal_on_idx = cal_on_idx[0:cal_on_cnt-2]
        cal_on_cnt -= 1
    end
    while (cal_off_idx[cal_off_cnt-1] gt (sht_cls_idx[sht_cls_cnt-1]+410.0D*smpl_freq)) do begin
        cal_off_idx = cal_off_idx[0:cal_off_cnt-2]
        cal_off_cnt -= 1
    end

    print, 'A-cal on count: ', cal_on_cnt
    print, 'cal_on_idx[0],[cnt-1]: ', cal_on_idx[0], cal_on_idx[cal_on_cnt-1]
    print, 'A-cal off count: ', cal_off_cnt
    print, 'cal_off_idx[0],[cnt-1]: ', cal_off_idx[0], cal_off_idx[cal_off_cnt-1]

    cal_off_flg = intarr(n_sample)
    if(cal_off_cnt gt 0) then begin
        cal_off_flg[cal_off_idx] = 1
    endif


; mask for shutter and cal-lamp

    drv_mask = intarr(n_sample)
    drv_mask_idx = where(shtop eq 0 or (calon eq 1 and cal_dac ne 0), drv_mask_cnt)
    if(drv_mask_cnt ne 0) then begin
        drv_mask[drv_mask_idx] = 1
    endif

    drv_mask[0:sht_op_idx[0]-1] = 1
    drv_mask[sht_cls_idx[sht_cls_cnt-1]:n_sample-1] = 1


; Discrimination of AOT by # of CAL events

    AOT = 'AOT unknown'
    if ( cal_on_cnt eq 30 and sht_op_cnt eq 4 ) then AOT = 'PV_FIS01'
    if ( cal_on_cnt eq 9 and sht_op_cnt eq 4 ) then AOT = 'P12_FIS01'
    if ( sht_op_cnt eq 4 and cal_off_idx[0]-cal_on_idx[0] lt 5D*smpl_freq ) then begin
    AOT = 'P12_FIS01_SAA'
    while (cal_on_idx[2] lt (sht_op_idx[0]-270.0D*smpl_freq)) do begin
            cal_on_idx = cal_on_idx[1:cal_on_cnt-1]
            cal_on_cnt -= 1
    end
    while (cal_on_idx[2] gt (sht_op_idx[0]-270.0D*smpl_freq)) do begin
        cal_on_idxs = lonarr(cal_on_cnt+1)
            cal_on_idxs[1:cal_on_cnt] = cal_on_idx[0:cal_on_cnt-1]
        cal_on_idx = cal_on_idxs
            cal_on_cnt += 1
    end
    while (cal_off_idx[2] lt (sht_op_idx[0]-270.0D*smpl_freq)) do begin
            cal_off_idx = cal_off_idx[1:cal_off_cnt-1]
            cal_off_cnt -= 1
    end
    while (cal_off_idx[2] gt (sht_op_idx[0]-270.0D*smpl_freq)) do begin
        cal_off_idxs = lonarr(cal_off_cnt+1)
            cal_off_idxs[1:cal_off_cnt] = cal_off_idx[0:cal_off_cnt-1]
        cal_off_idx = cal_off_idxs
            cal_off_cnt += 1
    end
    endif
    if ( cal_on_cnt eq 27 and sht_op_cnt eq 2 ) then AOT = 'PV_FIS02'
    if ( cal_on_cnt eq 7 and sht_op_cnt eq 2 ) then AOT = 'P12_FIS02'
    if ( sht_op_cnt eq 2 and cal_off_idx[0]-cal_on_idx[0] lt 5D*smpl_freq ) then begin
    AOT = 'P12_FIS02_SAA'
    while (cal_on_idx[2] lt (sht_op_idx[0]-270.0D*smpl_freq)) do begin
            cal_on_idx = cal_on_idx[1:cal_on_cnt-1]
            cal_on_cnt -= 1
    end
    while (cal_on_idx[2] gt (sht_op_idx[0]-270.0D*smpl_freq)) do begin
        cal_on_idxs = lonarr(cal_on_cnt+1)
            cal_on_idxs[1:cal_on_cnt] = cal_on_idx[0:cal_on_cnt-1]
        cal_on_idx = cal_on_idxs
            cal_on_cnt += 1
    end
    while (cal_off_idx[2] lt (sht_op_idx[0]-270.0D*smpl_freq)) do begin
            cal_off_idx = cal_off_idx[1:cal_off_cnt-1]
            cal_off_cnt -= 1
    end
    while (cal_off_idx[2] gt (sht_op_idx[0]-270.0D*smpl_freq)) do begin
        cal_off_idxs = lonarr(cal_off_cnt+1)
            cal_off_idxs[1:cal_off_cnt] = cal_off_idx[0:cal_off_cnt-1]
        cal_off_idx = cal_off_idxs
            cal_off_cnt += 1
    end
    endif
    print, 'AOT:    ', AOT
    if (AOT eq 'AOT unknown') then begin
        print, '%ERROR: AOT is not discriminated.'
        error_status = 1 ; set error status
        return
    endif

; Discrimination of RST mode by reset interval and DAC value of CAL

    cal_dac_dscr = cal_dac[cal_on_idx[3]]
    print, 'A-CAL DAC value:    ', cal_dac_dscr

    RST = 'RST unknown'
    ; '80'X -> 128
    ; '60'X -> 96
    ; '30'X -> 48
    ; '10'X -> 16
    if (data_type eq 'FIS_SW') then begin
        if (rst_int eq 0 and cal_dac_dscr eq 128) then RST = 'CDS'
        if (rst_int gt 0.3 and rst_int lt 0.7 and cal_dac_dscr eq 128) then RST = '05N'
        if (rst_int gt 0.7 and rst_int lt 1.5 and cal_dac_dscr eq 96) then RST = '10N'
        if (rst_int gt 1.5 and rst_int lt 2.5 and cal_dac_dscr eq 48) then RST = '20N'
        if (rst_int gt 1.5 and rst_int lt 2.5 and cal_dac_dscr eq 16) then RST = '20D'
    endif else begin
        if (rst_int eq 0 and cal_dac_dscr eq 96) then RST = 'CDS'
        if (rst_int gt 0.3 and rst_int lt 0.7 and cal_dac_dscr eq 96) then RST = '05N'
        if (rst_int gt 0.7 and rst_int lt 1.5 and cal_dac_dscr eq 96) then RST = '10N'
        if (rst_int gt 1.5 and rst_int lt 2.5 and cal_dac_dscr eq 96) then RST = '20N'
    endelse
    print, 'RST mode:   ', RST
    print, 'Calculated reset interval [s]:', rst_int
    if (RST eq 'RST unknown') then begin
        print, '%ERROR: RST mode is not discriminated.'
        error_status = 1 ; set error status
        return
    endif


;;;;;;; glitch detection and linear fitting for normal integration mode ;;;;;;;
if(smpl_mode eq 'Integration' and norm_idx_cnt gt 0) then begin

; reset index

    rstcnt_d = rstcnt - shift(rstcnt, 1)
    rst_idx = where(rstcnt_d ne 0, rcount)
    print, 'Number of reset: ', rcount


; diff rms noise calculation

    print, 'Glitch detection process is going on ...'

    diff1 = det - shift(det,0,1)
    diff2 = det - shift(det,0,2)
    scnd_diff = diff1 - shift(diff1,0,1)
    scnd_diff2 = diff2 - shift(diff2,0,2)

    diff_mean = dblarr(n_pix,rcount)
    diff_stdev = dblarr(n_pix,rcount)
    diff_mean2 = dblarr(n_pix,rcount)
    diff_stdev2 = dblarr(n_pix,rcount)
    base_level = diff1
    rms_noise = diff1
    base_level2 = diff2
    rms_noise2 = diff2

    i_mg_st = 4 ; head mergin to calculate rms noise after reset
    i_mg_en = 2 ; tail mergin to calculate rms noise before reset

    for pix = 0, n_pix-1 do begin

    for j = 0L, rcount-1L do begin

        if(j LT rcount-1L) then begin
            i_noise_st = rst_idx[j] + i_mg_st   ; start sample for noise calculation
            i_noise_en = rst_idx[j+1] - i_mg_en ; end sample for noise calculation
        endif else begin
            i_noise_st = rst_idx[rcount-1] + i_mg_st; start sample for noise calculation
            i_noise_en = n_sample - i_mg_en     ; end sample for noise calculation
        endelse
        n_noise = i_noise_en - i_noise_st + 1

        if(n_noise gt 0) then begin

            ramp_idx = lonarr(n_noise)
            for k = 0L, n_noise-1L do begin
                ramp_idx[k] = i_noise_st + k
            endfor

            ; mask data with pixel flag
            pf_gd_idx = where( pf_ng_total[pix, ramp_idx] eq 0, pf_gd_cnt)
            if pf_gd_cnt gt 0 then begin
		ramp_idx = ramp_idx[pf_gd_idx]
		n_noise = n_elements(ramp_idx)
	    endif else begin
		n_noise = 1
	    endelse

        endif

        if(n_noise GT 2) then begin ; in case of sufficient # of samples for fitting

            diff_mean[pix,j] = mean(diff1[pix, ramp_idx], /double)
            diff_stdev[pix,j] = stddev(diff1[pix, ramp_idx], /double)
            diff_mean2[pix,j] = mean(diff2[pix, ramp_idx], /double)
            diff_stdev2[pix,j] = stddev(diff2[pix, ramp_idx], /double)

            base_level[pix,ramp_idx] = diff_mean[pix,j]
            rms_noise[pix,ramp_idx] = diff_stdev[pix,j]
            base_level2[pix,ramp_idx] = diff_mean2[pix,j]
            rms_noise2[pix,ramp_idx] = diff_stdev2[pix,j]
    
        endif else begin ; in case of few # of samples for fitting

            base_level[pix,j] = 0.0D
            rms_noise[pix,j] =  0.0D
            base_level2[pix,j] =  0.0D
            rms_noise2[pix,j] =  0.0D

        endelse

    endfor ; reset count

    endfor ; pix number


; generate easy spike flg

    easy_gl = intarr(n_pix, n_sample) ; easy glitch status

    ; data affected by glitch - trailing: 4 samples

    spk_idx = where( $
        ( abs(diff1-base_level) $
            GT spk_thrld*rms_noise $
        and abs(scnd_diff $
            +shift(scnd_diff,0,-1) $
            +shift(scnd_diff,0,-2)) $
            LT spk_thrld*rms_noise ) $
        or $
        ( abs(diff1-base_level $
            +shift(diff1-base_level,0,-1)) $
            GT spk_thrld*rms_noise $
        and abs(scnd_diff $
            +shift(scnd_diff,0,-1) $
            +shift(scnd_diff,0,-2) $
            +shift(scnd_diff,0,-3)) $
            LT spk_thrld*rms_noise ) $
        or $
        ( abs(diff1-base_level $
            +shift(diff1-base_level,0,-1) $
            +shift(diff1-base_level,0,-2)) $
            GT spk_thrld*rms_noise $
        and abs(scnd_diff $
            +shift(scnd_diff,0,-1) $
            +shift(scnd_diff,0,-2) $
            +shift(scnd_diff,0,-3) $
            +shift(scnd_diff,0,-4)) $
            LT spk_thrld*rms_noise ) $
        or $
        ( abs(diff2-base_level2) $
            GT spk_thrld*rms_noise2 $
        and abs(scnd_diff2 $
            +shift(scnd_diff2,0,-2) $
            +shift(scnd_diff2,0,-4)) $
            LT spk_thrld*rms_noise2 ) $
        or $
        ( abs(diff2-base_level2 $
            +shift(diff2-base_level2,0,-2)) $
            GT spk_thrld*rms_noise2 $
        and abs(scnd_diff2 $
            +shift(scnd_diff2,0,-1) $
            +shift(scnd_diff2,0,-2) $
            +shift(scnd_diff2,0,-3) $
            +shift(scnd_diff2,0,-4)) $
            LT spk_thrld*rms_noise2 ) $
        or $
        ( abs(diff2-base_level2 $
            +shift(diff2-base_level2,0,-2) $
            +shift(diff2-base_level2,0,-4)) $
            GT spk_thrld*rms_noise2 $
        and abs(scnd_diff2 $
            +shift(scnd_diff2,0,-1) $
            +shift(scnd_diff2,0,-2) $
            +shift(scnd_diff2,0,-3) $
            +shift(scnd_diff2,0,-4) $
            +shift(scnd_diff2,0,-5) $
            +shift(scnd_diff2,0,-6) $
            +shift(scnd_diff2,0,-7) $
            +shift(scnd_diff2,0,-8)) $
            LT spk_thrld*rms_noise2 ) $
        , spk_cnt1)

    if spk_cnt1 gt 0 then easy_gl[spk_idx] = 1 ; set glitch status
    easy_gl[pf_ng_total] = 0 ; mask bad data
    eg_gl_idx = where(easy_gl ne 0, spk_cnt)
    print, '# of glitches: ', spk_cnt


; memory release
    diff1 = 0
    diff2 = 0
    scnd_diff = 0
    scnd_diff2 = 0
    diff_mean = 0
    diff_stdev = 0
    diff_mean2 = 0
    diff_stdev2 = 0
    base_level = 0
    rms_noise = 0
    base_level2 = 0
    rms_noise2 = 0


; linear fit

    q_mean = dblarr(4,rcount*n_int) ; mean values of Q-para in each ramp segment
    coeff = dblarr(n_pix,rcount*n_int,2) ; fitting result
    error = dblarr(n_pix,rcount*n_int,2) ; fitting error
    eg_gl_cnt = intarr(n_pix,rcount*n_int) ; total easy glitch count in fitting range
    x_ave = dblarr(rcount*n_int) ; average time index in fitting range
    drv_mask_tot = intarr(rcount*n_int) ; flg for mask (shutter, cal)
    cal_flg_tot = intarr(rcount*n_int)
    
    i_gmg_tr = 4 ; trailing mergin for fitting after glitch
    i_gmg_ld = 2 ; leading mergin for fitting before glitch

    for pix = 0, n_pix-1 do begin
    
    for j = 0L, rcount-1L do begin ; loop for single ramp descriminated by rst status

        if(j LT rcount-1L) then begin
            i_fit_st = rst_idx[j]   ; start sample for fitting
            i_fit_en = rst_idx[j+1] ; end sample for fitting
        endif else begin
            i_fit_st = rst_idx[rcount-1]    ; start sample for fitting
            i_fit_en = n_sample     ; end sample for fitting
        endelse
        n_fit = i_fit_en - i_fit_st + 1

    nn_fit_sum = 0L

    for fit_cnt = 0, n_int-1 do begin ; ramp split into "n_int" sub-ramps

        if(fit_cnt lt (n_fit mod n_int)) then begin
            nn_fit = n_fit/n_int + 1
        endif else begin
            nn_fit = n_fit/n_int
        endelse

        if(nn_fit gt 0) then begin

            ramp_idx = lonarr(nn_fit)
            for k = 0L, nn_fit-1L do begin
                ramp_idx[k] = i_fit_st + nn_fit_sum + k
            endfor

            ; mask data with pixel flag
            pf_gd_idx = where( pf_ng_total[pix, ramp_idx] eq 0, pf_gd_cnt)
            if pf_gd_cnt gt 0 then begin
		ramp_idx = ramp_idx[pf_gd_idx]
		nn_fit = n_elements(ramp_idx)
	    endif else begin
		nn_fit = 1
	    endelse

        endif

        if(nn_fit gt 2) then begin ; in case of sufficient # of samples for fitting

            p_spk_idx = where( easy_gl[pix,ramp_idx] gt 0 $
                    and rstcnt_d[ramp_idx] eq 0 $
                    , n_spk)

            if(keyword_set(glitch_rmv_off) eq 1) then n_spk = 0 ; to skip glitch remove process

            if(n_spk eq 0) then begin ; in case of no glitch in a ramp

                ret = linfit(ramp_idx, det[pix, ramp_idx], /double, SIGMA=sigma)
                coeff[pix,n_int*j+fit_cnt,1] = ret[1]   ; slope
                error[pix,n_int*j+fit_cnt,1] = sigma[1] ; slope error

            endif else begin ; in case of some glitches in a ramp

            ;;; this loop should be improved by taking the effect after glitch into consideration ;;;

                result = dblarr(n_spk+1,2)

                p_ramp_idx = ramp_idx[p_spk_idx]
                
                for i_spk = 0, n_spk do begin ; loop for ramp segments split by glitches 

                    if(i_spk eq 0) then begin ; first part of ramp
                        idx_0 = 0
                        idx_1 = p_spk_idx[0] - i_gmg_ld
                    endif
                    if(i_spk gt 0 and i_spk lt n_spk) then begin
                        idx_0 = p_spk_idx[i_spk-1] + i_gmg_tr
                        idx_1 = p_spk_idx[i_spk] - i_gmg_ld
                    endif
                    if(i_spk eq n_spk) then begin ; final part of ramp
                        idx_0 = p_spk_idx[n_spk-1] + i_gmg_tr
                        idx_1 = n_elements(ramp_idx) - 1
                    endif

                    np_fit = idx_1 - idx_0 + 1

                    if(np_fit gt 2) then begin

                        p_fit_idx = ramp_idx[idx_0:idx_1]

                        ret = linfit(p_fit_idx, det[pix, p_fit_idx], /double, SIGMA=sigma)
                        result[i_spk,0] = ret[1]
                        result[i_spk,1] = sigma[1]

                    endif else begin

                        result[i_spk,0] = 0.0D
                        result[i_spk,1] = 0.0D
                
                    endelse

                endfor ; # of glitches

                w_coeff_sum = 0.0D
                w_sum = 0.0D
                n_lim = n_spk
                for i = 0, n_lim do begin
                    if(result[i,0] ne 0 and result[i,1] gt 0) then begin
                    w_coeff_sum = w_coeff_sum + result[i,0]/result[i,1]^2
                    w_sum = w_sum + 1.0D/result[i,1]^2
                    endif
                endfor

		if (w_sum gt 0) then begin
                	coeff[pix,n_int*j+fit_cnt,1] = w_coeff_sum/w_sum
                	error[pix,n_int*j+fit_cnt,1] = sqrt(1.0D/w_sum)
		endif

            endelse

            eg_gl_cnt[pix,n_int*j+fit_cnt] = total(easy_gl[pix,ramp_idx])

            if(pix eq 9) then begin
                x_ave[n_int*j+fit_cnt] = total(ramp_idx)/double(nn_fit)
                drv_mask_tot[n_int*j+fit_cnt] = total(drv_mask[ramp_idx])
                cal_flg_tot[n_int*j+fit_cnt] = total(cal_off_flg[ramp_idx])
                for k = 0, 3 do begin
                    q_mean[k,n_int*j+fit_cnt] = mean(q[k,ramp_idx],/double)
                endfor
            endif

        endif else begin ; in case # of samples for fitting is not enough

            coeff[pix,n_int*j+fit_cnt,*] = 0.0D
            error[pix,n_int*j+fit_cnt,*] = 0.0D

            eg_gl_cnt[pix,n_int*j+fit_cnt] = easy_gl[pix, rst_idx[j]]

            if(pix eq 9) then begin
                x_ave[n_int*j+fit_cnt] =  rst_idx[j]
                drv_mask_tot[n_int*j+fit_cnt] = drv_mask[rst_idx[j]]
                cal_flg_tot[n_int*j+fit_cnt] = cal_off_flg[rst_idx[j]]
                for k = 0, 3 do begin
                    q_mean[k,n_int*j+fit_cnt] = q[k, rst_idx[j]]
                endfor
            endif

        endelse

        nn_fit_sum = nn_fit_sum + nn_fit

    endfor ; fit_cnt

    endfor ; j: reset count

    endfor ; pixel number


; unit conversion for slope calculation

    coeff = coeff*smpl_freq     ; [V/sample] -> [V/s]
    error = error*smpl_freq     ; [V/sample] -> [V/s]


endif


;;;;;;; glitch detection and averaging for CDS mode ;;;;;;;
if(smpl_mode eq 'CDS' and cds_idx_cnt gt 0) then begin

; number of samples for averaging

    n_int_dbl = double(n_int)
    rst_int = 1.0D ; nominal averaging time [s]
    n_ave_cds = fix(smpl_freq*rst_int/n_int_dbl)
;   n_data_cds = fix(cds_idx_cnt/n_ave_cds)
    n_data_cds = fix(n_sample/n_ave_cds)
    if (n_data_cds*n_ave_cds gt n_sample) then n_data_cds = n_data_cds - 1
    print, 'Number of average data block: ', n_data_cds
    rcount = n_data_cds/n_int_dbl


; rms noise calculation for CDS data

    print, 'Glitch detection process for CDS is now going ...'

    cds_diff = cds - shift(cds,0,1)

    cds_mean = dblarr(n_pix,n_data_cds)
    cds_stdev = dblarr(n_pix,n_data_cds)
    cds_base_level = cds
    cds_rms_noise = cds

    for pix = 0, n_pix-1 do begin

    for j = 0L, n_data_cds-1L do begin

        if(n_ave_cds GE 5) then begin
            n_noise = n_ave_cds
        endif else begin ; too few samples to calculate rms noise
            n_noise = 5         
        endelse

        ave_idx = lonarr(n_noise)
        for k = 0L, n_noise-1L do begin
            ave_idx[k] = j*n_noise + k
        endfor

        ; mask data with pixel flag
        pf_gd_idx = where( pf_undef_anom[pix, ave_idx] eq 0, pf_gd_cnt)
        if pf_gd_cnt gt 0 then begin
	    ave_idx = ave_idx[pf_gd_idx]
	    n_noise = n_elements(ave_idx)
	endif else begin
	    n_noise = 1
	endelse

        cds_mean[pix,j] = mean(cds[pix, ave_idx], /double)
        cds_stdev[pix,j] = stddev(cds[pix, ave_idx], /double)

        cds_base_level[pix,ave_idx] = cds_mean[pix,j]
        cds_rms_noise[pix,ave_idx] = cds_stdev[pix,j]
    
    endfor ; for j

    endfor ; pix number


; generate easy spike flg

    easy_gl = intarr(n_pix, n_data_cds) ; easy glitch status

    ; data affected by glitch - trailing: 4 samples

    cds_spk_idx = where( $
        ( abs(cds-cds_base_level) $
            GT spk_thrld*cds_rms_noise $
        and abs(cds_diff $
            +shift(cds_diff,0,-1) $
            +shift(cds_diff,0,-2)) $
            LT spk_thrld*cds_rms_noise ) $
        or $
        ( abs(cds-cds_base_level $
            +shift(cds-cds_base_level,0,-1)) $
            GT spk_thrld*cds_rms_noise $
        and abs(cds_diff $
            +shift(cds_diff,0,-1) $
            +shift(cds_diff,0,-2) $
            +shift(cds_diff,0,-3)) $
            LT spk_thrld*cds_rms_noise ) $
        or $
        ( abs(cds-cds_base_level $
            +shift(cds-cds_base_level,0,-1) $
            +shift(cds-cds_base_level,0,-2)) $
            GT spk_thrld*cds_rms_noise $
        and abs(cds_diff $
            +shift(cds_diff,0,-1) $
            +shift(cds_diff,0,-2) $
            +shift(cds_diff,0,-3) $
            +shift(cds_diff,0,-4)) $
            LT spk_thrld*cds_rms_noise ) $
        , spk_cnt1)

    if spk_cnt1 gt 0 then easy_gl[cds_spk_idx] = 1 ; set glitch status
    easy_gl[pf_ng_total] = 0 ; mask bad data
    eg_gl_idx = where(easy_gl ne 0, spk_cnt)
    print, '# of glitches: ', spk_cnt


; memory release
    cds_diff = 0
    cds_mean = 0
    cds_stdev = 0
    cds_base_level = 0
    cds_rms_noise = 0


; averaging CDS data

    q_mean = dblarr(4,n_data_cds) ; mean values of Q-para
    cds_mean = dblarr(n_pix,n_data_cds) ; mean values of CDS data
    cds_error = dblarr(n_pix,n_data_cds) ; rms error
    eg_gl_cnt = intarr(n_pix,n_data_cds) ; total easy glitch count
    x_ave = dblarr(n_data_cds) ; averaged time index
    drv_mask_tot = intarr(n_data_cds) ; flg for mask (shutter, cal)
    cal_flg_tot = intarr(n_data_cds)
    coeff = dblarr(n_pix,n_data_cds,2)
    error = dblarr(n_pix,n_data_cds,2)
    
    i_gmg_tr = 4 ; trailing mergin for fitting after glitch
    i_gmg_ld = 2 ; leading mergin for fitting before glitch

    for pix = 0, n_pix-1 do begin
    
    for j = 0L, n_data_cds-1L do begin

        if(n_ave_cds gt 1) then begin ; in case of sufficient # of samples for rms calculation

            ave_idx = lonarr(n_ave_cds)
            for k = 0L, n_ave_cds-1L do begin
                ave_idx[k] = j*n_ave_cds + k
            endfor

            ; mask data with pixel flag
            pf_gd_idx = where( pf_undef_anom[pix, ave_idx] eq 0, pf_gd_cnt)
	    if pf_gd_cnt gt 0 then begin
		ave_idx = ave_idx[pf_gd_idx]
                n_ave_cds = n_elements(ave_idx)
	    endif else begin
                n_ave_cds = 1
	    endelse

            p_spk_idx = where(easy_gl[pix,ave_idx] gt 0, n_spk)

            if(keyword_set(glitch_rmv_off) eq 1) then n_spk = 0 ; to skip glitch remove process

            if(n_spk eq 0) then begin ; in case of no glitch in a range for averaging

                cds_mean[pix,j] = mean(cds[pix,ave_idx], /double)
                cds_error[pix,j] = stddev(cds[pix,ave_idx], /double)

            endif else begin ; in case of some glitches in a range for averaging

            ;;; this loop should be improved by taking the effect after glitch into consideration ;;;

                result = dblarr(n_spk+1,2)

                p_ave_idx = ave_idx[p_spk_idx]
                
                for i_spk = 0, n_spk do begin ; loop for ramp segments split by glitches 

                    if(i_spk eq 0) then begin ; first part
                        idx_0 = ave_idx[0]
                        idx_1 = p_ave_idx[0] - i_gmg_ld
                    endif
                    if(i_spk gt 0 and i_spk lt n_spk) then begin
                        idx_0 = p_ave_idx[i_spk-1] + i_gmg_tr
                        idx_1 = p_ave_idx[i_spk] - i_gmg_ld
                    endif
                    if(i_spk eq n_spk) then begin ; final part of ramp
                        idx_0 = p_ave_idx[n_spk-1] + i_gmg_tr
                        idx_1 = ave_idx[n_ave_cds-1]
                    endif

                    np_ave = idx_1 - idx_0 + 1

                            if(np_ave gt 2) then begin

                        p_ave_fit_idx = lonarr(np_ave)
                        for k = 0, np_ave-1 do begin
                            p_ave_fit_idx[k] = idx_0 + k
                        endfor

                        result[i_spk,0] = mean(cds[pix,p_ave_fit_idx], /double)
                        result[i_spk,1] = stddev(cds[pix,p_ave_fit_idx], /double)

                    endif else begin

                        result[i_spk,0] = 0.0D
                        result[i_spk,1] = 0.0D
                
                    endelse

                endfor ; # of glitches

                w_ave_sum = 0.0D
                w_w_sum = 0.0D
                n_lim = n_spk
                for i = 0, n_lim do begin
                    if(result[i,0] ne 0 and result[i,1] gt 0) then begin
                    w_ave_sum = w_ave_sum + result[i,0]/result[i,1]^2
                    w_w_sum = w_w_sum + 1.0D/result[i,1]^2
                    endif
                endfor

		if (w_w_sum gt 0) then begin
			cds_mean[pix,j] = w_ave_sum/w_w_sum
			cds_error[pix,j] = sqrt(1.0D/w_w_sum)
		endif

            endelse

            eg_gl_cnt[pix,j] = total(easy_gl[pix,ave_idx])

            if(pix eq 9) then begin
                x_ave[j] = total(ave_idx)/double(n_ave_cds)
                drv_mask_tot[j] = total(drv_mask[ave_idx])
                cal_flg_tot[j] = total(cal_off_flg[ave_idx])
                for k = 0, 3 do begin
                    q_mean[k,j] = mean(q[k,ave_idx],/double)
                endfor
            endif

        endif else begin ; in case # of samples for fitting is not enough

            cds_mean[pix,j] = cds[pix,j*n_ave_cds]
            cds_error[pix,j] = 0.0D

            eg_gl_cnt[pix,j] = easy_gl[pix,j*n_ave_cds]

            if(pix eq 9) then begin
                x_ave[j] =  j*n_ave_cds
                drv_mask_tot[j] = drv_mask[j*n_ave_cds]
                cal_flg_tot[j] = cal_off_flg[j*n_ave_cds]
                for k = 0, 3 do begin
                    q_mean[k,j] = q[k,j*n_ave_cds]
                endfor
            endif

        endelse

    endfor ; for j

    endfor ; pixel number


; unit conversion for slope calculation

    coeff[*,*,1] = cds_mean*smpl_freq   ;[V/sample] -> [V/s]
    error[*,*,1] = cds_error*smpl_freq  ;[V/sample] -> [V/s]


endif



;;;;;;; Common part for normal integration and CDS modes ;;;;;;;

; make filename
    str = strsplit(infile,'.',/extract)
    file_header = str[0]
    file_footer = '.txt'


; Making CAL table

    print, '*** Making CAL table ***'
    if (AOT eq 'PV_FIS01') then begin
        filename = file_header + '_cal_0' + file_footer
        ss_flux_ave, filename, x_ave, coeff, error, n_pix, plot_pix, NO_DISP=no_display, $
            cal_on_idx[0], cal_off_idx[0]-1
        filename = file_header + '_cal_1' + file_footer
        ss_flux_ave, filename, x_ave, coeff, error, n_pix, plot_pix, NO_DISP=no_display, $
            cal_on_idx[3], cal_off_idx[3]-1
        filename = file_header + '_cal_2' + file_footer
        ss_flux_ave, filename, x_ave, coeff, error, n_pix, plot_pix, NO_DISP=no_display, $
            cal_on_idx[14], cal_off_idx[14]-1
        filename = file_header + '_cal_3' + file_footer
        ss_flux_ave, filename, x_ave, coeff, error, n_pix, plot_pix, NO_DISP=no_display, $
            cal_on_idx[18], cal_off_idx[18]-1
        filename = file_header + '_cal_4' + file_footer
        ss_flux_ave, filename, x_ave, coeff, error, n_pix, plot_pix, NO_DISP=no_display, $
            cal_on_idx[22], cal_off_idx[22]-1
        filename = file_header + '_cal_5' + file_footer
        ss_flux_ave, filename, x_ave, coeff, error, n_pix, plot_pix, NO_DISP=no_display, $
            cal_on_idx[27], cal_off_idx[27]-1
        filename = file_header + '_cal_6' + file_footer
        ss_flux_ave, filename, x_ave, coeff, error, n_pix, plot_pix, NO_DISP=no_display, $
            cal_on_idx[29], cal_off_idx[29]-1
    endif
    if (AOT eq 'P12_FIS01') then begin
        filename = file_header + '_cal_0' + file_footer
        ss_flux_ave, filename, x_ave, coeff, error, n_pix, plot_pix, NO_DISP=no_display, $
            cal_on_idx[0], cal_off_idx[0]-1
        filename = file_header + '_cal_1' + file_footer
        ss_flux_ave, filename, x_ave, coeff, error, n_pix, plot_pix, NO_DISP=no_display, $
            cal_on_idx[3], cal_off_idx[3]-1
        filename = file_header + '_cal_2' + file_footer
        ss_flux_ave, filename, x_ave, coeff, error, n_pix, plot_pix, NO_DISP=no_display, $
            cal_on_idx[4], cal_off_idx[4]-1
        filename = file_header + '_cal_3' + file_footer
        ss_flux_ave, filename, x_ave, coeff, error, n_pix, plot_pix, NO_DISP=no_display, $
            cal_on_idx[5], cal_off_idx[5]-1
        filename = file_header + '_cal_4' + file_footer
        ss_flux_ave, filename, x_ave, coeff, error, n_pix, plot_pix, NO_DISP=no_display, $
            cal_on_idx[6], cal_off_idx[6]-1
        filename = file_header + '_cal_5' + file_footer
        ss_flux_ave, filename, x_ave, coeff, error, n_pix, plot_pix, NO_DISP=no_display, $
            cal_on_idx[7], cal_off_idx[7]-1
        filename = file_header + '_cal_6' + file_footer
        ss_flux_ave, filename, x_ave, coeff, error, n_pix, plot_pix, NO_DISP=no_display, $
            cal_on_idx[8], cal_off_idx[8]-1
    endif
    if (AOT eq 'P12_FIS01_SAA') then begin
        filename = file_header + '_cal_0' + file_footer
        ss_flux_ave, filename, x_ave, coeff, error, n_pix, plot_pix, NO_DISP=no_display, $
            cal_on_idx[3], cal_off_idx[3]-1
        filename = file_header + '_cal_1' + file_footer
        ss_flux_ave, filename, x_ave, coeff, error, n_pix, plot_pix, NO_DISP=no_display, $
            cal_on_idx[3], cal_off_idx[3]-1
        filename = file_header + '_cal_2' + file_footer
        ss_flux_ave, filename, x_ave, coeff, error, n_pix, plot_pix, NO_DISP=no_display, $
            cal_on_idx[4], cal_off_idx[4]-1
        filename = file_header + '_cal_3' + file_footer
        ss_flux_ave, filename, x_ave, coeff, error, n_pix, plot_pix, NO_DISP=no_display, $
            cal_on_idx[5], cal_off_idx[5]-1
        filename = file_header + '_cal_4' + file_footer
        ss_flux_ave, filename, x_ave, coeff, error, n_pix, plot_pix, NO_DISP=no_display, $
            cal_on_idx[6], cal_off_idx[6]-1
        filename = file_header + '_cal_5' + file_footer
        ss_flux_ave, filename, x_ave, coeff, error, n_pix, plot_pix, NO_DISP=no_display, $
            cal_on_idx[7], cal_off_idx[7]-1
        filename = file_header + '_cal_6' + file_footer
        ss_flux_ave, filename, x_ave, coeff, error, n_pix, plot_pix, NO_DISP=no_display, $
            cal_on_idx[8], cal_off_idx[8]-1
    endif
    if (AOT eq 'PV_FIS02') then begin
        filename = file_header + '_cal_0' + file_footer
        ss_flux_ave, filename, x_ave, coeff, error, n_pix, plot_pix, NO_DISP=no_display, $
            cal_on_idx[0], cal_off_idx[0]-1
        filename = file_header + '_cal_1' + file_footer
        ss_flux_ave, filename, x_ave, coeff, error, n_pix, plot_pix, NO_DISP=no_display, $
            cal_on_idx[3], cal_off_idx[3]-1
        filename = file_header + '_cal_2' + file_footer
        ss_flux_ave, filename, x_ave, coeff, error, n_pix, plot_pix, NO_DISP=no_display, $
            cal_on_idx[17], cal_off_idx[17]-1
        filename = file_header + '_cal_3' + file_footer
        ss_flux_ave, filename, x_ave, coeff, error, n_pix, plot_pix, NO_DISP=no_display, $
            cal_on_idx[24], cal_off_idx[24]-1
        filename = file_header + '_cal_4' + file_footer
        ss_flux_ave, filename, x_ave, coeff, error, n_pix, plot_pix, NO_DISP=no_display, $
            cal_on_idx[26], cal_off_idx[26]-1
    endif
    if (AOT eq 'P12_FIS02') then begin
        filename = file_header + '_cal_0' + file_footer
        ss_flux_ave, filename, x_ave, coeff, error, n_pix, plot_pix, NO_DISP=no_display, $
            cal_on_idx[0], cal_off_idx[0]-1
        filename = file_header + '_cal_1' + file_footer
        ss_flux_ave, filename, x_ave, coeff, error, n_pix, plot_pix, NO_DISP=no_display, $
            cal_on_idx[3], cal_off_idx[3]-1
        filename = file_header + '_cal_2' + file_footer
        ss_flux_ave, filename, x_ave, coeff, error, n_pix, plot_pix, NO_DISP=no_display, $
            cal_on_idx[4], cal_off_idx[4]-1
        filename = file_header + '_cal_3' + file_footer
        ss_flux_ave, filename, x_ave, coeff, error, n_pix, plot_pix, NO_DISP=no_display, $
            cal_on_idx[5], cal_off_idx[5]-1
        filename = file_header + '_cal_4' + file_footer
        ss_flux_ave, filename, x_ave, coeff, error, n_pix, plot_pix, NO_DISP=no_display, $
            cal_on_idx[6], cal_off_idx[6]-1
    endif
    if (AOT eq 'P12_FIS02_SAA') then begin
        filename = file_header + '_cal_0' + file_footer
        ss_flux_ave, filename, x_ave, coeff, error, n_pix, plot_pix, NO_DISP=no_display, $
            cal_on_idx[3], cal_off_idx[3]-1
        filename = file_header + '_cal_1' + file_footer
        ss_flux_ave, filename, x_ave, coeff, error, n_pix, plot_pix, NO_DISP=no_display, $
            cal_on_idx[3], cal_off_idx[3]-1
        filename = file_header + '_cal_2' + file_footer
        ss_flux_ave, filename, x_ave, coeff, error, n_pix, plot_pix, NO_DISP=no_display, $
            cal_on_idx[4], cal_off_idx[4]-1
        filename = file_header + '_cal_3' + file_footer
        ss_flux_ave, filename, x_ave, coeff, error, n_pix, plot_pix, NO_DISP=no_display, $
            cal_on_idx[5], cal_off_idx[5]-1
        filename = file_header + '_cal_4' + file_footer
        ss_flux_ave, filename, x_ave, coeff, error, n_pix, plot_pix, NO_DISP=no_display, $
            cal_on_idx[6], cal_off_idx[6]-1
    endif


; making Dark table

    print, '*** Making Dark table ***'
    if (AOT eq 'PV_FIS01') then begin
        filename = file_header + '_dark_0' + file_footer
        ss_flux_ave, filename, x_ave, coeff, error, n_pix, plot_pix, NO_DISP=no_display, $
            cal_off_idx[1], cal_on_idx[2]-1
        filename = file_header + '_dark_1' + file_footer
        ss_flux_ave, filename, x_ave, coeff, error, n_pix, plot_pix, NO_DISP=no_display, $
            cal_off_idx[2], cal_on_idx[3]-1
        filename = file_header + '_dark_2' + file_footer
        ss_flux_ave, filename, x_ave, coeff, error, n_pix, plot_pix, NO_DISP=no_display, $
            cal_off_idx[3], cal_on_idx[4]-1
        filename = file_header + '_dark_3' + file_footer
        ss_flux_ave, filename, x_ave, coeff, error, n_pix, plot_pix, NO_DISP=no_display, $
            cal_off_idx[4], sht_op_idx[0]-1
        filename = file_header + '_dark_4' + file_footer
        ss_flux_ave, filename, x_ave, coeff, error, n_pix, plot_pix, NO_DISP=no_display, $
            cal_off_idx[14], sht_op_idx[1]-1
        filename = file_header + '_dark_5' + file_footer
        ss_flux_ave, filename, x_ave, coeff, error, n_pix, plot_pix, NO_DISP=no_display, $
            cal_off_idx[18], sht_op_idx[2]-1
        filename = file_header + '_dark_6' + file_footer
        ss_flux_ave, filename, x_ave, coeff, error, n_pix, plot_pix, NO_DISP=no_display, $
            cal_off_idx[22], sht_op_idx[3]-1
        filename = file_header + '_dark_7' + file_footer
        ss_flux_ave, filename, x_ave, coeff, error, n_pix, plot_pix, NO_DISP=no_display, $
            cal_off_idx[25], cal_on_idx[26]-1
        filename = file_header + '_dark_8' + file_footer
        ss_flux_ave, filename, x_ave, coeff, error, n_pix, plot_pix, NO_DISP=no_display, $
            cal_off_idx[26], cal_on_idx[27]-1
        filename = file_header + '_dark_9' + file_footer
        ss_flux_ave, filename, x_ave, coeff, error, n_pix, plot_pix, NO_DISP=no_display, $
            cal_off_idx[27], cal_on_idx[28]-1
        filename = file_header + '_dark_10' + file_footer
        ss_flux_ave, filename, x_ave, coeff, error, n_pix, plot_pix, NO_DISP=no_display, $
            cal_off_idx[28], cal_off_idx[28]+fix(smpl_freq*60)
    endif
    if (AOT eq 'P12_FIS01' or AOT eq 'P12_FIS01_SAA') then begin
        filename = file_header + '_dark_0' + file_footer
        ss_flux_ave, filename, x_ave, coeff, error, n_pix, plot_pix, NO_DISP=no_display, $
            cal_off_idx[1], cal_on_idx[2]-1
        filename = file_header + '_dark_1' + file_footer
        ss_flux_ave, filename, x_ave, coeff, error, n_pix, plot_pix, NO_DISP=no_display, $
            cal_off_idx[2], cal_on_idx[3]-1
        filename = file_header + '_dark_2' + file_footer
        ss_flux_ave, filename, x_ave, coeff, error, n_pix, plot_pix, NO_DISP=no_display, $
            cal_off_idx[3], sht_op_idx[0]-1
        filename = file_header + '_dark_3' + file_footer
        ss_flux_ave, filename, x_ave, coeff, error, n_pix, plot_pix, NO_DISP=no_display, $
            sht_cls_idx[0], cal_on_idx[4]-1
        filename = file_header + '_dark_4' + file_footer
        ss_flux_ave, filename, x_ave, coeff, error, n_pix, plot_pix, NO_DISP=no_display, $
            sht_cls_idx[1], cal_on_idx[5]-1
        filename = file_header + '_dark_5' + file_footer
        ss_flux_ave, filename, x_ave, coeff, error, n_pix, plot_pix, NO_DISP=no_display, $
            sht_cls_idx[2], cal_on_idx[6]-1
        filename = file_header + '_dark_6' + file_footer
        ss_flux_ave, filename, x_ave, coeff, error, n_pix, plot_pix, NO_DISP=no_display, $
            sht_cls_idx[3], cal_on_idx[7]-1
        filename = file_header + '_dark_7' + file_footer
        ss_flux_ave, filename, x_ave, coeff, error, n_pix, plot_pix, NO_DISP=no_display, $
            cal_off_idx[7], cal_off_idx[7]+fix(smpl_freq*120)
    endif
    if (AOT eq 'PV_FIS02') then begin
        filename = file_header + '_dark_0' + file_footer
        ss_flux_ave, filename, x_ave, coeff, error, n_pix, plot_pix, NO_DISP=no_display, $
            cal_off_idx[1], cal_on_idx[2]-1
        filename = file_header + '_dark_1' + file_footer
        ss_flux_ave, filename, x_ave, coeff, error, n_pix, plot_pix, NO_DISP=no_display, $
            cal_off_idx[2], cal_on_idx[3]-1
        filename = file_header + '_dark_2' + file_footer
        ss_flux_ave, filename, x_ave, coeff, error, n_pix, plot_pix, NO_DISP=no_display, $
            cal_off_idx[3], cal_on_idx[4]-1
        filename = file_header + '_dark_3' + file_footer
        ss_flux_ave, filename, x_ave, coeff, error, n_pix, plot_pix, NO_DISP=no_display, $
            cal_off_idx[4], sht_op_idx[0]-1
        filename = file_header + '_dark_4' + file_footer
        ss_flux_ave, filename, x_ave, coeff, error, n_pix, plot_pix, NO_DISP=no_display, $
            cal_off_idx[17], sht_op_idx[1]-1
        filename = file_header + '_dark_5' + file_footer
        ss_flux_ave, filename, x_ave, coeff, error, n_pix, plot_pix, NO_DISP=no_display, $
            cal_off_idx[22], cal_on_idx[23]-1
        filename = file_header + '_dark_6' + file_footer
        ss_flux_ave, filename, x_ave, coeff, error, n_pix, plot_pix, NO_DISP=no_display, $
            cal_off_idx[23], cal_on_idx[24]-1
        filename = file_header + '_dark_7' + file_footer
        ss_flux_ave, filename, x_ave, coeff, error, n_pix, plot_pix, NO_DISP=no_display, $
            cal_off_idx[24], cal_on_idx[25]-1
        filename = file_header + '_dark_8' + file_footer
        ss_flux_ave, filename, x_ave, coeff, error, n_pix, plot_pix, NO_DISP=no_display, $
            cal_off_idx[25], cal_off_idx[25]+fix(smpl_freq*60)
    endif
    if (AOT eq 'P12_FIS02' or AOT eq 'P12_FIS02_SAA') then begin
        filename = file_header + '_dark_0' + file_footer
        ss_flux_ave, filename, x_ave, coeff, error, n_pix, plot_pix, NO_DISP=no_display, $
            cal_off_idx[1], cal_on_idx[2]-1
        filename = file_header + '_dark_1' + file_footer
        ss_flux_ave, filename, x_ave, coeff, error, n_pix, plot_pix, NO_DISP=no_display, $
            cal_off_idx[2], cal_on_idx[3]-1
        filename = file_header + '_dark_2' + file_footer
        ss_flux_ave, filename, x_ave, coeff, error, n_pix, plot_pix, NO_DISP=no_display, $
            cal_off_idx[3], sht_op_idx[0]-1
        filename = file_header + '_dark_3' + file_footer
        ss_flux_ave, filename, x_ave, coeff, error, n_pix, plot_pix, NO_DISP=no_display, $
            sht_cls_idx[0], cal_on_idx[4]-1
        filename = file_header + '_dark_4' + file_footer
        ss_flux_ave, filename, x_ave, coeff, error, n_pix, plot_pix, NO_DISP=no_display, $
            sht_cls_idx[1], cal_on_idx[5]-1
        filename = file_header + '_dark_5' + file_footer
        ss_flux_ave, filename, x_ave, coeff, error, n_pix, plot_pix, NO_DISP=no_display, $
            cal_off_idx[5], cal_off_idx[5]+fix(smpl_freq*120)
    endif


; making Local Flat table

    print, '*** Making Local Flat table ***'

    ; Set the default value
    if (n_elements(t_flat_start) eq 0) then begin
    	if (AOT eq 'PV_FIS01' or AOT eq 'PV_FIS02') then t_flat_start=420+120
    	if (AOT eq 'P12_FIS01' or AOT eq 'P12_FIS02') then t_flat_start=420+120
    	if (AOT eq 'P12_FIS01_SAA' or AOT eq 'P12_FIS02_SAA') then t_flat_start=420+120
    endif
    if (n_elements(t_flat_end) eq 0) then begin
    	if (AOT eq 'PV_FIS01' or AOT eq 'PV_FIS02') then t_flat_end=420+330
    	if (AOT eq 'P12_FIS01' or AOT eq 'P12_FIS02') then t_flat_end=420+210
    	if (AOT eq 'P12_FIS01_SAA' or AOT eq 'P12_FIS02_SAA') then t_flat_end=420+210
    endif

    if (AOT eq 'PV_FIS01') then begin
	if (t_flat_start lt 421) then begin
		t_flat_start = 421
		print, 'Start time for flat data should be after shutter open!!!'
	endif
	print, 'Start time for flat data: ', t_flat_start
	if (t_flat_end gt 902) then begin
		t_flat_end = 902
		print, 'End time for flat data should be before shutter open at the turn!!!'
	endif
	print, 'End time for flat data: ', t_flat_end
    endif
    if (AOT eq 'PV_FIS02') then begin
	if (t_flat_start lt 421) then begin
		t_flat_start = 421
		print, 'Start time for flat data should be after shutter open!!!'
	endif
	print, 'Start time for flat data: ', t_flat_start
	if (t_flat_end gt 1052) then begin
		t_flat_end = 1052
		print, 'End time for flat data should be before shutter open at the turn!!!'
	endif
	print, 'End time for flat data: ', t_flat_end
    endif
    if (AOT eq 'P12_FIS01') then begin
	if (t_flat_start lt 421) then begin
		t_flat_start = 421
		print, 'Start time for flat data should be after shutter open!!!'
	endif
	print, 'Start time for flat data: ', t_flat_start
	if (t_flat_end gt 787) then begin
		t_flat_end = 787
		print, 'End time for flat data should be before shutter open at the turn!!!'
	endif
	print, 'End time for flat data: ', t_flat_end
    endif
    if (AOT eq 'P12_FIS02') then begin
	if (t_flat_start lt 421) then begin
		t_flat_start = 421
		print, 'Start time for flat data should be after shutter open!!!'
	endif
	print, 'Start time for flat data: ', t_flat_start
	if (t_flat_end gt 975) then begin
		t_flat_end = 975
		print, 'End time for flat data should be before shutter open at the turn!!!'
	endif
	print, 'End time for flat data: ', t_flat_end
    endif
    if (AOT eq 'P12_FIS01_SAA') then begin
	if (t_flat_start lt 511) then begin
		t_flat_start = 511
		print, 'Start time for flat data should be after shutter open!!!'
	endif
	print, 'Start time for flat data: ', t_flat_start
	if (t_flat_end gt 787) then begin
		t_flat_end = 787
		print, 'End time for flat data should be before shutter open at the turn!!!'
	endif
	print, 'End time for flat data: ', t_flat_end
    endif
    if (AOT eq 'P12_FIS02_SAA') then begin
	if (t_flat_start lt 511) then begin
		t_flat_start = 511
		print, 'Start time for flat data should be after shutter open!!!'
	endif
	print, 'Start time for flat data: ', t_flat_start
	if (t_flat_end gt 975) then begin
		t_flat_end = 975
		print, 'End time for flat data should be before shutter open at the turn!!!'
	endif
	print, 'End time for flat data: ', t_flat_end
    endif

    if (AOT eq 'PV_FIS01' or AOT eq 'PV_FIS02') then begin
        filename = file_header + '_flat' + file_footer
        ss_flux_ave, filename, x_ave, coeff, error, n_pix, plot_pix, NO_DISP=no_display, $
		sht_op_idx[0]+fix(smpl_freq*(t_flat_start-420)), $
		sht_op_idx[0]+fix(smpl_freq*(t_flat_end-420))
    endif
    if (AOT eq 'P12_FIS01' or AOT eq 'P12_FIS02') then begin
        filename = file_header + '_flat' + file_footer
        ss_flux_ave, filename, x_ave, coeff, error, n_pix, plot_pix, NO_DISP=no_display, $
		sht_op_idx[0]+fix(smpl_freq*(t_flat_start-420)), $
		sht_op_idx[0]+fix(smpl_freq*(t_flat_end-420))
    endif
    if (AOT eq 'P12_FIS01_SAA' or AOT eq 'P12_FIS02_SAA') then begin
        filename = file_header + '_flat' + file_footer
        ss_flux_ave, filename, x_ave, coeff, error, n_pix, plot_pix, NO_DISP=no_display, $
		sht_op_idx[0]+fix(smpl_freq*(t_flat_start-510)), $
		sht_op_idx[0]+fix(smpl_freq*(t_flat_end-510))
    endif


; making CAL after effect table

    if (AOT eq 'PV_FIS01' or AOT eq 'PV_FIS02') then begin
    print, '*** Making AFT-CAL effect table ***'
        filename = file_header + '_aftcal' + file_footer
        ss_aftcal_ave, filename, x_ave, coeff, error, cal_flg_tot, n_int, n_pix, plot_pix, $
            NO_DISP=no_display, $
            sht_op_idx[0]+fix(smpl_freq*30), sht_op_idx[0]+fix(smpl_freq*330)
    endif



;;; output linear-fitting results ;;;

; no mask for shutter and cal-lamp

    x = x_ave
    n_line = n_elements(x)

    det = dblarr(n_pix,n_line)
    err = dblarr(n_pix,n_line)

    det[*,*] = coeff[*,*,1]
    err[*,*] = error[*,*,1]


; calculate pixel position in RADEC and draw image of each array column

        root = GETENV('FISDR_ROOT')
 
        if(data_type eq 'FIS_SW') then begin
                pix_pos_file = root+'/slowscan/lib/FIS-SW_pix_pos_distort1.dat'
        endif else begin
                pix_pos_file = root+'/slowscan/lib/FIS-LW_pix_pos_distort1.dat'
        endelse

    ss_q_to_radec, q_mean, data_type, pix_pos_file, ra, dec


; make filename
    dat_file = '_fit_pr.txt'
    filename = file_header + dat_file
    print, 'Output text data:', filename

; write time_series output TXT file
    openw, unit0, filename, /get_lun

    if(data_type eq 'FIS_SW') then begin
        for j=0L, n_line-1L do begin
           printf,unit0,x[j],ra[*,j],dec[*,j],det[*,j],err[*,j],eg_gl_cnt[*,j], $
           format='(F16.3,2X,100(F12.7,2X),100(F12.7,2X),100(F12.7,2X),100(F12.7,2X),100(F12.7,2X))'
        endfor
    endif else begin
        for j=0L, n_line-1L do begin
           printf,unit0,x[j],ra[*,j],dec[*,j],det[*,j],err[*,j],eg_gl_cnt[*,j], $
           format='(F16.3,2X,75(F12.7,2X),75(F12.7,2X),75(F12.7,2X),75(F12.7,2X),75(F12.7,2X))'
        endfor
    endelse

    free_lun, unit0


; Save IDL data

    save, x_ave, q_mean, coeff, error, eg_gl_cnt, drv_mask_tot, cal_flg_tot, $
        data_type, AOT, RST, n_int, rcount, rst_int, $
        ramp_cor_off, glitch_rmv_off, aocu_ads, $
        filename=file_header+'_pr.sav'
    print, 'Output IDL-format data:', file_header+'_pr.sav'


    error_status = 0 ; set error status
    MESSAGE, 'FINISH.',/CONTINUE

end
