; $Id: ldsclient__define.pro,v 1.4 2007/02/28 10:27:55 cyamauch Exp $
;
; Copyright (C) 2004 Hajime BABA  All rights reserved.
;
;+
; CLASS_NAME:
;   LDSRPC_DESERIALIZER
;
; PURPOSE:
;   This class provides useful methods by witch the user can easily
;   access to the LDS.
;
; CATEGORY:
;   Data definition
;
; CALLING SEQUENCE:
;   ldc = OBJ_NEW('ldsclient')
;
; INPUTS:
;   'ldsclient' Name of object class
;
; OUTPUTS:
;   ldc         Name of object instance
;
; RESTRICTIONS:
;   None
;
; COMMENTS:
;   See chapter 'Object Basics' in 'Building IDL Applications' in detail.
;
;-

;----------------------------------------------------------------------------
;+
; CALLING SEQUENCE:
;   info = ldc->get_terminfo( MODE=mode, GB_PROCESSED=gb_processed, $
;                             FMTTYPE=fmttype, $
;                             DATE=date, AFTIME=aftime, SPAN=span )
;
; PURPOSE:
;   This method sends a request to get the registration information of
;   the LDS files and receives results from the LDS.
;
; INPUTS:
;   NONE
;
; OPTIONAL KEYWORDS:
;   MODE          Data query mode: ("Fli" or "Rel": default: "Fli")
;   GB_PROCESSED  Flag whether data is processed by GreenBox or not
;                 (0: not_processed, 1: processed.  Default: 0)
;   FMTTYPE       'FIS_SW' or 'FIS_LW'.
;   DATE          Start time of requested information.
;                 DATE should be set in UTC.
;                 (STRING FORMAT: '2006-07-10T23:40:00.000Z')
;   AFTIME        Start time of requested information.
;                 DATE is ignored when AFTIME is set.
;   SPAN          Span of request information (seconds)
;
; OUTPUTS:
;   info          Array of information structure
;                 0 is returned if failed.
;
; EXAMPLE:
;
;     IDL> ldc = obj_new('ldsclient')
;     IDL> info = ldc->get_terminfo( MODE='Fli', FMTTYPE='FIS_LW', $
;                                    DATE='20070201003000', SPAN=86400 )
;     IDL> n = ldc->print_results()
;     IDL> OBJ_DESTROY, ldc
;
;-
;----------------------------------------------------------------------------

FUNCTION ldsclient::get_terminfo, MODE=mode, GB_PROCESSED=gb_processed, $
                                  FMTTYPE=fmttype, $
                                  DATE=date, AFTIME=aftime, SPAN=span
    MESSAGE, 'Called', /INFORMATIONAL

    status = self->init_method_call()

    IF KEYWORD_SET(mode) THEN self.ldsMethodCall.mode = mode
    IF N_ELEMENTS(gb_processed) THEN self.ldsMethodCall.gb_processed = gb_processed
    IF KEYWORD_SET(fmttype) THEN self.ldsMethodCall.fmttype = fmttype
    IF KEYWORD_SET(date) THEN self.ldsMethodCall.date = date $
    ELSE self.ldsMethodCall.date = '20070101000000'
    IF N_ELEMENTS(aftime) THEN BEGIN
        self.ldsMethodCall.aftime = aftime
        self.ldsMethodCall.date = ''
    ENDIF
    IF N_ELEMENTS(span) THEN self.ldsMethodCall.span = span $
    ELSE self.ldsMethodCall.span = 3600.0*24.0

    result_p = self->ldsrpc::call( 'LDSfront.get_terminfo', $
                                   self.ldsMethodCall )

    IF NOT PTR_VALID(result_p) THEN RETURN, 0

    RETURN, (*result_p).param_0
END

;----------------------------------------------------------------------------
;+
; CALLING SEQUENCE:
;   info = ldc->clipout_tsd( MODE=mode, GB_PROCESSED=gb_processed, $
;                            FMTTYPE=fmttype, $
;                            DATE=date, AFTIME=aftime, SPAN=span, $
;                            GZIP=gzip, FILENAME=filename, DIRNAME=dirname )
;
; PURPOSE:
;   This method sends a request for clipping out a TSD file, downloads
;   it and receives results from the LDS.
;
; INPUTS:
;   NONE
;
; OPTIONAL KEYWORDS:
;   MODE          Data query mode: ('Fli' or 'Rel': default: 'Fli')
;   GB_PROCESSED  Flag whether data is processed by GreenBox or not
;                 (0: not_processed, 1: processed.  Default: 0)
;   FMTTYPE       'FIS_SW' or 'FIS_LW'.
;   DATE          Start time of requested information.
;                 DATE should be set in UTC.
;                 (STRING FORMAT: '2006-07-10T23:40:00.000Z')
;   AFTIME        Start time of requested information.
;                 DATE is ignored when AFTIME is set.
;   SPAN          Span of request information (seconds)
;   GZIP          Set 1 when you want get a compressed (gzipped) file.
;                 (default: 0)
;   FILENAME      Name of downloaded TSD file.  The name of this file
;                 depends on the server's response, when FILENAME is
;                 not set.
;   DIRNAME       Name of the destination directory to save the downloaded
;                 file.
;
; OUTPUTS:
;   info          Array of information structure
;                 0 is returned if failed.
;
; EXAMPLE:
;
;     IDL> ldc = obj_new('ldsclient')
;     IDL> info = ldc->clipout_tsd( MODE='Fli', FMTTYPE='FIS_LW', $
;                                   DATE='20070201003000', SPAN=2010, $
;                                   GZIP=1, DIRNAME='dest_dir' )
;     IDL> n = ldc->print_results()
;     IDL> OBJ_DESTROY, ldc
;
;-
;----------------------------------------------------------------------------

FUNCTION ldsclient::clipout_tsd, MODE=mode, GB_PROCESSED=gb_processed, $
                                 FMTTYPE=fmttype, $
                                 DATE=date, AFTIME=aftime, SPAN=span, $
                                 GZIP=gzip, FILENAME=filename, DIRNAME=dirname
    MESSAGE, 'Called', /INFORMATIONAL

    status = self->init_method_call()

    IF KEYWORD_SET(mode) THEN self.ldsMethodCall.mode = mode
    IF N_ELEMENTS(gb_processed) THEN self.ldsMethodCall.gb_processed = gb_processed
    IF KEYWORD_SET(fmttype) THEN self.ldsMethodCall.fmttype = fmttype
    IF KEYWORD_SET(date) THEN self.ldsMethodCall.date = date $
    ELSE self.ldsMethodCall.date = '20070101000000'
    IF N_ELEMENTS(aftime) THEN BEGIN
        self.ldsMethodCall.aftime = aftime
        self.ldsMethodCall.date = ''
    ENDIF
    IF N_ELEMENTS(span) THEN self.ldsMethodCall.span = span $
    ELSE self.ldsMethodCall.span = 3600.0
    IF N_ELEMENTS(gzip) THEN self.ldsMethodCall.gzip = gzip

    result_p = self->ldsrpc::call( 'LDSfront.clipout_tsd',self.ldsMethodCall, $
                                   FILENAME=filename, DIRNAME=dirname )
    IF NOT PTR_VALID(result_p) THEN RETURN, 0

    RETURN, (*result_p).param_0
END

;----------------------------------------------------------------------------
;+
;
; CALLING SEQUENCE:
;   info = ldc->get_posinfo( MODE=mode, GB_PROCESSED=gb_processed, $
;                            FMTTYPE=fmttype, $
;                            PACKETID=packetid, ORIGIN=origin, $
;                            DATE=date, AFTIME=aftime, SPAN=span, $
;                            LON=lon, LAT=lat, RADIUS=radius, $
;                            SCANSPEED_MIN=scanspeed_min, $
;                            SCANSPEED_MAX=scanspeed_max, $ )
;                            [/J2000], [/B1950], [/EQUATORIAL], [/GALACTIC] )
;
; INPUTS:
;   NONE
;
; PURPOSE:
;   This method sends a request to get the positional information of
;   the TSD files and receives results from the LDS.
;
; OPTIONAL KEYWORDS:
;   MODE          Data query mode: ("Fli" or "Rel": default: "Fli")
;   GB_PROCESSED  Flag whether data is processed by GreenBox or not
;                 (0: not_processed, 1: processed.  Default: 0)
;   FMTTYPE       'FIS_SW' or 'FIS_LW'
;   PACKETID      Packetid in the FIS_OBS BTE.
;   ORIGIN        Origin of positional information. 
;                 'PR','GADS' or 'AOCU' can be set.
;   DATE          Start time of requested information.
;                 DATE should be set in UTC.
;                 (STRING FORMAT: '2006-07-10T23:40:00.000Z')
;   AFTIME        Start time of requested information.
;                 DATE is ignored when AFTIME is set.
;   SPAN          Span of request information (seconds)
;   LON           R.A. or longitude (degree)
;   LAT           Dec. or latitude (degree)
;   RADIUS        Radius of FOV (degree)
;   /J2000        Original equinox of coordinates (default)
;   /B1950        Another equinox of coordinates
;   /EQUATORIAL   Input is equatorial coordinate system(RA,DEC) (default)
;   /GALACTIC     Input is galactic coordinate system
;
; OUTPUTS:
;   info          Array of information structure
;                 0 is returned if failed.
;
; OUTPUT KEYWORDS:
;   NONE
;
; RESTRICTIONS:
;   Return value becomes 0 if both time and coordinates are specified.
;
; COMMENTS:
;   * Do you want an /ECLIPTIC option?
; 
; EXAMPLE:
;     IDL> ldc = obj_new('ldsclient')
;     IDL> info = ldc->get_posinfo( LON=268.0, LAT=66.0, RADIUS=0.5, SCANSPEED_MIN=0.05, SCANSPEED_MAX=0.07 )
;     IDL> n = ldc->print_results()
;     IDL> OBJ_DESTROY, ldc
;
;-
;----------------------------------------------------------------------------

FUNCTION ldsclient::get_posinfo, MODE=mode, GB_PROCESSED=gb_processed, $
                                 FMTTYPE=fmttype, $
                                 PACKETID=packetid, ORIGIN=origin, $
                                 DATE=date, AFTIME=aftime, SPAN=span, $
                                 LON=lon, LAT=lat, RADIUS=radius, $
                                 SCANSPEED_MIN=scanspeed_min, $
                                 SCANSPEED_MAX=scanspeed_max, $
                                 J2000=j2000, EQUATORIAL=equatorial, $
                                 B1950=b1950, GALACTIC=galactic
    MESSAGE, 'Called', /INFORMATIONAL

    status = self->init_method_call()

    IF KEYWORD_SET(mode) THEN self.ldsMethodCall.mode = mode
    IF N_ELEMENTS(gb_processed) THEN self.ldsMethodCall.gb_processed = gb_processed
    IF KEYWORD_SET(fmttype) THEN self.ldsMethodCall.fmttype = fmttype
    IF N_ELEMENTS(packetid) THEN self.ldsMethodCall.packetid = packetid
    IF KEYWORD_SET(origin) THEN self.ldsMethodCall.origin = origin
    IF KEYWORD_SET(date) THEN self.ldsMethodCall.date = date
    IF N_ELEMENTS(aftime) THEN self.ldsMethodCall.aftime = aftime
    IF N_ELEMENTS(span) THEN self.ldsMethodCall.span = span
    IF N_ELEMENTS(radius) THEN self.ldsMethodCall.radius = radius
    IF N_ELEMENTS(scanspeed_min) THEN self.ldsMethodCall.scanspeed_min = scanspeed_min
    IF N_ELEMENTS(scanspeed_max) THEN self.ldsMethodCall.scanspeed_max = scanspeed_max

    IF N_ELEMENTS(lon) AND N_ELEMENTS(lat) THEN BEGIN
        ; default: (ra,dec) in J2000
        ra = lon
        dec = lat
        IF KEYWORD_SET(b1950) THEN BEGIN
            ; astrolib
            JPRECESS, lon, lat, ra, dec
        ENDIF
        IF KEYWORD_SET(galactic) THEN BEGIN
            ; j=2: convert gl,gb  --> ra,dec
            GLACTC, ra, dec, 2000, lon, lat, 2
        ENDIF
        self.ldsMethodCall.ra = ra
        self.ldsMethodCall.dec = dec
    ENDIF

    result_p = self->ldsrpc::call( 'LDSfront.get_posinfo', $
                                   self.ldsMethodCall )

    IF NOT PTR_VALID(result_p) THEN RETURN, 0

    RETURN, (*result_p).param_0
END

;----------------------------------------------------------------------------
;+
;
; CALLING SEQUENCE:
;   n = ldc->print_results( RESULTS=info, MAX=max )
;
; INPUTS:
;   NONE
;
; PURPOSE:
;   This method prints the results received from the LDS.
;
; OPTIONAL KEYWORDS:
;   RESULTS       Return value of get_terminfo(), clipout_tsd() or 
;                 get_posinfo()
;   MAX           Number of rows to print results
;
; OUTPUTS:
;   n             Number of rows.
;
; OUTPUT KEYWORDS:
;   NONE
;
;-
;----------------------------------------------------------------------------

FUNCTION ldsclient::print_results, RESULTS=info, MAX=max
    MESSAGE, 'Called', /INFORMATIONAL

    IF NOT KEYWORD_SET(info) THEN BEGIN
        IF PTR_VALID(self->ldsrpc::params()) THEN BEGIN
            info = (*(self->ldsrpc::params())).param_0
        ENDIF ELSE BEGIN
            RETURN, 0
        ENDELSE
    ENDIF

    IF N_TAGS(info) GT 0 THEN BEGIN
        members = TAG_NAMES(info.(0))
        buf = ''
        FOR j=0,N_TAGS(info.(0))-1 DO BEGIN
            buf = buf + members[j] + STRING(09B)        ; TAB
        ENDFOR
        print, buf
        FOR i=0,N_TAGS(info)-1 DO BEGIN
            IF N_ELEMENTS(max) THEN BEGIN
                IF i GE max THEN BREAK
            ENDIF
            buf = ''
            FOR j=0,N_TAGS(info.(i))-1 DO BEGIN
                buf = buf + STRING(info.(i).(j)) + STRING(09B)  ; TAB
            ENDFOR
            print, buf
        ENDFOR
    ENDIF

    RETURN, N_TAGS(info)
END

FUNCTION ldsclient::set_url, url
    s = self->ldsrpc::proxy( url )
    RETURN, self
END

;----------------------------------------------------------------------------
; for private
;
FUNCTION ldsclient::init_method_call
    COMPILE_OPT hidden
    self.ldsMethodCall.mode = 'Fli'
    self.ldsMethodCall.gb_processed = 0
    self.ldsMethodCall.fmttype = 'FIS_SW'
    self.ldsMethodCall.date = ''
    self.ldsMethodCall.aftime = 0
    self.ldsMethodCall.span = 0
    self.ldsMethodCall.packetid = -1
    self.ldsMethodCall.origin = ''
    self.ldsMethodCall.ra = -999.0
    self.ldsMethodCall.dec = -999.0
    self.ldsMethodCall.radius = 0
    self.ldsMethodCall.scanspeed_min = -1.0
    self.ldsMethodCall.scanspeed_max = -1.0
    self.ldsMethodCall.gzip = 0
    RETURN, self
END

; INIT should return a scalar TRUE value if the initialization
; is successful, and FALSE if the initialization fails.
FUNCTION ldsclient::INIT, DEFAULT_URL=default_url
    COMPILE_OPT hidden
    MESSAGE, 'Called.', /INFORMATIONAL
    IF NOT KEYWORD_SET(default_url) THEN BEGIN
        default_url = 'http://www.ir.isas.jaxa.jp/AKARI/iris_data/LDSv2/cgi-bin/nph-ldsrpc.cgi'
    ENDIF
    RETURN, self->ldsrpc::INIT(DEFAULT_PROXY=default_url)
END

PRO ldsclient::CLEANUP
    COMPILE_OPT hidden
    MESSAGE, 'Called.', /INFORMATIONAL
    self->ldsrpc::CLEANUP
END

;----------------------------------------------------------------------------
;Routine to define the class structure
;
PRO ldsclient__define
    ; Not intended to be called directly by regular users
    COMPILE_OPT idl2, hidden
    ON_ERROR, 2                         ; Return to caller if an error occurs

    STRUCT_HIDE, { ldsMethodCall, $
                   mode: 'Fli', $
                   gb_processed: 0, $
                   fmttype: 'FIS_SW', $
                   date: '', $
                   aftime: 0.0, $
                   span: 0.0, $
                   packetid: -1, $
                   origin: '', $
                   ra: -999.0, $
                   dec: -999.0, $
                   radius: 0.0, $
                   scanspeed_min: -1.0, $
                   scanspeed_max: -1.0, $
                   gzip: 0 }

    STRUCT_HIDE, { ldsclient, $         ; structure name
                   INHERITS ldsrpc, $   ; inheritance
                   ldsMethodCall: {ldsMethodCall} }
                   
END

;----------------------------------------------------------------------------
; End of file
;----------------------------------------------------------------------------
